/** 
 *******************************************************************************
 * @file    main.c
 * @brief   A-ENC32 Sample Application.\n
 * @version V1.0.0
 * 
 * DO NOT USE THIS SOFTWARE WITHOUT THE SOFTWARE LICENSE AGREEMENT.
 * 
 * Copyright(C) Toshiba Electronic Device Solutions Corporation 2022
 *******************************************************************************
 */
#ifdef __cplusplus
 extern "C" {
#endif

/*------------------------------------------------------------------------------*/
/*  Includes                                                                    */
/*------------------------------------------------------------------------------*/
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include "bsp.h"
#include "sys_timer.h"
#include "bsp_led.h"
#include "bsp_timer.h"
#include "a_enc32.h"
/**
 *  @addtogroup Example
 *  @{
 */

/** 
 *  @addtogroup A-ENC32 A-ENC32 Sample Appli
 *  @{
 */
/*------------------------------------------------------------------------------*/
/*  Structure Definition                                                        */
/*------------------------------------------------------------------------------*/
/*----------------------------------*/
/** 
 * @brief Instance Information.
*/
/*----------------------------------*/
typedef struct
{
    t32a_t      t32a;                       /*!< Application :T32A.     */
    led_t       led[(uint32_t)BSP_LED_MAX]; /*!< Application :LED                */
    timer_t     timer;                      /*!< Application :Timer (1sec timer). */
    enc_t       enc;                        /*!< Application :A-ENC32.     */
} instance_t;

/*------------------------------------------------------------------------------*/
/*  Macro Function                                                              */
/*------------------------------------------------------------------------------*/
/** 
 *  @addtogroup A-ENC32_Private_macro A-ENC32 Private Macro
 *  @{
 */

/** 
 *  @defgroup LEDBlinkConfiguration LED Blink Time Configuration
 *  @brief    LED Blink Interval Time(ms).
 *  @{
 */
#define CFG_LED_BLINK_TIME      ((uint32_t)(1000))
/** 
 *  @}
 */ /* End of group LEDBlinkConfiguration */

/** 
 *  @addtogroup LEDBlinkConfiguration LED Blink Time Configuration
 *  @brief    LED Blink Off Time.
 *  @{
 */
#define CFG_LED_OFF_TIME             ((uint32_t)(50))
/** 
 *  @}
 */ /* End of group LEDBlinkConfiguration */

/** 
 *  @addtogroup CountConfiguration Time Configuration
 *  @brief    Interval Time(Interrupt Cycle).
 *  @{
 */
#define CFG_OUTPUT_COUNT_TIME       ((uint32_t)(10))
/** 
 *  @}
 */ /* End of group CountConfiguration */
#define MAIN_NULL       ((void *)0)     /*!< NULL. */


/** 
 *  @}
 */ /* End of group A-ENC32_Private_macro */

/*------------------------------------------------------------------------------*/
/*  Private Member                                                              */
/*------------------------------------------------------------------------------*/
/** 
 *  @addtogroup A-ENC32_Private_variables A-ENC32 Private Variables
 *  @{
 */
static instance_t instance;                       /*!< Instance.                    */
static uint32_t enccount = 0;                     /* ENC Counter           */
static uint32_t enccount_old = 0;                 /* ENC Counter old       */
static volatile uint32_t speed = 0;               /* speed                 */
static uint32_t sec_count = 0;                    /* 1sec timer            */
static volatile int32_t cw_flag = 0;              /* Direction of Rotation */
static uint32_t counter =  CFG_OUTPUT_COUNT_TIME; /* Time Counter          */
/**
 *  @}
 */ /* End of group A-ENC32_Private_variables */

/*------------------------------------------------------------------------------*/
/*  Private Function                                                            */
/*------------------------------------------------------------------------------*/
/** 
 *  @addtogroup A-ENC32_Private_fuctions A-ENC32 Private Fuctions
 *  @{
 */
static void timer_interval_handler(uint32_t id);
static void variable_initialize(void);
static TXZ_Result application_initialize(void);
static TXZ_Result driver_initialize(void);

/*--------------------------------------------------*/
/** 
  * @brief  Variable Initialize
  * @param  -
  * @return -
  * @retval -
  * @note   -
  */
/*--------------------------------------------------*/
static void variable_initialize(void)
{
    uint32_t i;

    /* Instance. */
    instance.t32a.p_instance    = MAIN_NULL;
    instance.enc.p_instance     = MAIN_NULL;
    for (i=0; i<(uint32_t)BSP_LED_MAX; i++)
    {
        instance.led[i].init.p_gpio = MAIN_NULL;
    }
    instance.timer.init.p_t32a  = MAIN_NULL;
}

/*--------------------------------------------------*/
/** 
  * @brief  Application Initialize.
  * @param  -
  * @return Result.
  * @retval TXZ_SUCCESS :Success.
  * @retval TXZ_ERROR   :Failure.
  * @note   -
  */
/*--------------------------------------------------*/
static TXZ_Result application_initialize(void)
{
   TXZ_Result result = TXZ_SUCCESS;

    /*----------------------*/
    /* LED                  */
    /*----------------------*/
    {
        uint32_t i;
        led_t *p_led = MAIN_NULL;
        for (i=0; i<(uint32_t)BSP_LED_MAX; i++)
        {
            p_led = &instance.led[i];

            p_led->init.id                  = (uint32_t)p_led;
            p_led->init.p_gpio              = bsp_get_gpio_instance_address();
            p_led->init.port.group          = bsp_get_gpio_group_led((BSPLed)(i));
            p_led->init.port.num            = bsp_get_gpio_num_led((BSPLed)(i));
            p_led->init.blink.func          = TXZ_ENABLE;
            p_led->init.blink.interval.on   = CFG_LED_BLINK_TIME;
            p_led->init.blink.interval.off  = CFG_LED_BLINK_TIME;
            p_led->init.state           = LED_STATE_OFF;
            led_initialize(p_led);
        }
    }

    /*----------------------*/
    /* A-ENC32              */
    /*----------------------*/
#if defined(TMPM4KLA) || defined(TMPM4MLA)
    instance.enc.p_instance   = TSB_EN2;
#else
    instance.enc.p_instance   = TSB_EN0;
#endif
    instance.enc.con.cmpsel   = ENTNCR_CMPSEL_INTMATCH;
    instance.enc.con.udmd     = ENTNCR_UDMD_UP;
    instance.enc.con.tovmd    = ENTNCR_TOVMD_CONTINUE;
    instance.enc.con.mcmpmd   = ENTNCR_MCMPMD_MATCH;
    instance.enc.con.decmd    = ENTNCR_DECMD_NOW;
    instance.enc.con.sdten    = ENTNCR_SDTEN_DISABLE;
    instance.enc.con.mode     = ENTNCR_MODE_ENCODE;
    instance.enc.con.p3en     = ENTNCR_P3EN_2;
    instance.enc.con.trgcapmd = ENTNCR_TRGCAPMD_CLEAR;
    instance.enc.con.zen      = ENTNCR_ZEN_DISABLE;
    instance.enc.con.enrun    = ENTNCR_ENRUN_DISABLE;
    instance.enc.con.zeact    = ENTNCR_ZEACT_HIGH;
    instance.enc.con.endev    = ENTNCR_ENDEV_1;
    enc_set_control(&instance.enc);
    instance.enc.inpcr.nct       = 0x1F;
    instance.enc.inpcr.syncnczen = ENINPCR_SYNCNCZEN_STOP;
    instance.enc.inpcr.syncsplmd = ENINPCR_SYNCSPLMD_ON;
    instance.enc.inpcr.syncsplen = ENINPCR_SYNCSPLEN_CONTINUE;
    enc_set_input(&instance.enc);
    instance.enc.intcr.mcmpie = ENINTCR_MCMPIE_DISABLE;
    instance.enc.intcr.rldie  = ENINTCR_RLDIE_DISABLE;
    instance.enc.intcr.cmpie  = ENINTCR_CMPIE_DISABLE;
    instance.enc.intcr.errie  = ENINTCR_ERRIE_DISABLE;
    instance.enc.intcr.capie  = ENINTCR_CAPIE_DISABLE;
    instance.enc.intcr.tplsie = ENINTCR_TPLSIE_DISABLE;
    enc_set_intcontrol(&instance.enc);
    instance.enc.clk = ENCLKCR_SPLCKS_0;
    enc_set_clk(&instance.enc);
    instance.enc.reload = 0xFFFFFFFF;
    enc_set_reload(&instance.enc);
    instance.enc.intcmp = 0x00000000;
    enc_set_int(&instance.enc);
    return (result);
}

/*--------------------------------------------------*/
/** 
  * @brief  Interval Time Handler.
  * @param  id        :User ID
  * @return -
  * @retval -
  * @note   for 1ms Timer.
  * @note   Call by Timer.
  */
/*--------------------------------------------------*/
static void timer_interval_handler(uint32_t id)
{
    uint32_t enc_status;

    if (counter-- == 0)
    {
        counter = CFG_OUTPUT_COUNT_TIME;
        enc_get_status(&instance.enc, &enc_status);
        if(enc_status & (0x00000001 << 13))
        {
            cw_flag = 1;
            led_turn_off(&instance.led[1]);
        }
        else
        {
            cw_flag = 2;
            led_turn_off(&instance.led[0]);
        }
        if(sec_count++ >= 100)
        {
            sec_count = 0;
            enc_get_count(&instance.enc, &enccount);
            speed = abs((int)(enccount - enccount_old));
            enccount_old = enccount;
        }
    }
}

/*--------------------------------------------------*/
/** 
  * @brief  Driver Initialize
  * @param  -
  * @return Result.
  * @retval TXZ_SUCCESS :Success
  * @retval TXZ_ERROR   :Failure
  * @note   -
  */
/*--------------------------------------------------*/
static TXZ_Result driver_initialize(void)
{
    TXZ_Result result = TXZ_SUCCESS;

    /*----------------------*/
    /* GPIO                 */
    /*----------------------*/
    /* Driver initialize has done within BSP. */
    /*----------------------*/
    /* T32A                 */
    /*----------------------*/
    {
        t32a_t *p_t32a = &instance.t32a;

        /*--- Construct  ---*/
        /* Register Allocation */
        switch (bsp_get_timer_ch(BSP_TIMER_1MS))
        {
        case 0:
            p_t32a->p_instance = TSB_T32A0;
            break;
        case 1:
            p_t32a->p_instance = TSB_T32A1;
            break;
        case 2:
            p_t32a->p_instance = TSB_T32A2;
            break;
        case 3:
            p_t32a->p_instance = TSB_T32A3;
            break;
        case 4:
            p_t32a->p_instance = TSB_T32A4;
            break;
        case 5:
            p_t32a->p_instance = TSB_T32A5;
            break;
        default:
            p_t32a->p_instance = MAIN_NULL;
            break;
        }
        /*--- Initialize ---*/
        p_t32a->init_mode.mode.halt = T32A_DBG_HALT_STOP;
        p_t32a->init_mode.mode.mode = T32A_MODE_16;
        if (p_t32a->p_instance != MAIN_NULL)
        {
            if (t32a_mode_init(p_t32a) != TXZ_SUCCESS)
            {
                result = TXZ_ERROR;
            }
        }
        else
        {
            result = TXZ_ERROR;
        }
        /*----------------------*/
        /* Timer                */
        /*----------------------*/
        /* 1ms timer */
        {
            timer_t *p_timer = &instance.timer;

            p_timer->init.id       = (uint32_t)&instance.timer;
            p_timer->init.p_t32a   = &instance.t32a;
            p_timer->init.type     = bsp_get_timer_type(BSP_TIMER_1MS);
            p_timer->init.irq      = bsp_get_timer_nvic(BSP_TIMER_1MS);
            p_timer->init.interval = 1000;
            p_timer->init.handler  = timer_interval_handler;
            timer_initialize(p_timer);
        }
    }

    return (result);
}

/**
 *  @}
 */ /* End of group A-ENC32_Private_fuctions */

/*------------------------------------------------------------------------------*/
/*  Public Function                                                             */
/*------------------------------------------------------------------------------*/
/** 
 *  @addtogroup A-ENC32_Exported_functions A-ENC32 Exported Functions
 *  @{
 */
/*--------------------------------------------------*/
/** 
  * @brief  Main Function.
  * @param  -
  * @return -
  * @retval -
  * @note   -
  */
/*--------------------------------------------------*/
int main(void)
{
    /*----------------------*/
    /* BSP initialize       */
    /*----------------------*/
    bsp_initialize();
    /* Variable initialize */
    variable_initialize();
    /* Timer driver initialization */
    if (driver_initialize() == TXZ_ERROR){
        
    }
    /* Application initialization */
    if (application_initialize() == TXZ_ERROR){
        
    }
    
    /* Timer Start*/
    timer_start(&instance.timer);

    enc_set_enable(&instance.enc);


    /*------------------------*/
    /* Loop */
    /*------------------------*/
    /* LED0 blink speed based on the rolling of mouse wheel */
    while(1)
    {
        if(speed > 0)
        {
            if(cw_flag == 1)
            {
                led_turn_on(&instance.led[0]);
                wait(speed);
                led_turn_off(&instance.led[0]);
                wait(CFG_LED_OFF_TIME);
            }
            else if(cw_flag == 2)
            {
                led_turn_on(&instance.led[1]);
                wait(speed);
                led_turn_off(&instance.led[1]);
                wait(CFG_LED_OFF_TIME);
            }
        }
        else
        {
            led_turn_off(&instance.led[0]);
            led_turn_off(&instance.led[1]);
        }
    } 
}

/*--------------------------------------------------*/
/** 
  * @brief  1ms Timer.
  * @param  timer :1ms Timer.
  * @return -
  * @retval -
  * @note   This function is called BSP.
  */
/*--------------------------------------------------*/
void irq_timer(BSPTimer timer)
{
    switch (timer)
    {
    case BSP_TIMER_1MS:
        timer_irq_handler(&instance.timer);
        break;
    default:
        /* no processing */
        break;
    }
}

#ifdef __DEBUG__
/*--------------------------------------------------*/
/** 
  * @brief  Failure Assert.
  * @note   for debug
  */
/*--------------------------------------------------*/
void assert_failed(char *file, int32_t line)
{
    while (1) {
        __NOP();
    }
}
#endif
/**
 *  @}
 */ /* End of group A-ENC32_Exported_functions */

/**
 *  @}
 */ /* End of group A-ENC32 */

/**
 *  @} 
 */ /* End of group Example */

#ifdef __cplusplus
}
#endif /* __cplusplus */
