/** 
 *******************************************************************************
 * @file    main.c
 * @brief   SIWDT Sample Application.
 * @version V1.0.0
 * 
 * DO NOT USE THIS SOFTWARE WITHOUT THE SOFTWARE LICENSE AGREEMENT.
 * 
 * Copyright(C) Toshiba Electronic Device Solutions Corporation 2022
 *******************************************************************************
 */
#ifdef __cplusplus
 extern "C" {
#endif

/*------------------------------------------------------------------------------*/
/*  Includes                                                                    */
/*------------------------------------------------------------------------------*/
#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include "bsp.h"
#include "bsp_led.h"
#include "bsp_timer.h"
#include "siwdt.h"
#include "sys_timer.h"

/**
 *  @addtogroup Example
 *  @{
 */

/** 
 *  @defgroup SIWDT_Sample_Appli SIWDT Sample Appli
 *  @{
 */
/*------------------------------------------------------------------------------*/
/*  Configuration                                                               */
/*------------------------------------------------------------------------------*/
#define MAIN_NULL       ((void *)0)     /*!< NULL. */
/*------------------------------------------------------------------------------*/
/*  Macro Function                                                              */
/*------------------------------------------------------------------------------*/
/** 
 *  @defgroup SIWDT_Private_macro SIWDT Private Macro
 *  @{
 */

//#define DEMO_WDT_CLEAR  /* When you comment out this line,  NMI interrupt is occurred by WDT */ 

/** 
 *  @defgroup WDTClockConfiguration WDT Clock Configuration
 *  @brief    WDT Clock.
 *  @{
 */
#define CFG_WDT_CLOCK             (SIWDxMOD_WDCLS_FSYS)
/** 
 *  @}
 */ /* End of group WDTClockConfiguration */

/** 
 *  @defgroup WDTTimeConfiguration WDT Time Configuration
 *  @brief    WDT Time.
 *  @{
 */
#define CFG_WDT_TIME             (SIWDxMOD_WDTP_2_25)
/** 
 *  @}
 */ /* End of group WDTTimeConfiguration */

/** 
 *  @defgroup LEDBlinkConfiguration LED Blink Time Configuration
 *  @brief    LED Blink Interval Time(Interrupt Cycle).
 *  @{
 */
#define CFG_LED_BLINK_TIME             ((uint32_t)(200))
/** 
 *  @}
 */ /* End of group LEDBlinkConfiguration */

/** 
 *  @}
 */ /* End of group SIWDT_Private_macro */

/*------------------------------------------------------------------------------*/
/*  Enumerated Type Definition                                                  */
/*------------------------------------------------------------------------------*/
/** 
 *  @defgroup SIWDT_Private_define SIWDT Private Define
 *  @{
 */

/** 
 *  @}
 */ /* End of group SIWDT_Private_define */

/*------------------------------------------------------------------------------*/
/*  Structure Definition                                                        */
/*------------------------------------------------------------------------------*/
/** 
 *  @defgroup SIWDT_Private_typedef SIWDT Private Typedef
 *  @{
 */
 
typedef struct
{
    t32a_t      t32a;                       /*!< Driver      :T32A.     */
    led_t       led[(uint32_t)BSP_LED_MAX]; /*!< Application :LED                */
    timer_t     timer;                      /*!< Application :Timer (1ms timer). */
} instance_t;

/** 
 *  @}
 */ /* End of group SIWDT_Private_typedef */
/*------------------------------------------------------------------------------*/
/*  Private Member                                                              */
/*------------------------------------------------------------------------------*/
/** 
 *  @defgroup SIWDT_Private_variables SIWDT Private Variables
 *  @{
 */
static instance_t instance; /*!< Instance.                    */
static uint32_t   count = 0;
static uint32_t   led_flag = 0;
static volatile uint32_t   wdt_flag = 0;
/**
 *  @}
 */ /* End of group SIWDT_Private_variables */

/*------------------------------------------------------------------------------*/
/*  Private Function                                                            */
/*------------------------------------------------------------------------------*/
/** 
 *  @defgroup SIWDT_Private_functions SIWDT Private Functions
 *  @{
 */

/**
 *  @}
 */ /* End of group SIWDT_Private_functions */

/*------------------------------------------------------------------------------*/
/*  Private Function                                                            */
/*------------------------------------------------------------------------------*/
static void variable_initialize(void);
static TXZ_Result driver_initialize(void);
static TXZ_Result application_initialize(void);
static void timer_interval_handler(uint32_t id);

/** 
 *  @defgroup SIWDT_Exported_functions SIWDT Exported Functions
 *  @{
 */

/*--------------------------------------------------*/
/** 
  * @brief  Variable Initialize
  * @param  -
  * @return -
  * @retval -
  * @note   -
  */
/*--------------------------------------------------*/
static void variable_initialize(void)
{
    /* Instance. */
    instance.t32a.p_instance    = MAIN_NULL;
    instance.timer.init.p_t32a  = MAIN_NULL;
    {
        uint32_t i;
        /* Instance. */
        for (i=0; i<(uint32_t)BSP_LED_MAX; i++)
        {
            instance.led[i].init.p_gpio = MAIN_NULL;
        }
    }
}

/*--------------------------------------------------*/
/** 
  * @brief  Driver Initialize
  * @param  -
  * @retval TXZ_SUCCESS :Success
  * @retval TXZ_ERROR   :Failure
  * @note   -
  */
/*--------------------------------------------------*/
static TXZ_Result driver_initialize(void)
{
    TXZ_Result result = TXZ_SUCCESS;

    /*----------------------*/
    /* GPIO                 */
    /*----------------------*/
    /* Driver initialize has done within BSP. */
    /*----------------------*/
    /* T32A                 */
    /*----------------------*/
    {
        t32a_t *p_t32a = &instance.t32a;

        /*--- Construct  ---*/
        /* Register Allocation */
        switch (bsp_get_timer_ch(BSP_TIMER_1MS))
        {
        case 0:
            p_t32a->p_instance = TSB_T32A0;
            break;
        case 1:
            p_t32a->p_instance = TSB_T32A1;
            break;
        case 2:
            p_t32a->p_instance = TSB_T32A2;
            break;
        case 3:
            p_t32a->p_instance = TSB_T32A3;
            break;
        case 4:
            p_t32a->p_instance = TSB_T32A4;
            break;
        case 5:
            p_t32a->p_instance = TSB_T32A5;
            break;
        default:
            p_t32a->p_instance = MAIN_NULL;
            break;
        }
        /*--- Initialize ---*/
        p_t32a->init_mode.mode.halt = T32A_DBG_HALT_STOP;
        p_t32a->init_mode.mode.mode = T32A_MODE_16;
        if (p_t32a->p_instance != MAIN_NULL)
        {
            if (t32a_mode_init(p_t32a) != TXZ_SUCCESS)
            {
                result = TXZ_ERROR;
            }
        }
        else
        {
            result = TXZ_ERROR;
        }
    }
    return (result);
}

/*--------------------------------------------------*/
/** 
  * @brief  Application Initialize.
  * @param  -
  * @return Result.
  * @retval TXZ_SUCCESS :Success.
  * @retval TXZ_ERROR   :Failure.
  * @note   -
  */
/*--------------------------------------------------*/
static TXZ_Result application_initialize(void)
{
   TXZ_Result result = TXZ_SUCCESS;
    /*----------------------*/
    /* LED                  */
    /*----------------------*/
    {
        uint32_t i;
        led_t *p_led = MAIN_NULL;

        for (i=0; i<(uint32_t)BSP_LED_MAX; i++)
        {
            p_led = &instance.led[i];

            p_led->init.id                  = (uint32_t)p_led;
            p_led->init.p_gpio              = bsp_get_gpio_instance_address();
            p_led->init.port.group          = bsp_get_gpio_group_led((BSPLed)(i));
            p_led->init.port.num            = bsp_get_gpio_num_led((BSPLed)(i));
            p_led->init.blink.func          = TXZ_DISABLE;
            p_led->init.blink.interval.on   = 0;
            p_led->init.blink.interval.off  = 0;
            p_led->init.state           = LED_STATE_OFF;
            led_initialize(p_led);
        }
    }

    /*----------------------*/
    /* Timer                */
    /*----------------------*/
    /* 1ms timer */
    {
        timer_t *p_timer = &instance.timer;

        p_timer->init.id       = (uint32_t)&instance.timer;
        p_timer->init.p_t32a   = &instance.t32a;
        p_timer->init.type     = bsp_get_timer_type(BSP_TIMER_1MS);
        p_timer->init.irq      = bsp_get_timer_nvic(BSP_TIMER_1MS);
        p_timer->init.interval = 1000;
        p_timer->init.handler  = timer_interval_handler;
        timer_initialize(p_timer);
    }


    return (result);
}

/*--------------------------------------------------*/
/** 
  * @brief  Main Function.
  * @param  -
  * @return -
  * @retval -
  * @note   -
  */
/*--------------------------------------------------*/
int main(void)
{
    /*----------------------*/
    /* BSP initialize       */
    /*----------------------*/
    bsp_initialize();
    /*----------------------*/
    /* Initialize           */
    /*----------------------*/
    /* Variable initialize */
    variable_initialize();
    /*--- driver      ---*/
    if (driver_initialize() != TXZ_SUCCESS)
    {
    }

    /*--- application ---*/
    if (application_initialize() != TXZ_SUCCESS)
    {
    }

    /*----------------------*/
    /* WDT Disable          */
    /*----------------------*/
    wdt_disable(TSB_SIWD0);
    wdt_wdcls_set(TSB_SIWD0, CFG_WDT_CLOCK);
    wdt_wdtp_set(TSB_SIWD0, CFG_WDT_TIME);
    wdt_rescr_set(TSB_SIWD0, SIWDxMOD_RESCR_INT);
    /*----------------------*/
    /* WDT Enable          */
    /*----------------------*/   
    wdt_enable(TSB_SIWD0);
    
    timer_start(&instance.timer);

    while (1) {
#ifdef DEMO_WDT_CLEAR
    wait(1000U);
    wdt_clear(TSB_SIWD0);
#endif
    }
}

/*--------------------------------------------------*/
/** 
  * @brief  Interval Time Handler.
  * @param  id        :User ID
  * @return -
  * @retval -
  * @note   for 1ms Timer.
  * @note   Call by Timer.
  */
/*--------------------------------------------------*/
static void timer_interval_handler(uint32_t id)
{
    if (id == (uint32_t)&instance.timer)
    {
        /* Increment Tick. Tick is used to driver's Timeout. */
        hal_inc_tick();
        /* Interval Time Count. */
        count++;
        if (count >= CFG_LED_BLINK_TIME)
        {
            if (led_flag && (!wdt_flag))
            {
                led_turn_on(&instance.led[0]);
                led_turn_on(&instance.led[1]);
                led_turn_on(&instance.led[2]);
                led_turn_on(&instance.led[3]);
            } else {
                led_turn_off(&instance.led[0]);
                led_turn_off(&instance.led[1]);
                led_turn_off(&instance.led[2]);
                led_turn_off(&instance.led[3]);
            }
            led_flag = led_flag ^ 0x1;
            count = 0;
        }
    }
}

/*--------------------------------------------------*/
/** 
  * @brief  1ms Timer.
  * @param  timer :1ms Timer.
  * @return -
  * @retval -
  * @note   This function is called BSP.
  */
/*--------------------------------------------------*/
void irq_timer(BSPTimer timer)
{
    switch (timer)
    {
    case BSP_TIMER_1MS:
        timer_irq_handler(&instance.timer);
        break;
    default:
        /* no processing */
        break;
    }
}

/*--------------------------------------------------*/
/** 
  * @brief  NMI Handler
  * @param  -
  * @return -
  * @retval -
  */
/*--------------------------------------------------*/
void NMI_Handler(void)
{
    except_t except;
    except.p_instance2 = TSB_IB;
    except.p_instance3 = TSB_IMN;
    if(imn_get_flag(&except, 16) == INTERRUPT_DETECT)
    {
        wdt_disable(TSB_SIWD0);
        /*--- IBNIC00: INTPCLR ---*/
        except_clear_intp(&except, INT_REGISTER_B, 0, INT_NIC);
        while(except_get_intp(&except, INT_REGISTER_B, 0, INT_NIC) == INTERRUPT_DETECT)
        {
            ;
        }
        /* LED turn off */
        led_turn_off(&instance.led[0]);
        led_turn_off(&instance.led[1]);
        led_turn_off(&instance.led[2]);
        led_turn_off(&instance.led[3]);
        wdt_flag = 1;
    }
}

#ifdef __DEBUG__
/*--------------------------------------------------*/
/** 
  * @brief  Failure Assert.
  * @note   for debug
  */
/*--------------------------------------------------*/
void assert_failed(char *file, int32_t line)
{
    while (1) {
        __NOP();
    }
}
#endif
/**
 *  @}
 */ /* End of group SIWDT_Exported_functions */

/**
 *  @}
 */ /* End of group SIWDT_Sample_Appli */

/**
 *  @} 
 */ /* End of group Example */

#ifdef __cplusplus
}
#endif /* __cplusplus */
