/** 
 *******************************************************************************
 * @file    main.c
 * @brief   T32A MEASURE Sample Application.
 * @version V1.0.0
 * 
 * DO NOT USE THIS SOFTWARE WITHOUT THE SOFTWARE LICENSE AGREEMENT.
 * 
 * Copyright(C) Toshiba Electronic Device Solutions Corporation 2023
 *******************************************************************************
 */
#ifdef __cplusplus
 extern "C" {
#endif
/**
 *  @addtogroup Example
 *  @{
 */

/** 
 *  @defgroup T32A_PPG T32A_PPG Sample Appli
 *  @{
 */

/** 
 *  @defgroup MAIN_PRIVATE  MAIN Private
 *  @brief    MAIN Private Group.
 *  @{
 */

/*------------------------------------------------------------------------------*/
/*  Includes                                                                    */
/*------------------------------------------------------------------------------*/
#include <stdio.h>
#include "bsp.h"
#include "bsp_sw.h"
#include "bsp_timer.h"
#include "bsp_uart_io.h"
/*------------------------------------------------------------------------------*/
/*  Enumerated Type Definition                                                  */
/*------------------------------------------------------------------------------*/

/*------------------------------------------------------------------------------*/
/*  Structure Definition                                                        */
/*------------------------------------------------------------------------------*/

/*------------------------------------------------------------------------------*/
/*  Private Member                                                              */
/*------------------------------------------------------------------------------*/
/** 
 *  @defgroup Timer_EVENT_COUNT_Private_variables Timer_EVENT_COUNT Private Variables
 *  @{
 */
static t32a_t   timer_PPG;                /*!< Instance       :Timer Driver  */
static t32a_t   timer_Measure;               /*!< Instance       :Timer Driver  */
/**
 *  @}
 */ /* End of group Timer_EVENT_COUNT_Private_variables */
uint8_t count = 0U;
volatile uint8_t update_flag = 0U; 
uint32_t pre_value = 0U;
uint32_t now_value = 0U;
uint32_t diff_value = 0U; 
uint32_t freq_value = 0U; 
/**
 *  @}
 */ /* End of group T32A_MEASURE_Private_variables */
/*------------------------------------------------------------------------------*/
/*  Configuration                                                               */
/*------------------------------------------------------------------------------*/
/*----------------------*/
/*  UART Setting        */
/*----------------------*/
/** 
 *  @defgroup T32A_Setting T32A Setting
 *  @{
 */
#define CFG_T32A_MEASURE_RGx             ((uint32_t)40000)  /*!< Measure cycle((1 / (T0m / 512)) * 40000 = 256ms) */
#define CFG_T32A_MEASURE_COUNT           ((uint32_t)2)      /*!< Measure cycle(256 * 2 = 512ms) */
/** 
 *  @}
 */ /* End of group T32A_Setting */
/** 
 *  @}
 */ /* End of group Timer_EVENT_COUNT_Private_define */
#define MAIN_NULL       ((void *)0)     /*!< NULL. */
#define USER_ID_REMOTE  ((uint32_t)0)   /*!< User ID :Remote Control (Any value) */
#define USER_ID_UART    ((uint32_t)1)   /*!< User ID :USB UART (Any value) */

#define BSP_T32A_MEASURE                 TSB_T32A3

#define BSP_T32A_PPG                     TSB_T32A3
#define BSP_INTT32A_PPG_IRQn             INTT32A03AC_IRQn
/** 
 *  @}
 */ /* End of group SWPollingConfiguration */
/*------------------------------------------------------------------------------*/
/*  Private Function                                                            */
/*------------------------------------------------------------------------------*/
static TXZ_Result application_initialize(void);
static TXZ_Result InitT32A_PPG(void);
static TXZ_Result InitT32A_Measure(void);

/*--------------------------------------------------*/
/** 
  * @brief  Interval Time Handler.
  * @param  id        :User ID
  * @return -
  * @retval -
  * @note   for 1ms Timer.
  * @note   Call by Timer.
  */
/*--------------------------------------------------*/
/** 
  * @brief  Application Initialize.
  * @param  -
  * @return Result.
  * @retval TXZ_SUCCESS :Success.
  * @retval TXZ_ERROR   :Failure.
  * @note   -
  */
/*--------------------------------------------------*/
static TXZ_Result application_initialize(void)
{
   TXZ_Result result = TXZ_SUCCESS;

    /*----------------------*/
    /* Usb Uart             */
    /*----------------------*/
    /* Tx & Rx use same UART channel. */
    {
        uart_io_initial_setting_t param;

        param.id         = USER_ID_UART;
        switch (bsp_get_usb_uart_tx_ch(BSP_USB_UART_0))
        {
        case 0:
            param.p_reg  = TSB_UART0;
            break;
        case 1:
            param.p_reg  = TSB_UART1;
            break;
        case 2:
            param.p_reg  = TSB_UART2;
            break;
        default:
            param.p_reg  = MAIN_NULL;
            break;
        }
        param.boudrate   = bsp_get_usb_uart_tx_baudrate(BSP_USB_UART_0);
        param.irq.tx     = bsp_get_usb_uart_tx_nvic(BSP_USB_UART_0);
        param.irq.rx     = bsp_get_usb_uart_rx_nvic(BSP_USB_UART_0);
        param.irq.err    = bsp_get_usb_uart_err_nvic(BSP_USB_UART_0);
        if (param.p_reg != MAIN_NULL)
        {
            result  = uart_io_initialize(&param);
        }
        else
        {
            result = TXZ_ERROR;
        }
    }
    return (result);
}

/*--------------------------------------------------*/
/** 
  * @brief  Initialize T32A Signal Output Function
  * @param  -
  * @retval -
  * @note   -
  */
/*--------------------------------------------------*/
static TXZ_Result InitT32A_PPG(void)
{
    TXZ_Result result = TXZ_SUCCESS;

    /*------------------------------*/
    /*  Signal Output Timer         */
    /*------------------------------*/
    /*--- Construct (Register Allocation) ---*/
    /* Register Allocation */
    timer_PPG.p_instance = BSP_T32A_PPG;
    /*--- Initialize: Timer   ---*/
    /* Mode             */
    timer_PPG.init_mode.mode.halt = T32A_DBG_HALT_RUN;
    timer_PPG.init_mode.mode.mode = T32A_MODE_16;
    if (t32a_mode_init(&timer_PPG) != TXZ_SUCCESS)
    {
        result = TXZ_ERROR;
    }
    /*--- Initialize: Timer A ---*/
    {
        t32a_initial_setting_t *p_init = &timer_PPG.init[T32A_TIMERA];

        /* Run Control      */
        p_init->runx.sftstp        = T32A_COUNT_STOP;
        p_init->runx.sftsta        = T32A_COUNT_DONT_START;
        p_init->runx.run           = T32A_RUN_ENABLE;
        /* Counter Control  */
        p_init->crx.prscl          = T32A_PRSCLx_512;
        p_init->crx.clk            = T32A_CLKx_PRSCLx;
        p_init->crx.wbf            = T32A_WBF_DISABLE;
        p_init->crx.updn           = T32A_COUNT_UP;
        p_init->crx.reld           = T32A_RELOAD_TREGx;
        p_init->crx.stop           = T32A_STOP_NON;
        p_init->crx.start          = T32A_START_NON;
        /* Output Control 0 */
        p_init->outcrx0.ocr        = T32A_OCR_DISABLE;
        /* Output Control 1 */
        p_init->outcrx1.ocrcap1    = T32A_OCRCAPx1_DISABLE;
        p_init->outcrx1.ocrcap0    = T32A_OCRCAPx0_DISABLE;
        p_init->outcrx1.ocrcmp1    = T32A_OCRCMPx1_INVERSION;
        p_init->outcrx1.ocrcmp0    = T32A_OCRCMPx0_DISABLE;
        /* Timer Counter 1  */
        p_init->rgx0.rgx0          = 0;
        p_init->rgx1.rgx1          = CFG_T32A_MEASURE_RGx;
        /* Reload  */
        p_init->reldx.reld         = 0;
        /* Counter Capture  */
        p_init->tmrx.tmrx          = 0;
        /* Counter Capture Control A0 */
        p_init->capx0.capx0        = 0;
        /* Counter Capture Control A1 */
        p_init->capx1.capx1        = 0;
        /* Interrupt mask register Set */
        p_init->imx.imuf           = T32A_IMUFx_MASK_REQ;
        p_init->imx.imof           = T32A_IMOFx_MASK_REQ;
        p_init->imx.imx1           = T32A_IMx1_MASK_NOREQ;
        p_init->imx.imx0           = T32A_IMx0_MASK_REQ;
        /* DMA Request      */
        p_init->dma_req.dmaenx2    = T32A_DMAENx2_DISABLE;
        p_init->dma_req.dmaenx1    = T32A_DMAENx1_DISABLE;
        p_init->dma_req.dmaenx0    = T32A_DMAENx0_DISABLE;
        /* User ID          */
        p_init->id                 = 0;
        /* Handler          */
        p_init->handler_T   = T32A_NULL;
        p_init->handler_TC0 = T32A_NULL;
        p_init->handler_TC1 = T32A_NULL;
        if (t32a_timer_init(&timer_PPG, T32A_TIMERA) != TXZ_SUCCESS)
        {
            result = TXZ_ERROR;
        }
        t32a_SWcounter_start(&timer_PPG, T32A_TIMERA);
    }
    return(result);
}

/*--------------------------------------------------*/
/** 
  * @brief  Initialize T32A Reference Signal Input Function
  * @param  -
  * @retval -
  * @note   -
  */
/*--------------------------------------------------*/
static TXZ_Result InitT32A_Measure(void)
{
    TXZ_Result result = TXZ_SUCCESS;

    /*-------------------------------*/
    /*  Reference Signal Input Timer */
    /*-------------------------------*/
    /*--- Construct (Register Allocation) ---*/
    /* Register Allocation */
    timer_Measure.p_instance = BSP_T32A_MEASURE;
    /*--- Initialize: Timer   ---*/
    /* Mode             */
    timer_Measure.init_mode.mode.halt = T32A_DBG_HALT_RUN;
    timer_Measure.init_mode.mode.mode = T32A_MODE_16;
    if (t32a_mode_init(&timer_Measure) != TXZ_SUCCESS)
    {
        result = TXZ_ERROR;
    }
    /*--- Initialize: Timer B ---*/
    {
        t32a_initial_setting_t *p_init = &timer_Measure.init[T32A_TIMERB];

        /* Run Control      */
        p_init->runx.sftstp        = T32A_COUNT_STOP;
        p_init->runx.sftsta        = T32A_COUNT_DONT_START;
        p_init->runx.run           = T32A_RUN_ENABLE;
        /* Counter Control  */
        p_init->crx.prscl          = T32A_PRSCLx_1;
        p_init->crx.clk            = T32A_CLKx_TIM_RISING_EDGE;
        p_init->crx.wbf            = T32A_WBF_DISABLE;
        p_init->crx.updn           = T32A_COUNT_UP;
        p_init->crx.reld           = T32A_RELOAD_NON;
        p_init->crx.stop           = T32A_STOP_NON;
        p_init->crx.start          = T32A_START_NON;
        /* Output Control 0 */
        p_init->outcrx0.ocr        = T32A_OCR_DISABLE;
        /* Output Control 1 */
        p_init->outcrx1.ocrcap1    = T32A_OCRCAPx1_DISABLE;
        p_init->outcrx1.ocrcap0    = T32A_OCRCAPx0_DISABLE;
        p_init->outcrx1.ocrcmp1    = T32A_OCRCMPx1_DISABLE;
        p_init->outcrx1.ocrcmp0    = T32A_OCRCMPx0_DISABLE;
        /* Timer Counter 1  */
        p_init->rgx0.rgx0          = 0;
        p_init->rgx1.rgx1          = 0;
        /* Reload  */
        p_init->reldx.reld         = 0;
        /* Counter Capture  */
        p_init->tmrx.tmrx          = 0;
        /* Capture Control */
        p_init->capcrx.capmx1      = T32A_CAPMx1_DISABLE;
        p_init->capcrx.capmx0      = T32A_CAPMx0_INx0_RISING_EDGE;
        /* Counter Capture Control A0 */
        p_init->capx0.capx0        = 0;
        /* Counter Capture Control A1 */
        p_init->capx1.capx1        = 0;
        /* Interrupt mask register Set */
        p_init->imx.imuf           = T32A_IMUFx_MASK_REQ;
        p_init->imx.imof           = T32A_IMOFx_MASK_REQ;
        p_init->imx.imx1           = T32A_IMx1_MASK_REQ;
        p_init->imx.imx0           = T32A_IMx0_MASK_REQ;
        /* DMA Request      */
        p_init->dma_req.dmaenx2    = T32A_DMAENx2_DISABLE;
        p_init->dma_req.dmaenx1    = T32A_DMAENx1_DISABLE;
        p_init->dma_req.dmaenx0    = T32A_DMAENx0_DISABLE;
        /* User ID          */
        p_init->id                 = 0;
        /* Handler          */
        p_init->handler_T   = T32A_NULL;
        p_init->handler_TC0 = T32A_NULL;
        p_init->handler_TC1 = T32A_NULL;
    }
    if (t32a_timer_init(&timer_Measure, T32A_TIMERB) != TXZ_SUCCESS)
    {
        result = TXZ_ERROR;
    }
    t32a_SWcounter_start(&timer_Measure, T32A_TIMERB);
    return(result);
}

/**
 *  @}
 */ /* End of group MAIN_PRIVATE */

/*------------------------------------------------------------------------------*/
/*  Public Function                                                             */
/*------------------------------------------------------------------------------*/
/*--------------------------------------------------*/
/** 
  * @brief  Main Function.
  * @param  -
  * @retval -
  * @note   -
  */
/*--------------------------------------------------*/
int main(void)
{
    /*----------------------*/
    /* BSP initialize       */
    /*----------------------*/
    bsp_initialize();
    /* Application initialization */
    if(application_initialize() == TXZ_ERROR) {
        /* no processing */
    }
    if(InitT32A_PPG() == TXZ_ERROR) {
        /* no processing */
    }
    if(InitT32A_Measure() == TXZ_ERROR) {
        /* no processing */
    }
    NVIC_EnableIRQ(BSP_INTT32A_PPG_IRQn);

    printf("measuring in progress\n\n");

    /*----------------------*/
    /* Main Processing      */
    /*----------------------*/
    while(1){
        if( update_flag == 1 )
        {
            update_flag = 0;
            t32a_tmr_read(&timer_Measure, T32A_TIMERB, &now_value);
            if(now_value >= pre_value)diff_value = now_value - pre_value;
            else diff_value = now_value+ 0xFFFF - pre_value;
            pre_value = now_value;
            freq_value = diff_value * 2;
            printf("Frequency = %7dHz\n", freq_value);
        }
    }
}

/*--------------------------------------------------*/
/** 
  * @brief  USB UART reception IRQ.
  * @param  uart :USB UART.
  * @return -
  * @retval -
  * @note   This function is called BSP.
  */
/*--------------------------------------------------*/
void irq_usb_uart_rx(BSPUsbUart uart)
{
    switch (uart)
    {
    /*--- usb uart 0 ---*/
    case BSP_USB_UART_0:
        uart_io_rx_irq_handler();
        break;
    /*--- default    ---*/
    default:
        /* no processing */
        break;
    }

}

/*--------------------------------------------------*/
/** 
  * @brief  USB UART transmit IRQ.
  * @param  uart :USB UART.
  * @return -
  * @retval -
  * @note   This function is called BSP.
  */
/*--------------------------------------------------*/
void irq_usb_uart_tx(BSPUsbUart uart)
{
    switch (uart)
    {
    /*--- usb uart 0 ---*/
    case BSP_USB_UART_0:
        uart_io_tx_irq_handler();
        break;
    /*--- default    ---*/
    default:
        /* no processing */
        break;
    }
}

/*--------------------------------------------------*/
/** 
  * @brief  USB UART error IRQ.
  * @param  uart :USB UART.
  * @return -
  * @retval -
  * @note   This function is called BSP.
  */
/*--------------------------------------------------*/
void irq_usb_uart_err(BSPUsbUart uart)
{
    switch (uart)
    {
    /*--- usb uart 0 ---*/
    case BSP_USB_UART_0:
        uart_io_err_irq_handler();
        break;
    /*--- default    ---*/
    default:
        /* no processing */
        break;
    }
}

/*--------------------------------------------------*/
/** 
  * @brief  Timer.
  * @param  timer :Timer.
  * @return -
  * @retval -
  * @note   This function is called BSP.
  */
/*--------------------------------------------------*/
void irq_ppg(BSPOutputPulse ppg)
{
    uint32_t status;
    switch (ppg)
    {
    case BSP_OUT_PULSE_SAMPLE_A:
        t32a_get_status(&timer_PPG, &status, T32A_TIMERA);
        if( status & T32A_INTx1_FLG_MASK )
        {
            uint32_t value = (T32A_INTx0_FLG_CLR | T32A_INTx1_FLG_CLR);
            t32a_clear_status(&timer_PPG, value, T32A_TIMERA);
            if( ++count >= CFG_T32A_MEASURE_COUNT )
            {
                count = 0;
                update_flag = 1;
            }
        }
        break;
    default:
        /* no processing */
        break;
    }
}

#ifdef __DEBUG__
/*--------------------------------------------------*/
/** 
  * @brief  Failure Assert.
  * @note   for debug
  */
/*--------------------------------------------------*/
void assert_failed(char *file, int32_t line)
{
    while (1) {
        __NOP();
    }
}
#endif

/**
 *  @} 
 */ /* End of group T32A_PPG */

/**
 *  @} 
 */ /* End of group Example */

#ifdef __cplusplus
}
#endif /* __cplusplus */
