/** 
 *******************************************************************************
 * @file    main.c
 * @brief   TRM Sample Application.
 * @version V1.0.0
 * 
 * DO NOT USE THIS SOFTWARE WITHOUT THE SOFTWARE LICENSE AGREEMENT.
 * 
 * Copyright(C) Toshiba Electronic Device Solutions Corporation 2022
 *******************************************************************************
 */

#ifdef __cplusplus
 extern "C" {
#endif

/*------------------------------------------------------------------------------*/
/*  Includes                                                                    */
/*------------------------------------------------------------------------------*/
#include <stdio.h>
#include <stdlib.h>
#include "bsp_uart_io.h"
#include "bsp.h"
#include "trm.h"
     
/**
 *  @addtogroup Example
 *  @{
 */

/** 
 *  @defgroup TRMOSC TRMOSC Sample Appli
 *  @{
 */
/*------------------------------------------------------------------------------*/
/*  Macro Function                                                              */
/*------------------------------------------------------------------------------*/
/** 
 *  @defgroup TRMOSC_Private_macro TRMOSC Private Macro
 *  @{
 */
/* no define */
/** 
 *  @}
 */ /* End of group TRMOSC_Private_macro */


/*------------------------------------------------------------------------------*/
/*  Configuration                                                               */
/*------------------------------------------------------------------------------*/
/** 
 *  @defgroup TRMOSC_Private_define TRMOSC Private Define
 *  @{
 */
#define CAPTUREVAL10MHZ        (10416)         /*!< Caputer value @10MHz */
#define CAPTUREVAL2400bps      (21000)         /*!< Caputer value @2400bps */
#define FIXEDPOINT             (100)           /*!< fixed point 2 */
#define TRIMSTEP               (7)             /*!< trimming step 0.07% * 100 * FIXEDPOINT */
#define MARGINADJUST           (20)            /*!< Adjustment margin 0.2% * 100 * FIEXDPOINT */
#define TRIMLIMIT              (0x0000007FUL)  /*!< Setting limit */
#define TRIM_MAX               (0x00000100UL)  /*!< max value */
#define PPG_SELECT             1U
#define UART_SELECT            2U
#define HZ_1M                   (1000000UL)
/** 
 *  @}
 */ /* End of group TRMOSC_Private_define */


/*------------------------------------------------------------------------------*/
/*  Macro Definition                                                            */
/*------------------------------------------------------------------------------*/
/** 
 *  @defgroup TRMOSC_Private_define TRMOSC Private Define
 *  @{
 */
#define COUNTMAX               (10)            /*!< retry counter max value1 */

#define TRM_SELECT_TYPE    PPG_SELECT
/*#define TRM_SELECT_TYPE    UART_SELECT*/

#define TRM_INIT        (uint32_t)0x00000080

#define TRM_PPG_MIN     ((uint32_t)2)

#define CFG_PPG_CLOCK   ((uint32_t)(SystemCoreClock / HZ_1M))   /* 10MHz */
#define CFG_PPG_FRQ     ((float)(5))
#define CFG_PPG_DUTY    ((float)(0.5))

#define CFG_PPG_RGx0    ((uint32_t)((CFG_PPG_CLOCK / CFG_PPG_FRQ) * CFG_PPG_DUTY))
#define CFG_PPG_RGx1    ((uint32_t)(CFG_PPG_CLOCK / CFG_PPG_FRQ))

/*----------------------*/
/*  UART Setting        */
/*----------------------*/
#define MAIN_NULL       ((void *)0)     /*!< NULL. */
#define USER_ID_REMOTE  ((uint32_t)0)   /*!< User ID :Remote Control (Any value) */
#define USER_ID_UART    ((uint32_t)1)   /*!< User ID :USB UART (Any value) */


#define TRMOSC_RUN             (1)             /*!< run   */
#define TRMOSC_DONE            (0)             /*!< done  */
#define TRMOSC_ERROR           (-1)            /*!< error */

#define BSP_T32A_REFIN                 TSB_T32A1
#define BSP_INTT32A_REFIN_ACCAP0_IRQn  INTT32A01ACCAP0_IRQn
#define BSP_INTT32A_REFIN_ACCAP1_IRQn  INTT32A01ACCAP1_IRQn
#define BSP_T32A_REFIN_RUNA_SFTSTPA    TSB_T32A1_RUNA_SFTSTPA

#define BSP_T32A_SOUT                  TSB_T32A3
#define BSP_T32A_SOUT_RUNA_SFTSTAA     TSB_T32A3_RUNA_SFTSTAA

/** 
 *  @}
 */ /* End of group TRMOSC_Private_define */


/*------------------------------------------------------------------------------*/
/*  Enumerated Type Definition                                                  */
/*------------------------------------------------------------------------------*/
/** 
 *  @defgroup TRMOSC_Private_define TRMOSC Private Define
 *  @{
 */
/* no define */

/** 
 *  @}
 */ /* End of group TRMOSC_Private_define */



/*------------------------------------------------------------------------------*/
/*  Structure Definition                                                        */
/*------------------------------------------------------------------------------*/
/** 
 *  @defgroup TRMOSC_Private_typedef TRMOSC Private Typedef
 *  @{
 */
/*----------------------------------*/
/** 
 * @brief Adjustment Information.
*/
/*----------------------------------*/
typedef struct
{
    uint32_t MeasureVal;
    int32_t  MeasureDiff;
    uint32_t Trim;
    uint32_t TRM_Select_Type;
} IOSCADJ_TypeDef;
/**
 *  @}
 */ /* End of group TRMOSC_Private_typedef */

/*------------------------------------------------------------------------------*/
/*  Private Member                                                              */
/*------------------------------------------------------------------------------*/
/** 
 *  @defgroup TRMOSC_Private_variables TRMOSC Private Variables
 *  @{
 */
static uint32_t RetryCount=0;
static int32_t  AdjustResult;
static t32a_t   timer_sout;                /*!< Instance       :Timer Driver  */
static t32a_t   timer_refin;               /*!< Instance       :Timer Driver  */
static cg_t     cg;                        /*!< CG                            */
static IOSCADJ_TypeDef IoscAdjust;
static TRM_ControlTypeDef AdjustData;
static volatile int32_t  difference;
static trm_t trm;
/**
 *  @}
 */ /* End of group TRMOSC_Private_variables */


/*------------------------------------------------------------------------------*/
/*  Private Function                                                            */
/*------------------------------------------------------------------------------*/
static TXZ_Result application_initialize(void);
void InitT32A_Sout(void);
static void InitT32A_RefIn(uint32_t TRM_Select_Type);
void StartCapture(uint32_t TRM_Select_Type);
uint32_t GetCaptureData(void);
void StopCapture(void);
int32_t IOSCAdjustment(uint32_t TRM_Select_Type);
void InitTRM(void);
static void MeasureWave(IOSCADJ_TypeDef *Measure);
static int32_t CalcDiff(IOSCADJ_TypeDef *Diff);
static int32_t CalcAdjVal(IOSCADJ_TypeDef *Adjust);
static int32_t GetAdjustmentValue(void);
/*--------------------------------------------------*/
/** 
  * @brief  Measurement Waveform
  * @param  Measure  :TRMOSC adjustment Structure
  * @retval -
  * @note   -
  */
/*--------------------------------------------------*/
static void MeasureWave(IOSCADJ_TypeDef *Measure)
{
    StartCapture(Measure->TRM_Select_Type);
    Measure->MeasureVal = GetCaptureData();
    StopCapture();
}

/*--------------------------------------------------*/
/** 
  * @brief  Calculation of difference
  * @param  Diff  :TRMOSC adjustment Structure
  * @retval TRMOSC_DONE, TRMOSC_ERROR
  * @note   -
  */
/*--------------------------------------------------*/
static int32_t CalcDiff(IOSCADJ_TypeDef *Diff)
{
    volatile uint32_t absdiff;
    int32_t retval;

    if( Diff->TRM_Select_Type == PPG_SELECT )
    {
        Diff->MeasureDiff = (int32_t)((100*FIXEDPOINT)-((Diff->MeasureVal * (100*FIXEDPOINT)) / CAPTUREVAL10MHZ  ));
    }
    else
    {
        Diff->MeasureDiff = (int32_t)((100*FIXEDPOINT)-((Diff->MeasureVal * (100*FIXEDPOINT)) / CAPTUREVAL2400bps));
    }
/**/
    absdiff = (uint32_t)abs(Diff->MeasureDiff);
    if (absdiff <= MARGINADJUST) 
    {
        retval = TRMOSC_DONE;
    }
    else 
    {
        retval = TRMOSC_ERROR;
    }

    return (retval);
}

/*--------------------------------------------------*/
/** 
  * @brief  Calculation of adjustment value
  * @param  Adjust  :TRMOSC adjustment Structure
  * @retval TRMOSC_RUN, TRMOSC_ERROR
  * @note   -
  */
/*--------------------------------------------------*/
static int32_t CalcAdjVal(IOSCADJ_TypeDef *Adjust)
{
    int32_t retval;
    volatile uint32_t trim;
    uint32_t adjust_step;

    adjust_step = (uint32_t)(abs(Adjust->MeasureDiff)/TRIMSTEP);
    /* adjust step (at least 1)check */
    if(adjust_step == 0)
    {
       adjust_step += 1; 
    }

    /* plus diretion */
    if(Adjust->MeasureDiff > 0)
    {
        difference = (adjust_step*TRIMSTEP)+GetAdjustmentValue();
    }
    /* minus direction */
    else
    {
        difference = -(adjust_step*TRIMSTEP)+GetAdjustmentValue();
    }
     
    /* adujust value */    
    trim = (uint32_t)(abs(difference)/TRIMSTEP);
    
    if (difference < 0)
    {
        trim = TRIM_MAX-trim;
        if(trim > TRIMLIMIT)
        {
            Adjust->Trim = trim;
            retval = TRMOSC_RUN;
        }
        else 
        {
            retval = TRMOSC_ERROR;
        }
    }
    else 
    {
        if(trim <= TRIMLIMIT)
        {
            Adjust->Trim = trim;
            retval = TRMOSC_RUN;
        }
        else 
        {
            retval = TRMOSC_ERROR;
        }
    }

    return (retval);
}

/*--------------------------------------------------*/
/** 
  * @brief  Get Adjustment value
  * @param  -
  * @retval Adjustment value
  * @note   -
  */
/*--------------------------------------------------*/
static int32_t GetAdjustmentValue(void)
{
    uint32_t trimvalue;
    int32_t adjvalue;

    if (GetTrimmingControlValue(&trm) == TRMOSC_CONTROL_DISABLE) 
    {
        trimvalue = GetInitTrimValue(&trm);
    }
    else 
    {
        trimvalue = GetSetTrimValue(&trm);
    }
/**/
    if (trimvalue > TRIMLIMIT) 
    {
        trimvalue = TRIM_MAX - trimvalue;
        adjvalue = -(TRIMSTEP*trimvalue);
    }
    else 
    {
        adjvalue = TRIMSTEP*trimvalue;
    }

    return (adjvalue);
}

/*--------------------------------------------------*/
/** 
  * @brief  Adjustment of internal OSC
  * @param  -
  * @retval TRMOSC_DONE, TRMOSC_ERROR, TRMOSC_RUN
  * @note   -
  */
/*--------------------------------------------------*/
int32_t IOSCAdjustment(uint32_t TRM_Select_Type)
{
    int32_t retval;
    IoscAdjust.TRM_Select_Type = TRM_Select_Type;

    cg.p_instance = TSB_CG;

    MeasureWave(&IoscAdjust);                                           /* Measurement Waveform */
    retval = CalcDiff(&IoscAdjust);                                     /* Calculation of difference */
    if (retval == TRMOSC_ERROR) 
    {
        retval = CalcAdjVal(&IoscAdjust);                               /* Calculation of adjustment value */
        if(retval == TRMOSC_RUN) 
        {
            AdjustData.TrimmingControl     = TRMOSC_CONTROL_ENABLE;
            AdjustData.TrimmingValue = IoscAdjust.Trim;
            SetTrmOsc(&trm, &AdjustData);                                     /* Set TRMOSC register */

            /*--------------------------------*/
            /* Internal HOSC Warm-up Time Set */
            /*--------------------------------*/
            cg_set_warm_up_time(&cg, CG_WARM_UP_SRC_IHOSC1, WU_TIME_INT);
            /*----------------------*/
            /* Warm-up Timer Start  */
            /*----------------------*/
            cg_start_warmup();
            /*----------------------*/
            /* Check Warm-up Timer status */
            /*----------------------*/
            cg_wait_warmup();
        }
        else 
        {                                                           /* TRMOSC_ERROR */
            /* do nothing */
        }
    }
    else 
    {                                                               /* TRMOSC_DONE  */
        /* do nothing */
    }
    return (retval);
}

/*--------------------------------------------------*/
/** 
  * @brief  Initialize TRM Function
  * @param  -
  * @retval -
  * @note   -
  */
/*--------------------------------------------------*/
void InitTRM(void)
{
    TRM_ControlTypeDef InitData;

    cg.p_instance = TSB_CG;

    InitData.TrimmingControl     = TRMOSC_CONTROL_ENABLE;
    InitData.TrimmingValue = TRM_INIT ;
    SetTrmOsc(&trm, &InitData);
    /*--------------------------------*/
    /* Internal HOSC Warm-up Time Set */
    /*--------------------------------*/
    cg_set_warm_up_time(&cg, CG_WARM_UP_SRC_IHOSC1, WU_TIME_INT);
    /*----------------------*/
    /* Warm-up Timer Start  */
    /*----------------------*/
    cg_start_warmup();
    /*----------------------*/
    /* Check Warm-up Timer status */
    /*----------------------*/
    cg_wait_warmup();
}
/*--------------------------------------------------*/
/** 
  * @brief  Initialize T32A Signal Output Function
  * @param  -
  * @retval -
  * @note   -
  */
/*--------------------------------------------------*/
void InitT32A_Sout(void)
{
    TXZ_Result result = TXZ_SUCCESS;

    /*------------------------------*/
    /*  Signal Output Timer         */
    /*------------------------------*/
    /*--- Construct (Register Allocation) ---*/
    /* Register Allocation */
    timer_sout.p_instance = BSP_T32A_SOUT;
    /*--- Initialize: Timer   ---*/
    /* Mode             */
    timer_sout.init_mode.mode.halt = T32A_DBG_HALT_RUN;
    timer_sout.init_mode.mode.mode = T32A_MODE_16;
    if (t32a_mode_init(&timer_sout) != TXZ_SUCCESS)
    {
        result = TXZ_ERROR;
    }
    /*--- Initialize: Timer B ---*/
    {
        t32a_initial_setting_t *p_init = &timer_sout.init[T32A_TIMERB];

        /* Run Control      */
        p_init->runx.sftstp        = T32A_COUNT_STOP;
        p_init->runx.sftsta        = T32A_COUNT_DONT_START;
        p_init->runx.run           = T32A_RUN_ENABLE;
        /* Counter Control  */
        p_init->crx.prscl          = T32A_PRSCLx_1;
        p_init->crx.clk            = T32A_CLKx_PRSCLx;
        p_init->crx.wbf            = T32A_WBF_DISABLE;
        p_init->crx.updn           = T32A_COUNT_UP;
        p_init->crx.reld           = T32A_RELOAD_TREGx;
        p_init->crx.stop           = T32A_STOP_NON;
        p_init->crx.start          = T32A_START_NON;
        /* Output Control 0 */
        p_init->outcrx0.ocr        = T32A_OCR_DISABLE;
        /* Output Control 1 */
        p_init->outcrx1.ocrcap1    = T32A_OCRCAPx1_DISABLE;
        p_init->outcrx1.ocrcap0    = T32A_OCRCAPx0_DISABLE;
        p_init->outcrx1.ocrcmp1    = T32A_OCRCMPx1_INVERSION;
        p_init->outcrx1.ocrcmp0    = T32A_OCRCMPx0_INVERSION;
        /* Timer Counter 1  */
        if(CFG_PPG_RGx1 <= TRM_PPG_MIN){
            p_init->rgx0.rgx0          = 0;
            p_init->rgx1.rgx1          = 1;
        }
        else{
            p_init->rgx0.rgx0          = CFG_PPG_RGx0;
            p_init->rgx1.rgx1          = CFG_PPG_RGx1;
        }
        /* Reload  */
        p_init->reldx.reld         = 0;
        /* Counter Capture  */
        p_init->tmrx.tmrx          = 0;
        /* Counter Capture Control B0 */
        p_init->capx0.capx0        = 0;
        /* Counter Capture Control B1 */
        p_init->capx1.capx1        = 0;
        /* Interrupt mask register Set */
        p_init->imx.imuf           = T32A_IMUFx_MASK_REQ;
        p_init->imx.imof           = T32A_IMOFx_MASK_REQ;
        p_init->imx.imx1           = T32A_IMx1_MASK_REQ;
        p_init->imx.imx0           = T32A_IMx0_MASK_REQ;
        /* DMA Request      */
        p_init->dma_req.dmaenx2    = T32A_DMAENx2_DISABLE;
        p_init->dma_req.dmaenx1    = T32A_DMAENx1_DISABLE;
        p_init->dma_req.dmaenx0    = T32A_DMAENx0_DISABLE;
        /* User ID          */
        p_init->id                 = 0;
        /* Handler          */
        p_init->handler_T   = T32A_NULL;
        p_init->handler_TC0 = T32A_NULL;
        p_init->handler_TC1 = T32A_NULL;
        if (t32a_timer_init(&timer_sout, T32A_TIMERB) != TXZ_SUCCESS)
        {
            result = TXZ_ERROR;
        }
    }
    if (result != TXZ_SUCCESS)
    {
        /* no processing */
    }
}

/*--------------------------------------------------*/
/** 
  * @brief  Initialize T32A Reference Signal Input Function
  * @param  -
  * @retval -
  * @note   -
  */
/*--------------------------------------------------*/
static void InitT32A_RefIn(uint32_t TRM_Select_Type)
{
    TXZ_Result result = TXZ_SUCCESS;

    /*-------------------------------*/
    /*  Reference Signal Input Timer */
    /*-------------------------------*/
    /*--- Construct (Register Allocation) ---*/
    /* Register Allocation */
    timer_refin.p_instance = BSP_T32A_REFIN;
    /*--- Initialize: Timer   ---*/
    /* Mode             */
    timer_refin.init_mode.mode.halt = T32A_DBG_HALT_RUN;
    timer_refin.init_mode.mode.mode = T32A_MODE_16;
    if (t32a_mode_init(&timer_refin) != TXZ_SUCCESS)
    {
        result = TXZ_ERROR;
    }
    /*--- Initialize: Timer A ---*/
    {
        t32a_initial_setting_t *p_init = &timer_refin.init[T32A_TIMERA];

        /* Run Control      */
        p_init->runx.sftstp        = T32A_COUNT_STOP;
        p_init->runx.sftsta        = T32A_COUNT_DONT_START;
        p_init->runx.run           = T32A_RUN_ENABLE;
        /* Counter Control  */
        p_init->crx.prscl          = T32A_PRSCLx_2;
        p_init->crx.clk            = T32A_CLKx_PRSCLx;
        p_init->crx.wbf            = T32A_WBF_DISABLE;
        p_init->crx.updn           = T32A_COUNT_UP;
        p_init->crx.reld           = T32A_RELOAD_NON;
        p_init->crx.stop           = T32A_STOP_EXTTRG_RISING_EDGE;
        p_init->crx.start          = T32A_START_EXTTRG_TRAILING_EDGE;
        /* Counter Control  */
        if( TRM_Select_Type == PPG_SELECT )
        {
            p_init->crx.prscl          = T32A_PRSCLx_2;
            p_init->crx.clk            = T32A_CLKx_PRSCLx;
            p_init->crx.wbf            = T32A_WBF_DISABLE;
            p_init->crx.updn           = T32A_COUNT_UP;
            p_init->crx.reld           = T32A_RELOAD_NON;
            p_init->crx.stop           = T32A_STOP_EXTTRG_RISING_EDGE;
            p_init->crx.start          = T32A_START_EXTTRG_TRAILING_EDGE;
        }
        else
        {
            p_init->crx.prscl          = T32A_PRSCLx_2;
            p_init->crx.clk            = T32A_CLKx_PRSCLx;
            p_init->crx.wbf            = T32A_WBF_DISABLE;
            p_init->crx.updn           = T32A_COUNT_UP;
            p_init->crx.reld           = T32A_RELOAD_NON;
            p_init->crx.stop           = T32A_STOP_NON;
            p_init->crx.start          = T32A_START_EXTTRG_TRAILING_EDGE;
        }

        /* Output Control 0 */
        p_init->outcrx0.ocr        = T32A_OCR_DISABLE;
        /* Output Control 1 */
        p_init->outcrx1.ocrcap1    = T32A_OCRCAPx1_DISABLE;
        p_init->outcrx1.ocrcap0    = T32A_OCRCAPx0_DISABLE;
        p_init->outcrx1.ocrcmp1    = T32A_OCRCMPx1_DISABLE;
        p_init->outcrx1.ocrcmp0    = T32A_OCRCMPx0_DISABLE;
        /* Timer Counter 1  */
        p_init->rgx0.rgx0          = 0;
        p_init->rgx1.rgx1          = 0;
        /* Reload  */
        p_init->reldx.reld         = 0;
        /* Counter Capture  */
        p_init->tmrx.tmrx          = 0;
        /* Capture Control */
        if( TRM_Select_Type == PPG_SELECT )
        {
            p_init->capcrx.capmx1      = T32A_CAPMx1_INx0_RISING_EDGE;
            p_init->capcrx.capmx0      = T32A_CAPMx0_INx0_TRAILING_EDGE;
        }
        else
        {
            p_init->capcrx.capmx1      = T32A_CAPMx1_INx0_TRAILING_EDGE;
            p_init->capcrx.capmx0      = T32A_CAPMx0_INx0_TRAILING_EDGE;
        }
        /* Counter Capture Control A0 */
        p_init->capx0.capx0        = 0;
        /* Counter Capture Control A1 */
        p_init->capx1.capx1        = 0;
        /* Interrupt mask register Set */
        p_init->imx.imuf           = T32A_IMUFx_MASK_REQ;
        p_init->imx.imof           = T32A_IMOFx_MASK_REQ;
        p_init->imx.imx1           = T32A_IMx1_MASK_REQ;
        p_init->imx.imx0           = T32A_IMx0_MASK_REQ;
        /* DMA Request      */
        p_init->dma_req.dmaenx2    = T32A_DMAENx2_DISABLE;
        p_init->dma_req.dmaenx1    = T32A_DMAENx1_DISABLE;
        p_init->dma_req.dmaenx0    = T32A_DMAENx0_DISABLE;
        /* User ID          */
        p_init->id                 = 0;
        /* Handler          */
        p_init->handler_T   = T32A_NULL;
        p_init->handler_TC0 = T32A_NULL;
        p_init->handler_TC1 = T32A_NULL;
        if (t32a_timer_init(&timer_refin, T32A_TIMERA) != TXZ_SUCCESS)
        {
            result = TXZ_ERROR;
        }
    }
    if (result != TXZ_SUCCESS)
    {
        /* no processing */
    }
}

/*--------------------------------------------------*/
/** 
  * @brief  Start Capture
  * @param  -
  * @retval -
  * @note   -
  */
/*--------------------------------------------------*/
void StartCapture(uint32_t TRM_Select_Type)
{
    BSP_T32A_REFIN_RUNA_SFTSTPA = 1;
    if (t32a_SWcounter_stop(&timer_refin, T32A_TIMERA) != TXZ_SUCCESS)
    {
        /* no processing */
    }
    NVIC_ClearPendingIRQ(BSP_INTT32A_REFIN_ACCAP0_IRQn);
    NVIC_ClearPendingIRQ(BSP_INTT32A_REFIN_ACCAP1_IRQn);
    InitT32A_RefIn(TRM_Select_Type);
}

/*--------------------------------------------------*/
/** 
  * @brief  Get Capture Data
  * @param  -
  * @retval Capture value
  * @note   -
  */
/*--------------------------------------------------*/
uint32_t GetCaptureData(void)
{
    uint32_t retval = 0;

    for (;;)
    {
        NVIC_ClearPendingIRQ(BSP_INTT32A_REFIN_ACCAP0_IRQn);
        while (NVIC_GetPendingIRQ(BSP_INTT32A_REFIN_ACCAP0_IRQn) == 0)
        {
            /* waiting falling edge ... */
        }
        NVIC_ClearPendingIRQ(BSP_INTT32A_REFIN_ACCAP1_IRQn);
        while (NVIC_GetPendingIRQ(BSP_INTT32A_REFIN_ACCAP1_IRQn) == 0)
        {
            /* waiting raising edge ... */          
        }
        retval = (timer_refin.p_instance->CAPA1 & T32A_CAPx1_MASK);
        break;
    }   
    return(retval); 
}

/*--------------------------------------------------*/
/** 
  * @brief  Stop Capture
  * @param  -
  * @retval -
  * @note   -
  */
/*--------------------------------------------------*/
void StopCapture(void)
{
    BSP_T32A_REFIN_RUNA_SFTSTPA = 1;
    if (t32a_SWcounter_stop(&timer_refin, T32A_TIMERA) != TXZ_SUCCESS)
    {
        /* no processing */
    }
    NVIC_ClearPendingIRQ(BSP_INTT32A_REFIN_ACCAP0_IRQn);
    NVIC_ClearPendingIRQ(BSP_INTT32A_REFIN_ACCAP1_IRQn);
}
/** 
 *  @defgroup TRMOSC_Private_fuctions TRMOSC Private Fuctions
 *  @{
 */

/*--------------------------------------------------*/
/** 
  * @brief  Application Initialize.
  * @param  -
  * @return Result.
  * @retval TXZ_SUCCESS :Success.
  * @retval TXZ_ERROR   :Failure.
  * @note   -
  */
/*--------------------------------------------------*/
static TXZ_Result application_initialize(void)
{
   TXZ_Result result = TXZ_SUCCESS;

    /*----------------------*/
    /* Usb Uart             */
    /*----------------------*/
    /* Tx & Rx use same UART channel. */
    {
        uart_io_initial_setting_t param;

        param.id         = USER_ID_UART;
        switch (bsp_get_usb_uart_tx_ch(BSP_USB_UART_0))
        {
        case 0:
            param.p_reg  = TSB_UART0;
            break;
        case 1:
            param.p_reg  = TSB_UART1;
            break;
        case 2:
            param.p_reg  = TSB_UART2;
            break;
        default:
            param.p_reg  = MAIN_NULL;
            break;
        }
        param.boudrate   = bsp_get_usb_uart_tx_baudrate(BSP_USB_UART_0);
        param.irq.tx     = bsp_get_usb_uart_tx_nvic(BSP_USB_UART_0);
        param.irq.rx     = bsp_get_usb_uart_rx_nvic(BSP_USB_UART_0);
        param.irq.err    = bsp_get_usb_uart_err_nvic(BSP_USB_UART_0);
        if (param.p_reg != MAIN_NULL)
        {
            result  = uart_io_initialize(&param);
        }
        else
        {
            result = TXZ_ERROR;
        }
    }
    trm.p_instance = TSB_TRM;
    return (result);
}

/**
 *  @}
 */ /* End of group TRMOSC_Private_fuctions */


/*------------------------------------------------------------------------------*/
/*  Public Function                                                             */
/*------------------------------------------------------------------------------*/
/** 
 *  @defgroup TRMOSC_Exported_functions TRMOSC Exported Functions
 *  @{
 */

/*--------------------------------------------------*/
/** 
  * @brief  The main function of TRMOSC demo
  * @param  -
  * @retval -
  * @note   -
  */
/*--------------------------------------------------*/
int main(void)
{
    uint32_t i;

    /*----------------------*/
    /* BSP initialize       */
    /*----------------------*/
    bsp_initialize();
    /* Application initialization */
    if(application_initialize() == TXZ_ERROR){
        /* Do Nothing */
    }
    InitT32A_Sout();
    InitTRM();                   /* for debug */

    if (t32a_SWcounter_start(&timer_refin, T32A_TIMERA) != TXZ_SUCCESS)
    {
        /* no processing */
    }
    if (t32a_SWcounter_start(&timer_sout, T32A_TIMERB) != TXZ_SUCCESS)
    {
        /* no processing */
    }

    if(printf("%s\n", "please wait...") < 0){ /* Do Nothing */ };
    for(i=0; i<0x00400000; i++){};
    if(printf("%s\n", "---------------") < 0){ /* Do Nothing */ };
    if(printf("%s\n", "|    start    |") < 0){ /* Do Nothing */ };
    if(printf("%s\n", "---------------") < 0){ /* Do Nothing */ };

    for( RetryCount = 0; RetryCount <= COUNTMAX; RetryCount++ )
    {
        if(RetryCount >= COUNTMAX )
        {
            if( printf("%s\n", "TRMOSC_Error") < 0 ){ /* Do Nothing */ };
            break;
        }

        AdjustResult = IOSCAdjustment( TRM_SELECT_TYPE );
        if( AdjustResult == TRMOSC_DONE )
        {
            if( printf("%s\n", "TRMOSC_DONE") < 0 ){ /* Do Nothing */ };

            if(GetTrimmingControlValue(&trm) == TRMOSC_CONTROL_ENABLE) 
            {
                uint32_t tmp = GetSetTrimValue(&trm);

                if( printf("%s%X\n", "<TRIMSET>:", tmp) < 0 ){ /* Do Nothing */ };
            }
            break;
        }
        else if( AdjustResult == TRMOSC_RUN )
        {
            if( printf("%s\n", "TRMOSC_RUN") < 0 ){ /* Do Nothing */ };

            if(GetTrimmingControlValue(&trm) == TRMOSC_CONTROL_ENABLE) 
            {
                uint32_t tmp = GetSetTrimValue(&trm);

                if( printf("%s%X\n", "<TRIMSET>:", tmp) < 0 ){ /* Do Nothing */ };
            }
        }
        else
        {
            if( printf("%s\n", "TRMOSC_Error") < 0 ){ /* Do Nothing */ };
            break;
        }
    }

    while(1){};
}
/*--------------------------------------------------*/
/** 
  * @brief  USB UART reception IRQ.
  * @param  uart :USB UART.
  * @return -
  * @retval -
  * @note   This function is called BSP.
  */
/*--------------------------------------------------*/
void irq_usb_uart_rx(BSPUsbUart uart)
{
    switch (uart)
    {
    /*--- usb uart 0 ---*/
    case BSP_USB_UART_0:
        uart_io_rx_irq_handler();
        break;
    /*--- default    ---*/
    default:
        /* no processing */
        break;
    }

}

/*--------------------------------------------------*/
/** 
  * @brief  USB UART transmit IRQ.
  * @param  uart :USB UART.
  * @return -
  * @retval -
  * @note   This function is called BSP.
  */
/*--------------------------------------------------*/
void irq_usb_uart_tx(BSPUsbUart uart)
{
    switch (uart)
    {
    /*--- usb uart 0 ---*/
    case BSP_USB_UART_0:
        uart_io_tx_irq_handler();
        break;
    /*--- default    ---*/
    default:
        /* no processing */
        break;
    }
}

/*--------------------------------------------------*/
/** 
  * @brief  USB UART error IRQ.
  * @param  uart :USB UART.
  * @return -
  * @retval -
  * @note   This function is called BSP.
  */
/*--------------------------------------------------*/
void irq_usb_uart_err(BSPUsbUart uart)
{
    switch (uart)
    {
    /*--- usb uart 0 ---*/
    case BSP_USB_UART_0:
        uart_io_err_irq_handler();
        break;
    /*--- default    ---*/
    default:
        /* no processing */
        break;
    }
}


#ifdef __DEBUG__
/*--------------------------------------------------*/
/** 
  * @brief  Failure Assert.
  * @note   for debug
  */
/*--------------------------------------------------*/
void assert_failed(char *file, int32_t line)
{
    while (1) {
        __NOP();
    }
}
#endif
/**
 *  @}
 */ /* End of group TRMOSC_Exported_functions */

/**
 *  @}
 */ /* End of group TRMOSC */

/**
 *  @} 
 */ /* End of group Example */

#ifdef __cplusplus
}
#endif /* __cplusplus */
