/** 
 *******************************************************************************
 * @file    ei2c.h
 * @brief   This file provides all the functions prototypes for EI2C Class.
 * @version V1.0.0
 * 
 * DO NOT USE THIS SOFTWARE WITHOUT THE SOFTWARE LICENSE AGREEMENT.
 * 
 * Copyright(C) Toshiba Electronic Device Solutions Corporation 2022
 *******************************************************************************
 */
/*------------------------------------------------------------------------------*/
/*  Define to prevent recursive inclusion                                       */
/*------------------------------------------------------------------------------*/
#ifndef __EI2C_H
#define __EI2C_H

#ifdef __cplusplus
extern "C" {
#endif

/*------------------------------------------------------------------------------*/
/*  Includes                                                                    */
/*------------------------------------------------------------------------------*/
#include "driver_com.h"
#include "ei2c_config.h"

/**
 *  @addtogroup Periph_Driver Peripheral Driver
 *  @{
 */

/** 
 *  @addtogroup EI2C
 *  @brief      EI2C Driver.
 *  @{
 */
/*------------------------------------------------------------------------------*/
/*  Macro Function                                                              */
/*------------------------------------------------------------------------------*/
/** 
 *  @addtogroup EI2C_Private_macro
 *  @{
 */

/* no define */

/** 
 *  @}
 */ /* End of group EI2C_Private_macro */


/*------------------------------------------------------------------------------*/
/*  Configuration                                                               */
/*------------------------------------------------------------------------------*/
/** 
 *  @addtogroup EI2C_Private_define
 *  @{
 */

/* no define */

/** 
 *  @}
 */ /* End of group EI2C_Private_define */


/*------------------------------------------------------------------------------*/
/*  Macro Definition                                                            */
/*------------------------------------------------------------------------------*/
/** 
 *  @addtogroup EI2C_Private_define
 *  @{
 */


/** 
 *  @name  EI2C Macro Definition.
 *  @brief EI2C frequency Macro Definition.
 *  @{
 */
#define EI2C_UNUSE_VALUE      ((uint32_t)0xFFFFFFFF)  /*!< UNUSE Value             */
#define EI2C_UNIT_1US         ((uint32_t)1000000)     /*!< 1S=1000000us            */
/**
 *  @}
 */ /* End of name EI2C Definition */

/** 
 *  @name  EI2CSCL_MAX Macro Definition.
 *  @brief EI2CSCL_MAX frequency Macro Definition.
 *  @{
 */
#define EI2CSCL_FRQ_STD       ((uint32_t)100000)    /*!< STD  100kHz             */
#define EI2CSCL_FRQ_FM        ((uint32_t)400000)    /*!< Fm   400kHz             */
#define EI2CSCL_FRQ_FMP       ((uint32_t)1000000)   /*!< Fm+ 1000kHz             */
#define EI2CSCL_SCLH_STDMIN   ((uint32_t)4000)      /*!< SCLH  STD MIN           */
#define EI2CSCL_SCLH_FMMIN    ((uint32_t)600)       /*!< SCLH  FM  MIN           */
#define EI2CSCL_SCLH_FMPMIN   ((uint32_t)260)       /*!< SCLH  FMP MIN           */
#define EI2CSCL_SCLL_STDMIN   ((uint32_t)4700)      /*!< SCLL  STD MIN           */
#define EI2CSCL_SCLL_FMMIN    ((uint32_t)1300)      /*!< SCLL  FM  MIN           */
#define EI2CSCL_SCLL_FMPMIN   ((uint32_t)500)       /*!< SCLL  FMP MIN           */
#define EI2CSCL_TR_STD        ((uint32_t)1000)      /*!< TR  STD MAX(100000)     */
#define EI2CSCL_TF_STD        ((uint32_t)300)       /*!< TF  STD MAX(100000)     */
#define EI2CSCL_TR_FM         ((uint32_t)300)       /*!< TR  FM  MAX(400000)     */
#define EI2CSCL_TF_FM         ((uint32_t)300)       /*!< TF  FM  MAX(400000)     */
#define EI2CSCL_TR_FMP        ((uint32_t)120)       /*!< TR  FMP MAX(1000000)    */
#define EI2CSCL_TF_FMP        ((uint32_t)120)       /*!< TF  FMP MAX(1000000)    */
/**
 *  @}
 */ /* End of name EI2CSCL_MAX Macro Definition */

/** 
 *  @}
 */ /* End of group EI2C_Private_define */


/*------------------------------------------------------------------------------*/
/*  Enumerated Type Definition                                                  */
/*------------------------------------------------------------------------------*/
/** 
 *  @addtogroup EI2C_Private_define
 *  @{
 */

/* no define */

/** 
 *  @}
 */ /* End of group EI2C_Private_define */

/*------------------------------------------------------------------------------*/
/*  Structure Definition                                                        */
/*------------------------------------------------------------------------------*/
/** 
 *  @addtogroup EI2C_Private_typedef
 *  @{
 */

/*----------------------------------*/
/** 
 * @brief  Clock setting structure definition.
*/
/*----------------------------------*/
typedef struct
{
    uint32_t               sck;            /*!< Select internal SCL output clock frequency.                 */
    uint32_t               prs;            /*!< Prescaler clock frequency for generating the Serial clock.  */
    uint32_t               sclh;           /*!< Prescaler clock high width.                                 */
    uint32_t               scll;           /*!< Prescaler clock low width.                                  */
} ei2c_clock_t;

/*----------------------------------*/
/** 
 * @struct ei2c_control0_t
 * @brief  Control Setting structure definition.
*/
/*----------------------------------*/
typedef struct
{
    uint32_t dnf;       /*!< Digtal noise filter clear noise time.
                                : Use @ref EI2CxCR0_DNF   */
    uint32_t nfsel;     /*!< noise filter mode Control.
                                : Use @ref EI2CxCR0_NFSEL */
    uint32_t toper;     /*!< timeout check time Control.
                                : Use @ref EI2CxCR0_TOPER */
    uint32_t toe;       /*!< timeout enable setting. 
                                : Use @ref EI2CxCR0_TOE   */
    uint32_t espe;      /*!< stop condition error Enable setting. 
                                : Use @ref EI2CxCR0_ESPE  */
    uint32_t este;      /*!< start condition error Enable setting. 
                                : Use @ref EI2CxCR0_ESTE  */
    uint32_t nacke;     /*!< NACK Check Enable setting. 
                                : Use @ref EI2CxCR0_NACKE */
    uint32_t gce;       /*!< General call Enable Setting.
                                : Use @ref EI2CxCR0_GCE   */
    uint32_t ale;       /*!< Arbitration lost Enable setting.
                                : Use @ref EI2CxCR0_ALE   */

} ei2c_control0_t;

/*----------------------------------*/
/** 
 * @struct ei2c_control0_t
 * @brief  Control Setting structure definition.
*/
/*----------------------------------*/
typedef struct
{
    uint32_t safs;  /*!< Slave addressing format setting.
                                : Use @ref EI2CxAR1, EI2CxAR2   */
    uint32_t sa;    /*!< Slave address setting.
                                : Use @ref EI2CxAR1, EI2CxAR2   */
    uint32_t sae;   /*!< Slave Address Enable setting.
                                : Use @ref EI2CxAR1, EI2CxAR2   */
} ei2c_aar_setting_t;

/*----------------------------------*/
/** 
 * @brief  Initial setting structure definition.
*/
/*----------------------------------*/
typedef struct
{
    ei2c_clock_t          clock;            /*!< Serial clock setting.            */
    ei2c_control0_t       cr0;              /*!< Control setting.                 */
    ei2c_aar_setting_t    aar1;             /*!< Primary Slave address setting.     */
    ei2c_aar_setting_t    aar2;             /*!< Secondary Slave address setting.   */
} ei2c_initial_setting_t;

/*----------------------------------*/
/** 
 * @brief  I2C handle structure definition.
*/
/*----------------------------------*/
typedef struct
{
    TSB_EI2C_TypeDef       *p_instance;     /*!< Registers base address.           */
    ei2c_initial_setting_t  init;           /*!< Initial setting.                  */
} ei2c_t;

#if (EI2CS_EQUIPPED == AVAILABLE)
/*----------------------------------*/
/** 
 * @brief  Wakeup Control setting structure definition.
*/
/*----------------------------------*/
typedef struct
{
    uint8_t               sgcdi;          /*!< Select general call detect ON/OFF.
                                                : Use @ref EI2CxWUPCR1_SGCDI_xxx */
    uint8_t               ack;            /*!< Select ACK output.
                                                : Use @ref EI2CxWUPCR1_ACK_xxx */
    uint8_t               i2res;          /*!< EI2C BUS Rest.
                                                : Use @ref EI2CxWUPCR1_I2RES_xxx */
    uint8_t               intend;         /*!< Interrupt release.
                                                : Use @ref EI2CxWUPCR1_INTEND_xxx */
} EI2CS_wup_setting_t;

/*----------------------------------*/
/** 
 * @brief  Initial setting structure definition.
*/
/*----------------------------------*/
typedef struct
{
    EI2CS_wup_setting_t    wup;              /*!< Wakeup Control setting.          */
} EI2CS_initial_setting_t;

/*----------------------------------*/
/** 
 * @brief  I2CS handle structure definition.
*/
/*----------------------------------*/
typedef struct
{
    TSB_I2CS_TypeDef       *p_instance;     /*!< Registers base address.           */
    EI2CS_initial_setting_t init;            /*!< Initial setting.                  */
} EI2CS_t;
#endif
/**
 *  @}
 */ /* End of group EI2C_Private_typedef */

/*------------------------------------------------------------------------------*/
/*  Inline Functions                                                             */
/*------------------------------------------------------------------------------*/
/** 
 *  @addtogroup EI2C_Private_functions
 *  @{
 */
__STATIC_INLINE void EI2C_reset(ei2c_t *p_obj);
__STATIC_INLINE int32_t EI2C_port_high(ei2c_t *p_obj);
__STATIC_INLINE void EI2C_stop_condition(ei2c_t *p_obj);
__STATIC_INLINE uint32_t EI2C_read_data(ei2c_t *p_obj);
__STATIC_INLINE void EI2C_write_data(ei2c_t *p_obj, uint32_t data);
__STATIC_INLINE int32_t EI2C_restart(ei2c_t *p_obj);
__STATIC_INLINE void EI2C_set_ack(ei2c_t *p_obj, int32_t nack);
__STATIC_INLINE int32_t EI2C_get_ack(ei2c_t *p_obj);
__STATIC_INLINE int32_t EI2C_status_busy(ei2c_t *p_obj);
__STATIC_INLINE int32_t EI2C_master(ei2c_t *p_obj);
__STATIC_INLINE int32_t EI2C_transmitter(ei2c_t *p_obj);
__STATIC_INLINE int32_t EI2C_int_status(ei2c_t *p_obj);
__STATIC_INLINE void EI2C_clear_int_status(ei2c_t *p_obj);
__STATIC_INLINE void EI2C_enable_interrupt(ei2c_t *p_obj);
__STATIC_INLINE void EI2C_enable_interrupt_dma(ei2c_t *p_obj, int32_t tx);
__STATIC_INLINE void EI2C_disable_interrupt(ei2c_t *p_obj);
__STATIC_INLINE void EI2C_set_address1(ei2c_t *p_obj, uint32_t format, uint32_t addr, uint32_t enable);
__STATIC_INLINE void EI2C_set_address2(ei2c_t *p_obj, uint32_t format, uint32_t addr, uint32_t enable);
__STATIC_INLINE int32_t EI2C_slave_detected(ei2c_t *p_obj);
__STATIC_INLINE int32_t EI2C_status_arbitration(ei2c_t *p_obj);
__STATIC_INLINE int32_t EI2C_status_esp(ei2c_t *p_obj);
__STATIC_INLINE void EI2C_clear_int_nack(ei2c_t *p_obj);
__STATIC_INLINE int32_t EI2C_status_rbf(ei2c_t *p_obj);
__STATIC_INLINE void EI2C_set_acksel(ei2c_t *p_obj, int32_t nack);
__STATIC_INLINE void  EI2C_clear_int_aas1(ei2c_t *p_obj);
__STATIC_INLINE int32_t EI2C_get_bc(ei2c_t *p_obj);
/*--------------------------------------------------*/
/**
  * @brief     EI2C software reset.
  * @param     p_obj       :EI2C object.
  * @note      -
  */
/*--------------------------------------------------*/
__STATIC_INLINE void EI2C_reset(ei2c_t *p_obj)
{
#ifdef __DEBUG__
    if ((p_obj != EI2C_NULL) && (p_obj->p_instance != EI2C_NULL))
    {
        p_obj->p_instance->ARST = EI2CxARST_SWRES_10;
        p_obj->p_instance->ARST = EI2CxARST_SWRES_01;
    }
#else
    p_obj->p_instance->ARST = EI2CxARST_SWRES_10;
    p_obj->p_instance->ARST = EI2CxARST_SWRES_01;
#endif
}

/*--------------------------------------------------*/
/**
  * @brief     EI2C bus port high
  * @param     p_obj       :EI2C object.
  * @retval    true        :SDA and SCL Port High.
  * @retval    false       :Bus Error.
  * @note      -
  */
/*--------------------------------------------------*/
__STATIC_INLINE int32_t EI2C_port_high(ei2c_t *p_obj)
{
#ifdef __DEBUG__
    if ((p_obj != EI2C_NULL) && (p_obj->p_instance != EI2C_NULL))
    {
        return (((p_obj->p_instance->APM & EI2CxPM_SDA_SCL) == EI2CxPM_SDA_SCL));
    }
    return (0);
#else
    return (((p_obj->p_instance->APM & EI2CxPM_SDA_SCL) == EI2CxPM_SDA_SCL));
#endif
}

/*--------------------------------------------------*/
/**
  * @brief     Generate stop condition.
  * @param     p_obj       :EI2C object.
  * @retval    -
  * @note      -
  */
/*--------------------------------------------------*/
__STATIC_INLINE void EI2C_stop_condition(ei2c_t *p_obj)
{
#ifdef __DEBUG__
    if ((p_obj != EI2C_NULL) && (p_obj->p_instance != EI2C_NULL))
    {
       if(p_obj->p_instance->ASR0 & EI2CxSR0_MST == EI2CxSR0_MST )
       {
          p_obj->p_instance->ACR1 |= EI2CxCR1_SP;
       }
    }
#else
    if((p_obj->p_instance->ASR0&EI2CxSR0_MST) == EI2CxSR0_MST )
    {
       p_obj->p_instance->ACR1 |= EI2CxCR1_SP;
    }
#endif
}

/*--------------------------------------------------*/
/**
  * @brief     Generate stop condition.
  * @param     p_obj       :EI2C object.
  * @retval    -
  * @note      -
  */
/*--------------------------------------------------*/
__STATIC_INLINE void EI2C_acr1_clear(ei2c_t *p_obj)
{
#ifdef __DEBUG__
    if ((p_obj != EI2C_NULL) && (p_obj->p_instance != EI2C_NULL))
    {
       p_obj->p_instance->ACR1 = EI2CxCR1_CLEAR;
    }
#else
    p_obj->p_instance->ACR1 = EI2CxCR1_CLEAR;
#endif
}
/*--------------------------------------------------*/
/**
  * @brief     Read from Data buffer
  * @param     p_obj       :EI2C object.
  * @retval    result      :Read data.
  * @note      -
  */
/*--------------------------------------------------*/
__STATIC_INLINE uint32_t EI2C_read_data(ei2c_t *p_obj)
{
#ifdef __DEBUG__
    if ((p_obj != EI2C_NULL) && (p_obj->p_instance != EI2C_NULL))
    {
        return (p_obj->p_instance->ADBRR & EI2CxDBRR_DBR_MASK);
    }
    return (0);
#else
    return (p_obj->p_instance->ADBRR & EI2CxDBRR_DBR_MASK);
#endif
}

/*--------------------------------------------------*/
/**
  * @brief     Write to Data buffer.
  * @param     p_obj       :EI2C object.
  * @param     data        :Write data.
  * @retval    -
  * @note      -
  */
/*--------------------------------------------------*/
__STATIC_INLINE void EI2C_write_data(ei2c_t *p_obj, uint32_t data)
{
#ifdef __DEBUG__
    if ((p_obj != EI2C_NULL) && (p_obj->p_instance != EI2C_NULL))
    {
        p_obj->p_instance->ADBRT = (data & EI2CxDBRT_DBT_MASK);
    }
#else
    p_obj->p_instance->ADBRT = (data & EI2CxDBRT_DBT_MASK);
#endif
}

/*--------------------------------------------------*/
/**
  * @brief     Return restart condition
  * @param     p_obj    :EI2C object.
  * @retval    true     :Restart Detected.
  * @retval    false    :Restart Non-Detected.
  * @note      -
  */
/*--------------------------------------------------*/
__STATIC_INLINE int32_t EI2C_restart(ei2c_t *p_obj)
{
#ifdef __DEBUG__
    if ((p_obj != EI2C_NULL) && (p_obj->p_instance != EI2C_NULL))
    {
        __IO uint32_t opreg = p_obj->p_instance->ASR1;
        p_obj->p_instance->ASR1 &= ~EI2CxSR1_RSCF;
        return ((opreg & EI2CxSR1_RSCF) == EI2CxSR1_RSCF);
    }
    return (0);
#else
    __IO uint32_t opreg = p_obj->p_instance->ASR1;
    p_obj->p_instance->ASR1 &= ~EI2CxSR1_RSCF;
    return ((opreg & EI2CxSR1_RSCF) == EI2CxSR1_RSCF);
#endif
}

/*--------------------------------------------------*/
/**
  * @brief     Set Ack condition
  * @param     p_obj       :EI2C object.
  * @param     nack        :1 NACK, 0 ACK.
  * @retval    -
  * @note      -
  */
/*--------------------------------------------------*/
__STATIC_INLINE void EI2C_set_ack(ei2c_t *p_obj, int32_t nack)
{
#ifdef __DEBUG__
    if ((p_obj != EI2C_NULL) && (p_obj->p_instance != EI2C_NULL))
    {
        if (nack)
        {
            p_obj->p_instance->ACR1 |= EI2CxCR1_ACKWAIT;
        }
        else
        {
            p_obj->p_instance->ACR1 &= ~EI2CxCR1_ACKWAIT;
        }
    }
#else
    if (nack)
    {
        p_obj->p_instance->ACR1 |= EI2CxCR1_ACKWAIT;
    }
    else
    {
        p_obj->p_instance->ACR1 &= ~EI2CxCR1_ACKWAIT;
    }
#endif
}

/*--------------------------------------------------*/
/**
  * @brief     Return received Ack condition
  * @param     p_obj    :EI2C object.
  * @retval    true     :NACK Received.
  * @retval    false    :ACK Received.
  * @note      -
  */
/*--------------------------------------------------*/
__STATIC_INLINE int32_t EI2C_get_ack(ei2c_t *p_obj)
{
#ifdef __DEBUG__
    if ((p_obj != EI2C_NULL) && (p_obj->p_instance != EI2C_NULL))
    {
        return ((p_obj->p_instance->ASR1 & EI2CxSR1_NACK) == EI2CxSR1_NACK);
    }
    return (0);
#else
    return ((p_obj->p_instance->ASR1 & EI2CxSR1_NACK) == EI2CxSR1_NACK);
#endif
}

/*--------------------------------------------------*/
/**
  * @brief     Return Busy condition
  * @param     p_obj       :EI2C object.
  * @retval    true        :EI2C bus busy.
  * @retval    false       :EI2C bus free.
  * @note      -
  */
/*--------------------------------------------------*/
__STATIC_INLINE int32_t EI2C_status_busy(ei2c_t *p_obj)
{
#ifdef __DEBUG__
    if ((p_obj != EI2C_NULL) && (p_obj->p_instance != EI2C_NULL))
    {
        return ((p_obj->p_instance->ASR0 & EI2CxSR0_BB) == EI2CxSR0_BB);
    }
    return (0);
#else
    return ((p_obj->p_instance->ASR0 & EI2CxSR0_BB) == EI2CxSR0_BB);
#endif
}

/*--------------------------------------------------*/
/**
  * @brief     Return The Master status
  * @param     p_obj       :EI2C object.
  * @retval    true        :Master mode.
  * @retval    false       :Slave mode.
  * @note      -
  */
/*--------------------------------------------------*/
__STATIC_INLINE int32_t EI2C_master(ei2c_t *p_obj)
{
#ifdef __DEBUG__
    if ((p_obj != EI2C_NULL) && (p_obj->p_instance != EI2C_NULL))
    {
        return ((p_obj->p_instance->ASR0 & EI2CxSR0_MST) == EI2CxSR0_MST);
    }
    return (0);
#else
    return ((p_obj->p_instance->ASR0 & EI2CxSR0_MST) == EI2CxSR0_MST);
#endif
}

/*--------------------------------------------------*/
/**
  * @brief     Return The Transmitter
  * @param     p_obj       :EI2C object.
  * @retval    true        :Transmitter.
  * @retval    false       :Receiver.
  * @note      -
  */
/*--------------------------------------------------*/
__STATIC_INLINE int32_t EI2C_transmitter(ei2c_t *p_obj)
{
#ifdef __DEBUG__
    if ((p_obj != EI2C_NULL) && (p_obj->p_instance != EI2C_NULL))
    {
        return ((p_obj->p_instance->ASR0 & EI2CxSR0_TRX) == EI2CxSR0_TRX);
    }
    return (0);
#else
    return ((p_obj->p_instance->ASR0 & EI2CxSR0_TRX) == EI2CxSR0_TRX);
#endif
}

/*--------------------------------------------------*/
/**
  * @brief     Interrupt Status
  * @param     p_obj       :EI2C object.
  * @retval    true        :Interrupt Occurred.
  * @retval    false       :No Interrupt Occurred.
  * @note      -
  */
/*--------------------------------------------------*/
__STATIC_INLINE int32_t EI2C_int_status(ei2c_t *p_obj)
{
    /* Check the INTI2CxST status */
#ifdef __DEBUG__
    if ((p_obj != EI2C_NULL) && (p_obj->p_instance != EI2C_NULL))
    {
        if((p_obj->p_instance->ASR1 & EI2CxSR1_TOERR)
        || (p_obj->p_instance->ASR1 & EI2CxSR1_ESP)
        || (p_obj->p_instance->ASR1 & EI2CxSR1_EST)
        || (p_obj->p_instance->ASR1 & EI2CxSR1_AAS2)
        || (p_obj->p_instance->ASR1 & EI2CxSR1_AAS1)
        || (p_obj->p_instance->ASR1 & EI2CxSR1_GC)
        || (p_obj->p_instance->ASR1 & EI2CxSR1_AL)
        || (p_obj->p_instance->ASR1 & EI2CxSR1_NACK)
        || (p_obj->p_instance->ASR1 & EI2CxSR1_TEND)
        || (p_obj->p_instance->ASR1 & EI2CxSR1_SPCF)
        || (p_obj->p_instance->ASR1 & EI2CxSR1_RSCF)
        || (p_obj->p_instance->ASR1 & EI2CxSR1_STCF))
        {
           return (1);
        }
        else 
        {
           return (0);
        }
    }
    return (0);
#else
    if((p_obj->p_instance->ASR1 & EI2CxSR1_TOERR)
    || (p_obj->p_instance->ASR1 & EI2CxSR1_ESP)
    || (p_obj->p_instance->ASR1 & EI2CxSR1_EST)
    || (p_obj->p_instance->ASR1 & EI2CxSR1_AAS2)
    || (p_obj->p_instance->ASR1 & EI2CxSR1_AAS1)
    || (p_obj->p_instance->ASR1 & EI2CxSR1_GC)
    || (p_obj->p_instance->ASR1 & EI2CxSR1_AL)
    || (p_obj->p_instance->ASR1 & EI2CxSR1_NACK)
    || (p_obj->p_instance->ASR1 & EI2CxSR1_TEND)
    || (p_obj->p_instance->ASR1 & EI2CxSR1_SPCF)
    || (p_obj->p_instance->ASR1 & EI2CxSR1_RSCF)
    || (p_obj->p_instance->ASR1 & EI2CxSR1_STCF))
    {
       return (1);
    }
    else 
    {
       return (0);
    }
#endif
}

/*--------------------------------------------------*/
/**
  * @brief     Interrupt Status Clear
  * @param     p_obj       :EI2C object.
  * @retval    -
  * @note      -
  */
/*--------------------------------------------------*/
__STATIC_INLINE void EI2C_clear_int_status(ei2c_t *p_obj)
{
#ifdef __DEBUG__
    if ((p_obj != EI2C_NULL) && (p_obj->p_instance != EI2C_NULL))
    {
        p_obj->p_instance->ASR1 = EI2CxSR1_CLEAR;
    }
#else
    p_obj->p_instance->ASR1 = EI2CxSR1_CLEAR;
#endif
}

/*--------------------------------------------------*/
/**
  * @brief     Interrupt Status Clear
  * @param     p_obj       :EI2C object.
  * @retval    -
  * @note      -
  */
/*--------------------------------------------------*/
__STATIC_INLINE void EI2C_clear_int_stcf(ei2c_t *p_obj)
{
#ifdef __DEBUG__
    if ((p_obj != EI2C_NULL) && (p_obj->p_instance != EI2C_NULL))
    {
        p_obj->p_instance->ASR1 = EI2CxSR1_STCF;
    }
#else
    p_obj->p_instance->ASR1 = EI2CxSR1_STCF;
#endif
}

/*--------------------------------------------------*/
/**
  * @brief     Interrupt Status Clear
  * @param     p_obj       :EI2C object.
  * @retval    -
  * @note      -
  */
/*--------------------------------------------------*/
__STATIC_INLINE void EI2C_clear_int_rscf(ei2c_t *p_obj)
{
#ifdef __DEBUG__
    if ((p_obj != EI2C_NULL) && (p_obj->p_instance != EI2C_NULL))
    {
        p_obj->p_instance->ASR1 = EI2CxSR1_RSCF;
    }
#else
    p_obj->p_instance->ASR1 = EI2CxSR1_RSCF;
#endif
}
/*--------------------------------------------------*/
/**
  * @brief     Interrupt Status Clear
  * @param     p_obj       :EI2C object.
  * @retval    -
  * @note      -
  */
/*--------------------------------------------------*/
__STATIC_INLINE void EI2C_clear_int_tbe(ei2c_t *p_obj)
{
#ifdef __DEBUG__
    if ((p_obj != EI2C_NULL) && (p_obj->p_instance != EI2C_NULL))
    {
        p_obj->p_instance->ASR1 = EI2CxSR1_TBE;
    }
#else
    p_obj->p_instance->ASR1 = EI2CxSR1_TBE;
#endif
}

/*--------------------------------------------------*/
/**
  * @brief     Interrupt Status Clear
  * @param     p_obj       :EI2C object.
  * @retval    -
  * @note      -
  */
/*--------------------------------------------------*/
__STATIC_INLINE void EI2C_clear_int_tend(ei2c_t *p_obj)
{
#ifdef __DEBUG__
    if ((p_obj != EI2C_NULL) && (p_obj->p_instance != EI2C_NULL))
    {
        p_obj->p_instance->ASR1 = EI2CxSR1_TEND;
    }
#else
    p_obj->p_instance->ASR1 = EI2CxSR1_TEND;
#endif
}

/*--------------------------------------------------*/
/**
  * @brief     Interrupt Status Clear
  * @param     p_obj       :EI2C object.
  * @retval    -
  * @note      -
  */
/*--------------------------------------------------*/
__STATIC_INLINE void EI2C_clear_int_arbitration(ei2c_t *p_obj)
{
#ifdef __DEBUG__
    if ((p_obj != EI2C_NULL) && (p_obj->p_instance != EI2C_NULL))
    {
        p_obj->p_instance->ASR1 = EI2CxSR1_AL;
    }
#else
    p_obj->p_instance->ASR1 = EI2CxSR1_AL;
#endif
}
/*--------------------------------------------------*/
/**
  * @brief     Enable Interrupt setting.
  * @param     p_obj       :EI2C object.
  * @retval    -
  * @note      -
  */
/*--------------------------------------------------*/
__STATIC_INLINE void EI2C_enable_interrupt(ei2c_t *p_obj)
{
#ifdef __DEBUG__
    if ((p_obj != EI2C_NULL) && (p_obj->p_instance != EI2C_NULL))
    {
        p_obj->p_instance->AIE = (EI2CxIE_INTSTE | EI2CxIE_INTRSE | EI2CxIE_INTSPE | EI2CxIE_INTNACKE
         | EI2CxIE_INTALE | EI2CxIE_INTGCE | EI2CxIE_INTASE | EI2CxIE_INTESTE | EI2CxIE_INTESPE | EI2CxIE_INTTOE);
    }
#else
    p_obj->p_instance->AIE = (EI2CxIE_INTSTE | EI2CxIE_INTRSE | EI2CxIE_INTSPE | EI2CxIE_INTNACKE
     | EI2CxIE_INTALE | EI2CxIE_INTGCE | EI2CxIE_INTASE | EI2CxIE_INTESTE | EI2CxIE_INTESPE | EI2CxIE_INTTOE);
#endif
}

/*--------------------------------------------------*/
/**
  * @brief     Enable Interrupt setting.
  * @param     p_obj       :EI2C object.
  * @param     tx          :Direction of transfer(1=tx 0=rx).
  * @retval    -
  * @note      For DMA transfer.
  */
/*--------------------------------------------------*/
__STATIC_INLINE void EI2C_enable_interrupt_dma(ei2c_t *p_obj, int32_t tx)
{
#ifdef __DEBUG__
    if ((p_obj != EI2C_NULL) && (p_obj->p_instance != EI2C_NULL))
    {
        if (tx)
        {
            p_obj->p_instance->AIE = EI2CxIE_DMATX;
        }
        else
        {
            p_obj->p_instance->AIE = EI2CxIE_DMARX;
        }
    }
#else
    if (tx)
    {
        p_obj->p_instance->AIE = EI2CxIE_DMATX;
    }
    else
    {
        p_obj->p_instance->AIE = EI2CxIE_DMARX;
    }
#endif
}

/*--------------------------------------------------*/
/**
  * @brief     Disable Interrupt setting.
  * @param     p_obj       :EI2C object.
  * @retval    -
  * @note      -
  */
/*--------------------------------------------------*/
__STATIC_INLINE void EI2C_disable_interrupt(ei2c_t *p_obj)
{
#ifdef __DEBUG__
    if ((p_obj != EI2C_NULL) && (p_obj->p_instance != EI2C_NULL))
    {
        p_obj->p_instance->AIE = EI2CxIE_DISABLE;
    }
#else
    p_obj->p_instance->AIE = EI2CxIE_DISABLE;
#endif
}

/*--------------------------------------------------*/
/**
  * @brief     Set slave address.
  * @param     p_obj       :EI2C object.
  * @param     format      :Select the EI2C address setting.
  * @param     addr        :slave address.
  * @param     enable      :Enable or Disable setting.
  * @retval    -
  * @note      -
  */
/*--------------------------------------------------*/
__STATIC_INLINE void EI2C_set_address1(ei2c_t *p_obj, uint32_t format, uint32_t addr, uint32_t enable)
{
    __IO uint32_t aar1 = 0;
    
    if(format == EI2CxAR1_SAFS1_7BIT)
    {
        aar1 = (format | (addr & EI2CxAR1_SA1_7BITMASK) | enable);
    }
    else
    {
        aar1 = (format | (addr & EI2CxAR1_SA1_10BITMASK) | enable);
    }
    p_obj->p_instance->AAR1 = aar1;
}

/*--------------------------------------------------*/
/**
  * @brief     Set slave address.
  * @param     p_obj       :EI2C object.
  * @param     format      :Select the EI2C address setting.
  * @param     addr        :slave address.
  * @param     enable      :Enable or Disable setting.
  * @retval    -
  * @note      -
  */
/*--------------------------------------------------*/
__STATIC_INLINE void EI2C_set_address2(ei2c_t *p_obj, uint32_t format, uint32_t addr, uint32_t enable)
{
    __IO uint32_t aar2 = 0;
    
    if(format == EI2CxAR2_SAFS2_7BIT)
    {
        aar2 = (format | (addr & EI2CxAR2_SA2_7BITMASK) | enable);
    }
    else
    {
        aar2 = (format | (addr & EI2CxAR2_SA2_10BITMASK) | enable);
    }
    p_obj->p_instance->AAR2 = aar2;
}

/*--------------------------------------------------*/
/**
  * @brief     Detecting Slave Address
  * @param     p_obj       :EI2C object.
  * @retval    -
  * @note      -
  */
/*--------------------------------------------------*/
__STATIC_INLINE int32_t EI2C_slave_detected(ei2c_t *p_obj)
{
#ifdef __DEBUG__
    if ((p_obj != EI2C_NULL) && (p_obj->p_instance != EI2C_NULL))
    {
        return (((p_obj->p_instance->ASR1 & EI2CxSR1_AAS1) == EI2CxSR1_AAS1)
                 ||((p_obj->p_instance->ASR1 & EI2CxSR1_AAS2) == EI2CxSR1_AAS2));
    }
    return (0);
#else
    return (((p_obj->p_instance->ASR1 & EI2CxSR1_AAS1) == EI2CxSR1_AAS1)
             ||((p_obj->p_instance->ASR1 & EI2CxSR1_AAS2) == EI2CxSR1_AAS2));
#endif
}

/*--------------------------------------------------*/
/**
  * @brief     status arbitration
  * @param     p_obj       :EI2C object.
  * @retval    -
  * @note      -
  */
/*--------------------------------------------------*/
__STATIC_INLINE int32_t EI2C_status_arbitration(ei2c_t *p_obj)
{
#ifdef __DEBUG__
    if ((p_obj != EI2C_NULL) && (p_obj->p_instance != EI2C_NULL))
    {
        return ((p_obj->p_instance->ASR1 & EI2CxSR1_AL) == EI2CxSR1_AL);
    }
    return (0);
#else
    return ((p_obj->p_instance->ASR1 & EI2CxSR1_AL) == EI2CxSR1_AL);
#endif
}

/*--------------------------------------------------*/
/**
  * @brief     status stcf
  * @param     p_obj       :EI2C object.
  * @retval    -
  * @note      -
  */
/*--------------------------------------------------*/
__STATIC_INLINE int32_t EI2C_status_stcf(ei2c_t *p_obj)
{
#ifdef __DEBUG__
    if ((p_obj != EI2C_NULL) && (p_obj->p_instance != EI2C_NULL))
    {
        return ((p_obj->p_instance->ASR1 & EI2CxSR1_STCF) == EI2CxSR1_STCF);
    }
    return (0);
#else
    return ((p_obj->p_instance->ASR1 & EI2CxSR1_STCF) == EI2CxSR1_STCF);
#endif
}

/*--------------------------------------------------*/
/**
  * @brief     status rscf
  * @param     p_obj       :EI2C object.
  * @retval    -
  * @note      -
  */
/*--------------------------------------------------*/
__STATIC_INLINE int32_t EI2C_status_rscf(ei2c_t *p_obj)
{
#ifdef __DEBUG__
    if ((p_obj != EI2C_NULL) && (p_obj->p_instance != EI2C_NULL))
    {
        return ((p_obj->p_instance->ASR1 & EI2CxSR1_RSCF) == EI2CxSR1_RSCF);
    }
    return (0);
#else
    return ((p_obj->p_instance->ASR1 & EI2CxSR1_RSCF) == EI2CxSR1_RSCF);
#endif
}

/*--------------------------------------------------*/
/**
  * @brief     status stcf
  * @param     p_obj       :EI2C object.
  * @retval    -
  * @note      -
  */
/*--------------------------------------------------*/
__STATIC_INLINE int32_t EI2C_status_tbe(ei2c_t *p_obj)
{
#ifdef __DEBUG__
    if ((p_obj != EI2C_NULL) && (p_obj->p_instance != EI2C_NULL))
    {
        return ((p_obj->p_instance->ASR1 & EI2CxSR1_TBE) == EI2CxSR1_TBE);
    }
    return (0);
#else
    return ((p_obj->p_instance->ASR1 & EI2CxSR1_TBE) == EI2CxSR1_TBE);
#endif
}

/*--------------------------------------------------*/
/**
  * @brief     status stcf
  * @param     p_obj       :EI2C object.
  * @retval    -
  * @note      -
  */
/*--------------------------------------------------*/
__STATIC_INLINE int32_t EI2C_status_tend(ei2c_t *p_obj)
{
#ifdef __DEBUG__
    if ((p_obj != EI2C_NULL) && (p_obj->p_instance != EI2C_NULL))
    {
        return ((p_obj->p_instance->ASR1 & EI2CxSR1_TEND) == EI2CxSR1_TEND);
    }
    return (0);
#else
    return ((p_obj->p_instance->ASR1 & EI2CxSR1_TEND) == EI2CxSR1_TEND);
#endif
}

/*--------------------------------------------------*/
/**
  * @brief     status spcf
  * @param     p_obj       :EI2C object.
  * @retval    -
  * @note      -
  */
/*--------------------------------------------------*/
__STATIC_INLINE int32_t EI2C_status_spcf(ei2c_t *p_obj)
{
#ifdef __DEBUG__
    if ((p_obj != EI2C_NULL) && (p_obj->p_instance != EI2C_NULL))
    {
        return ((p_obj->p_instance->ASR1 & EI2CxSR1_SPCF) == EI2CxSR1_SPCF);
    }
    return (0);
#else
    return ((p_obj->p_instance->ASR1 & EI2CxSR1_SPCF) == EI2CxSR1_SPCF);
#endif
}
/*--------------------------------------------------*/
/**
  * @brief     status toerr
  * @param     p_obj       :EI2C object.
  * @retval    -
  * @note      -
  */
/*--------------------------------------------------*/
__STATIC_INLINE int32_t EI2C_status_toerr(ei2c_t *p_obj)
{
#ifdef __DEBUG__
    if ((p_obj != EI2C_NULL) && (p_obj->p_instance != EI2C_NULL))
    {
        return ((p_obj->p_instance->ASR1 & EI2CxSR1_TOERR) == EI2CxSR1_TOERR);
    }
    return (0);
#else
    return ((p_obj->p_instance->ASR1 & EI2CxSR1_TOERR) == EI2CxSR1_TOERR);
#endif
}

/*--------------------------------------------------*/
/**
  * @brief     status esp
  * @param     p_obj       :EI2C object.
  * @retval    -
  * @note      -
  */
/*--------------------------------------------------*/
__STATIC_INLINE int32_t EI2C_status_esp(ei2c_t *p_obj)
{
#ifdef __DEBUG__
    if ((p_obj != EI2C_NULL) && (p_obj->p_instance != EI2C_NULL))
    {
        return ((p_obj->p_instance->ASR1 & EI2CxSR1_ESP) == EI2CxSR1_ESP);
    }
    return (0);
#else
    return ((p_obj->p_instance->ASR1 & EI2CxSR1_ESP) == EI2CxSR1_ESP);
#endif
}

/*--------------------------------------------------*/
/**
  * @brief     Interrupt Status Clear
  * @param     p_obj       :EI2C object.
  * @retval    -
  * @note      -
  */
/*--------------------------------------------------*/
__STATIC_INLINE void EI2C_clear_int_nack(ei2c_t *p_obj)
{
#ifdef __DEBUG__
    if ((p_obj != EI2C_NULL) && (p_obj->p_instance != EI2C_NULL))
    {
        p_obj->p_instance->ASR1 = EI2CxSR1_NACK;
    }
#else
    p_obj->p_instance->ASR1 = EI2CxSR1_NACK;
#endif
}

/*--------------------------------------------------*/
/**
  * @brief     status rbf
  * @param     p_obj       :EI2C object.
  * @retval    -
  * @note      -
  */
/*--------------------------------------------------*/
__STATIC_INLINE int32_t EI2C_status_rbf(ei2c_t *p_obj)
{
#ifdef __DEBUG__
    if ((p_obj != EI2C_NULL) && (p_obj->p_instance != EI2C_NULL))
    {
        return ((p_obj->p_instance->ASR1 & EI2CxSR1_RBF) == EI2CxSR1_RBF);
    }
    return (0);
#else
    return ((p_obj->p_instance->ASR1 & EI2CxSR1_RBF) == EI2CxSR1_RBF);
#endif
}

/*--------------------------------------------------*/
/**
  * @brief     Set Ack condition
  * @param     p_obj       :EI2C object.
  * @param     nack        :1 NACK, 0 ACK.
  * @retval    -
  * @note      -
  */
/*--------------------------------------------------*/
__STATIC_INLINE void EI2C_set_acksel(ei2c_t *p_obj, int32_t nack)
{
#ifdef __DEBUG__
    if ((p_obj != EI2C_NULL) && (p_obj->p_instance != EI2C_NULL))
    {
        if (nack)
        {
            p_obj->p_instance->ACR1 |= EI2CxCR1_ACKSEL_NACK;
        }
        else
        {
            p_obj->p_instance->ACR1 &= ~EI2CxCR1_ACKSEL_NACK;
        }
    }
#else
    if (nack)
    {
        p_obj->p_instance->ACR1 |= EI2CxCR1_ACKSEL_NACK;
    }
    else
    {
        p_obj->p_instance->ACR1 &= ~EI2CxCR1_ACKSEL_NACK;
    }
#endif
}


/*--------------------------------------------------*/
/**
  * @brief     Interrupt Status Clear
  * @param     p_obj       :EI2C object.
  * @retval    -
  * @note      -
  */
/*--------------------------------------------------*/
__STATIC_INLINE void  EI2C_clear_int_aas1(ei2c_t *p_obj)
{
#ifdef __DEBUG__
    if ((p_obj != EI2C_NULL) && (p_obj->p_instance != EI2C_NULL))
    {
        p_obj->p_instance->ASR1 = EI2CxSR1_AAS1;
    }
#else
    p_obj->p_instance->ASR1 = EI2CxSR1_AAS1;
#endif
}

/*--------------------------------------------------*/
/**
  * @brief     Return BC
  * @param     p_obj    :EI2C object.
  * @retval    BC.
  * @note      -
  */
/*--------------------------------------------------*/
__STATIC_INLINE int32_t EI2C_get_bc(ei2c_t *p_obj)
{
#ifdef __DEBUG__
    if ((p_obj != EI2C_NULL) && (p_obj->p_instance != EI2C_NULL))
    {
        return ((p_obj->p_instance->ASR0 & EI2CxSR0_BC) != EI2CxSR0_INT);
    }
    return (0);
#else
    return ((p_obj->p_instance->ASR0 & EI2CxSR0_BC) != EI2CxSR0_INT);
#endif
}
/**
 *  @}
 */ /* End of group EI2C_Private_functions */

/*------------------------------------------------------------------------------*/
/*  Functions                                                                   */
/*------------------------------------------------------------------------------*/
/** 
 *  @addtogroup EI2C_Private_functions
 *  @{
 */
void EI2C_init(ei2c_t *p_obj);
void EI2C_restartcondition(ei2c_t *p_obj);
void EI2C_startcondition(ei2c_t *p_obj);
void EI2C_slave_init(ei2c_t *p_obj);
TXZ_Result EI2C_set_prs_scl_dig(ei2c_t *p_obj, uint32_t fsys, uint8_t dnf);
TXZ_Result EI2C_set_prs_scl_ana(ei2c_t *p_obj, uint32_t fsys);
#if (EI2CS_EQUIPPED == AVAILABLE)
void EI2CS_init(EI2CS_t *p_obj);
void EI2CS_get_status(EI2CS_t *p_obj, uint8_t *p_status);
void EI2CS_Primary_slave_adr_set(EI2CS_t *p_obj, uint8_t adr);
void EI2CS_Primary_slave_10b_adr_set(EI2CS_t *p_obj, uint32_t adr);
void EI2CS_Secondary_slave_adr_set(EI2CS_t *p_obj, uint8_t adr);
void EI2CS_Secondary_slave_10b_adr_set(EI2CS_t *p_obj, uint32_t adr);
#endif
/** 
 *  @}
 */ /* End of group EI2C_Private_functions */

/**
 *  @}
 */ /* End of group EI2C */

/**
 *  @} 
 */ /* End of group Periph_Driver */

#ifdef __cplusplus
}
#endif /* __cplusplus */
#endif /* __EI2C_H */


