/**
 *******************************************************************************
 * @file    cg.c
 * @brief   This file provides API functions for CG driver.
 * @version V1.0.0
 * 
 * DO NOT USE THIS SOFTWARE WITHOUT THE SOFTWARE LICENSE AGREEMENT.
 * 
 * Copyright(C) Toshiba Electronic Device Solutions Corporation 2022
 *******************************************************************************
 */

#ifdef __cplusplus
 extern "C" {
#endif

/*------------------------------------------------------------------------------*/
/*  Includes                                                                    */
/*------------------------------------------------------------------------------*/
#include "cg.h"
/**
 *  @addtogroup Periph_Driver
 *  @{
 */

/** 
 *  @addtogroup CG
 *  @brief      CG Driver.
 *  @{
 */

/*------------------------------------------------------------------------------*/
/*  Macro Definition                                                            */
/*------------------------------------------------------------------------------*/
/** 
 *  @defgroup CG_Private_define CG Private Define
 *  @{
 */
#define CG_GEAR_MASK            ((uint32_t)0x00000007)    /*!< CG GEAR mask */
#define CG_PRCK_MASK            ((uint32_t)0x00000F00)    /*!< CG PRCK mask */



#define CG_WUPHCR_WUCLK_EHOSC   ((uint32_t)0x00000100)    /*!< CG warm up timer clock EHOSC */
#define CG_WUPHCR_WUCLK_IHOSC1  ((uint32_t)0x00000000)    /*!< CG warm up timer clock IHOSC1 */

#define CG_GEAR_SET_MASK        ((uint32_t)0xFFFFFFF8)    /*!< CG FSYS mask for set */
#define CG_FSYS_MASK            ((uint32_t)0x00070000)    /*!< CG FSYS mask */

#define CG_FSYS_1               ((uint32_t)0x00000000)    /*!< CG fc register value */
#define CG_FSYS_2               ((uint32_t)0x00010000)    /*!< CG fc/2 register value */
#define CG_FSYS_4               ((uint32_t)0x00020000)    /*!< CG fc/4 register value */
#define CG_FSYS_8               ((uint32_t)0x00030000)    /*!< CG fc/8 register value */
#define CG_FSYS_16              ((uint32_t)0x00040000)    /*!< CG fc/16 register value */

#define CG_FSYS_1_MUL           ((uint32_t)0x00000001)    /*!< CG fc multiplication value */
#define CG_FSYS_2_MUL           ((uint32_t)0x00000002)    /*!< CG fc/2 multiplication value */
#define CG_FSYS_4_MUL           ((uint32_t)0x00000004)    /*!< CG fc/4 multiplication value */
#define CG_FSYS_8_MUL           ((uint32_t)0x00000008)    /*!< CG fc/8 multiplication value */
#define CG_FSYS_16_MUL          ((uint32_t)0x00000010)    /*!< CG fc/16 multiplication value */

#define CG_PRCK_SET_MASK        ((uint32_t)0xFFFFF0FF)    /*!< CG PRCK mask for set */
#define CG_PRCKST_MASK          ((uint32_t)0x0F000000)    /*!< CG PRCKST mask */

#define CG_PRCKST_1             ((uint32_t)0x00000000)    /*!< CG T0 fc register value */
#define CG_PRCKST_2             ((uint32_t)0x01000000)    /*!< CG T0 fc/2 register value */
#define CG_PRCKST_4             ((uint32_t)0x02000000)    /*!< CG T0 fc/4 register value */
#define CG_PRCKST_8             ((uint32_t)0x03000000)    /*!< CG T0 fc/8 register value */
#define CG_PRCKST_16            ((uint32_t)0x04000000)    /*!< CG T0 fc/16 register value */
#define CG_PRCKST_32            ((uint32_t)0x05000000)    /*!< CG T0 fc/32 register value */
#define CG_PRCKST_64            ((uint32_t)0x06000000)    /*!< CG T0 fc/64 register value */
#define CG_PRCKST_128           ((uint32_t)0x07000000)    /*!< CG T0 fc/128 register value */
#define CG_PRCKST_256           ((uint32_t)0x08000000)    /*!< CG T0 fc/256 register value */
#define CG_PRCKST_512           ((uint32_t)0x09000000)    /*!< CG T0 fc/512 register value */

#define CG_PRCK_1_DIV           ((uint32_t)0x00000001)    /*!< CG T0 fc division value */
#define CG_PRCK_2_DIV           ((uint32_t)0x00000002)    /*!< CG T0 fc/2 division value */
#define CG_PRCK_4_DIV           ((uint32_t)0x00000004)    /*!< CG T0 fc/4 division value */
#define CG_PRCK_8_DIV           ((uint32_t)0x00000008)    /*!< CG T0 fc/8 division value */
#define CG_PRCK_16_DIV          ((uint32_t)0x00000010)    /*!< CG T0 fc/16 division value */
#define CG_PRCK_32_DIV          ((uint32_t)0x00000020)    /*!< CG T0 fc/32 division value */
#define CG_PRCK_64_DIV          ((uint32_t)0x00000040)    /*!< CG T0 fc/64 division value */
#define CG_PRCK_128_DIV         ((uint32_t)0x00000080)    /*!< CG T0 fc/128 division value */
#define CG_PRCK_256_DIV         ((uint32_t)0x00000100)    /*!< CG T0 fc/256 division value */
#define CG_PRCK_512_DIV         ((uint32_t)0x00000200)    /*!< CG T0 fc/512 division value */

#define CG_MCKSEL_SET_MASK      ((uint32_t)0xFFFFFF3F)    /*!< CG MCKSEL mask for set */
#define CG_MCKSELPST_MASK       ((uint32_t)0xC0000000)    /*!< CG MCKSELPST mask */
#define CG_MCKSELGST_MASK       ((uint32_t)0x00C00000)    /*!< CG MCKSELGST mask */

#define CG_EOSCEN_MASK          ((uint32_t)0xFFFFFFF9)    /*!< CG EOSCEN mask */
#define CG_EOSCEN_GET_MASK      ((uint32_t)0x00000006)    /*!< CG EOSCEN get mask */
#define CG_PLL0SET_MASK         ((uint32_t)0x00000007)    /*!< CG PLL0SET mask */
#define CG_PROTECT_ENABLE       ((uint32_t)0x00000000)    /*!< CG register protect enable */
#define CG_PROTECT_DISABLE      ((uint32_t)0x000000C1)    /*!< CG register protect disable */
#define CG_SCOSEL_MASK          ((uint32_t)0xFFFFFFF1)    /*!< CG SCOSEL mask */
#define CG_SCODIV_MASK          ((uint32_t)0xFFFFFF8F)    /*!< CG SCODIV mask */
#define ELOSC_CFG_CLOCK         ((uint32_t)(32768))       /*!< Clock(hz).        */
/** 
 *  @}
 */ /* End of group CG_Private_define */


/*------------------------------------------------------------------------------*/
/*  Private Function                                                            */
/*------------------------------------------------------------------------------*/
/** 
 *  @defgroup CG_Private_fuctions CG Private Fuctions
 *  @{
 */


#ifdef __DEBUG__
__STATIC_INLINE int32_t check_param_gear(uint32_t param);
__STATIC_INLINE int32_t check_param_prck(uint32_t param);
__STATIC_INLINE int32_t check_param_eoscen(uint32_t param);
__STATIC_INLINE int32_t check_param_fcipen(uint32_t param);
__STATIC_INLINE int32_t check_param_spclken(uint32_t param);
__STATIC_INLINE int32_t check_param_mode(uint32_t param);
#if (SCOUT_EQUIPPED == AVAILABLE)
    __STATIC_INLINE int32_t check_param_scosel(uint32_t param);
    __STATIC_INLINE int32_t check_param_scodiv(uint32_t param);
#endif
#if (FSYSM_EQUIPPED == AVAILABLE)
    __STATIC_INLINE int32_t check_param_fsysm(uint32_t param);
#endif
#if (FSYSMENA_EQUIPPED == AVAILABLE)
    __STATIC_INLINE int32_t check_param_ipmena(uint32_t param);
#endif
#if (FSYSMENB_EQUIPPED == AVAILABLE)
    __STATIC_INLINE int32_t check_param_ipmenb(uint32_t param);
#endif
#if (FSYSMENC_EQUIPPED == AVAILABLE)
    __STATIC_INLINE int32_t check_param_ipmenc(uint32_t param);
#endif
#if (FSYSENA_EQUIPPED == AVAILABLE)
    __STATIC_INLINE int32_t check_param_ipena(uint32_t param);
#endif
#if (FSYSENB_EQUIPPED == AVAILABLE)
    __STATIC_INLINE int32_t check_param_ipenb(uint32_t param);
#endif
#if (EXTEND2_EQUIPPED == AVAILABLE)
    __STATIC_INLINE int32_t check_param_extend2(uint32_t param);
#endif


/*--------------------------------------------------*/
/** 
  * @brief  Check the 's parameter.
  * @param  param    :gear's parameter
  * @retval CG_PARAM_OK :Valid
  * @retval CG_PARAM_NG :Invalid
  */
/*--------------------------------------------------*/
__STATIC_INLINE int32_t check_param_gear(uint32_t param)
{
    int32_t result = CG_PARAM_NG;

    switch(param){
        case CG_GEAR_FC_1:
        case CG_GEAR_FC_2:
        case CG_GEAR_FC_4:
        case CG_GEAR_FC_8:
        case CG_GEAR_FC_16:
            result = CG_PARAM_OK;
            break;
        default:
            break;
    }

    return (result);
}

/*--------------------------------------------------*/
/** 
  * @brief  Check the 's parameter.
  * @param  param    :prck's parameter
  * @retval CG_PARAM_OK :Valid
  * @retval CG_PARAM_NG :Invalid
  */
/*--------------------------------------------------*/
__STATIC_INLINE int32_t check_param_prck(uint32_t param)
{
    int32_t result = CG_PARAM_NG;

    switch(param){
        case CG_PRCK_FC_1:
        case CG_PRCK_FC_2:
        case CG_PRCK_FC_4:
        case CG_PRCK_FC_8:
        case CG_PRCK_FC_16:
        case CG_PRCK_FC_32:
        case CG_PRCK_FC_64:
        case CG_PRCK_FC_128:
        case CG_PRCK_FC_256:
        case CG_PRCK_FC_512:
            result = CG_PARAM_OK;
            break;
        default:
            break;
    }

    return (result);
}

/*--------------------------------------------------*/
/** 
  * @brief  Check the 's parameter.
  * @param  param    :eoscen's parameter
  * @retval CG_PARAM_OK :Valid
  * @retval CG_PARAM_NG :Invalid
  */
/*--------------------------------------------------*/
__STATIC_INLINE int32_t check_param_eoscen(uint32_t param)
{
    int32_t result = CG_PARAM_NG;

    switch(param){
        case CG_EHOSCEN_UNUSE:
        case CG_EHOSCEN_EHOSC:
        case CG_EHOSCEN_EHOCLKIN:
            result = CG_PARAM_OK;
            break;
        default:
            break;
    }

    return (result);
}

/*--------------------------------------------------*/
/** 
  * @brief  Check the 's parameter.
  * @param  param    :fcipen's parameter
  * @retval CG_PARAM_OK :Valid
  * @retval CG_PARAM_NG :Invalid
  */
/*--------------------------------------------------*/
__STATIC_INLINE int32_t check_param_fcipen(uint32_t param)
{
    int32_t result = CG_PARAM_NG;

    if(param & CG_FCEN_ALL_ENABLE){
        result = CG_PARAM_OK;
    }

    return (result);
}

/*--------------------------------------------------*/
/** 
  * @brief  Check the 's parameter.
  * @param  param    :spclken's parameter
  * @retval CG_PARAM_OK :Valid
  * @retval CG_PARAM_NG :Invalid
  */
/*--------------------------------------------------*/
__STATIC_INLINE int32_t check_param_spclken(uint32_t param)
{
    int32_t result = CG_PARAM_NG;

    if(param & CG_SPCLKEN_ALL_ENABLE){
        result = CG_PARAM_OK;
    }

    return (result);
}

/*--------------------------------------------------*/
/** 
  * @brief  Check the 's parameter.
  * @param  param    :mode's parameter
  * @retval CG_PARAM_OK :Valid
  * @retval CG_PARAM_NG :Invalid
  */
/*--------------------------------------------------*/
__STATIC_INLINE int32_t check_param_mode(uint32_t param)
{
    int32_t result = CG_PARAM_NG;

    switch(param){
        case CG_STBY_IDLE:
        case CG_STBY_STOP1:
#if (STOP2_EQUIPPED == AVAILABLE)
        case CG_STBY_STOP2:
#endif
            result = CG_PARAM_OK;
            break;
        default:
            break;
    }

    return (result);
}

#if (SCOUT_EQUIPPED == AVAILABLE)
__STATIC_INLINE int32_t check_param_scosel(uint32_t param)
{
    int32_t result = CG_PARAM_NG;

    switch(param){
        case CG_SCOCR_SCOSEL_FOSC:
        case CG_SCOCR_SCOSEL_FC:
        case CG_SCOCR_SCOSEL_FS:
        case CG_SCOCR_SCOSEL_FSYS:
            result = CG_PARAM_OK;
            break;
        default:
            break;
    }

    return (result);
}
__STATIC_INLINE int32_t check_param_scodiv(uint32_t param)
{
    int32_t result = CG_PARAM_NG;

    switch(param){
        case CG_SCOCR_SCODIV_NONE:
        case CG_SCOCR_SCODIV_2:
        case CG_SCOCR_SCODIV_4:
        case CG_SCOCR_SCODIV_8:
        case CG_SCOCR_SCODIV_16:
            result = CG_PARAM_OK;
            break;
        default:
            break;
    }

    return (result);
}
#endif

#if (FSYSM_EQUIPPED == AVAILABLE)
__STATIC_INLINE int32_t check_param_fsysm(uint32_t param)
{
    int32_t result = CG_PARAM_NG;

    switch(param){
        case CG_MCKSEL_1:
        case CG_MCKSEL_2:
        case CG_MCKSEL_4:
            result = CG_PARAM_OK;
            break;
        default:
            break;
    }

    return (result);
}
#endif
#if (FSYSMENA_EQUIPPED == AVAILABLE)
/*--------------------------------------------------*/
/** 
  * @brief  Check the 's parameter.
  * @param  param    :IPMENA's parameter
  * @retval CG_PARAM_OK :Valid
  * @retval CG_PARAM_NG :Invalid
  */
/*--------------------------------------------------*/
__STATIC_INLINE int32_t check_param_ipmena(uint32_t param)
{
    int32_t result = CG_PARAM_NG;

    if(param & CG_FSYSMENA_ALL_ENABLE){
        result = CG_PARAM_OK;
    }

    return (result);
}
#endif

#if (FSYSMENB_EQUIPPED == AVAILABLE)
/*--------------------------------------------------*/
/** 
  * @brief  Check the 's parameter.
  * @param  param    :IPMENB's parameter
  * @retval CG_PARAM_OK :Valid
  * @retval CG_PARAM_NG :Invalid
  */
/*--------------------------------------------------*/
__STATIC_INLINE int32_t check_param_ipmenb(uint32_t param)
{
    int32_t result = CG_PARAM_NG;

    if(param & CG_FSYSMENB_ALL_ENABLE){
        result = CG_PARAM_OK;
    }

    return (result);
}
#endif

#if (FSYSMENC_EQUIPPED == AVAILABLE)
/*--------------------------------------------------*/
/** 
  * @brief  Check the 's parameter.
  * @param  param    :IPMENC's parameter
  * @retval CG_PARAM_OK :Valid
  * @retval CG_PARAM_NG :Invalid
  */
/*--------------------------------------------------*/
__STATIC_INLINE int32_t check_param_ipmenc(uint32_t param)
{
    int32_t result = CG_PARAM_NG;

    if(param & CG_FSYSMENC_ALL_ENABLE){
        result = CG_PARAM_OK;
    }

    return (result);
}
#endif

#if (FSYSENA_EQUIPPED == AVAILABLE)
/*--------------------------------------------------*/
/** 
  * @brief  Check the 's parameter.
  * @param  param    :IPENA's parameter
  * @retval CG_PARAM_OK :Valid
  * @retval CG_PARAM_NG :Invalid
  */
/*--------------------------------------------------*/
__STATIC_INLINE int32_t check_param_ipena(uint32_t param)
{
    int32_t result = CG_PARAM_NG;

    if(param & CG_FSYSENA_ALL_ENABLE){
        result = CG_PARAM_OK;
    }

    return (result);
}
#endif

#if (FSYSENB_EQUIPPED == AVAILABLE)
/*--------------------------------------------------*/
/** 
  * @brief  Check the 's parameter.
  * @param  param    :IPENB's parameter
  * @retval CG_PARAM_OK :Valid
  * @retval CG_PARAM_NG :Invalid
  */
/*--------------------------------------------------*/
__STATIC_INLINE int32_t check_param_ipenb(uint32_t param)
{
    int32_t result = CG_PARAM_NG;

    if(param & CG_FSYSENB_ALL_ENABLE){
        result = CG_PARAM_OK;
    }

    return (result);
}
#endif

#if (EXTEND2_EQUIPPED == AVAILABLE)
/*--------------------------------------------------*/
/** 
  * @brief  Check the 's parameter.
  * @param  param    :EXTEND2's parameter
  * @retval CG_PARAM_OK :Valid
  * @retval CG_PARAM_NG :Invalid
  */
/*--------------------------------------------------*/
__STATIC_INLINE int32_t check_param_extend2(uint32_t param)
{
    int32_t result = CG_PARAM_NG;

    switch(param){
        case CG_EXTEND2_RSV22:
        case CG_EXTEND2_RSV21:
        case CG_EXTEND2_RSV20:
            result = CG_PARAM_OK;
            break;
        default:
            break;
    }

    return (result);
}
#endif
#endif /* __DEBUG__ */

/**
 *  @}
 */ /* End of group CG_Private_functions */


/*------------------------------------------------------------------------------*/
/*  Public Function                                                             */
/*------------------------------------------------------------------------------*/
/** 
 *  @defgroup CG_Exported_functions CG Exported Functions
 *  @{
 */

/*--------------------------------------------------*/
/**
  * @brief     Set Gear according register values.
  * @param     p_obj : CG object.
  * @param     gear  : Gear Frequency.
  * @retval    -
  * @note      -
  * @attention This function is not available in interrupt.
  */
/*--------------------------------------------------*/
void cg_set_gear(cg_t *p_obj, uint32_t gear)
{
    uint32_t regval = 0;

    /*------------------------------*/
    /*  Parameter Check             */
    /*------------------------------*/
#ifdef __DEBUG__
    /* Check the CG_NULL of address. */
    assert_param(IS_POINTER_NOT_NULL(p_obj));
    assert_param(IS_POINTER_NOT_NULL(p_obj->p_instance));
    assert_param(check_param_gear(gear));
#endif /* #ifdef __DEBUG__ */

    /* Set Gear status. */
    regval = p_obj->p_instance->SYSCR & CG_GEAR_SET_MASK;
    regval |= gear;
    p_obj->p_instance->SYSCR = regval;
}

/*--------------------------------------------------*/
/**
  * @brief     Update Gear according register values.
  * @param     p_obj       :CG object.
  * @retval    Gear Frequency.
  * @note      -
  * @attention This function is not available in interrupt.
  */
/*--------------------------------------------------*/
uint32_t cg_get_gear(cg_t *p_obj)
{
    uint32_t result = 0U;

    /*------------------------------*/
    /*  Parameter Check             */
    /*------------------------------*/
#ifdef __DEBUG__
    /* Check the CG_NULL of address. */
    assert_param(IS_POINTER_NOT_NULL(p_obj));
    assert_param(IS_POINTER_NOT_NULL(p_obj->p_instance));
#endif /* #ifdef __DEBUG__ */

    /* System core clock update */
    SystemCoreClockUpdate();

    /* Get Gear status. */
    switch (p_obj->p_instance->SYSCR & CG_FSYS_MASK)
    {
    case CG_FSYS_1:             /* Gear -> fc */
        result = SystemCoreClock * CG_FSYS_1_MUL;
        break;
    case CG_FSYS_2:             /* Gear -> fc/2 */
        result = SystemCoreClock * CG_FSYS_2_MUL;
        break;
    case CG_FSYS_4:             /* Gear -> fc/4 */
        result = SystemCoreClock * CG_FSYS_4_MUL;
        break;
    case CG_FSYS_8:             /* Gear -> fc/8 */
        result = SystemCoreClock * CG_FSYS_8_MUL;
        break;
    case CG_FSYS_16:            /* Gear -> fc/16 */
        result = SystemCoreClock * CG_FSYS_16_MUL;
        break;
    default:
        result = 0U;
        break;
    }

    return (result);
}

/*--------------------------------------------------*/
/**
  * @brief     Set PrescalerClock according register values.
  * @param     p_obj : CG object.
  * @param     prck  : PrescalerClock Frequency.
  * @retval    -
  * @note      -
  * @attention This function is not available in interrupt.
  */
/*--------------------------------------------------*/
void cg_set_phyt0(cg_t *p_obj, uint32_t prck)
{
    uint32_t regval = 0;

    /*------------------------------*/
    /*  Parameter Check             */
    /*------------------------------*/
#ifdef __DEBUG__
    /* Check the CG_NULL of address. */
    assert_param(IS_POINTER_NOT_NULL(p_obj));
    assert_param(IS_POINTER_NOT_NULL(p_obj->p_instance));
    assert_param(check_param_prck(prck));
#endif /* #ifdef __DEBUG__ */

    /* Set PrescalerClock status. */
    regval = p_obj->p_instance->SYSCR & CG_PRCK_SET_MASK;
    regval |= prck;
    p_obj->p_instance->SYSCR = regval;
}

/*--------------------------------------------------*/
/**
  * @brief     Update PrescalerClock according register values.
  * @param     p_obj       :CG object.
  * @retval    PrescalerClock Frequency.
  * @note      -
  * @attention This function is not available in interrupt.
  */
/*--------------------------------------------------*/
uint32_t cg_get_phyt0(cg_t *p_obj)
{
    uint32_t result = 0U;

    /*------------------------------*/
    /*  Parameter Check             */
    /*------------------------------*/
#ifdef __DEBUG__
    /* Check the CG_NULL of address. */
    assert_param(IS_POINTER_NOT_NULL(p_obj));
    assert_param(IS_POINTER_NOT_NULL(p_obj->p_instance));
#endif /* #ifdef __DEBUG__ */

    /* System core clock update */
    SystemCoreClockUpdate();

    /* Get Gear status. */
    switch (p_obj->p_instance->SYSCR & CG_FSYS_MASK)
    {
    case CG_FSYS_1:             /* Gear -> fc */
        result = SystemCoreClock * CG_FSYS_1_MUL;
        break;
    case CG_FSYS_2:             /* Gear -> fc/2 */
        result = SystemCoreClock * CG_FSYS_2_MUL;
        break;
    case CG_FSYS_4:             /* Gear -> fc/4 */
        result = SystemCoreClock * CG_FSYS_4_MUL;
        break;
    case CG_FSYS_8:             /* Gear -> fc/8 */
        result = SystemCoreClock * CG_FSYS_8_MUL;
        break;
    case CG_FSYS_16:            /* Gear -> fc/16 */
        result = SystemCoreClock * CG_FSYS_16_MUL;
        break;
    default:
        result = 0U;
        break;
    }

    switch (p_obj->p_instance->SYSCR & CG_PRCKST_MASK)
    {
    case CG_PRCKST_1:             /* T0 -> fc */
        result /= CG_PRCK_1_DIV;
        break;
    case CG_PRCKST_2:             /* T0 -> fc/2 */
        result /= CG_PRCK_2_DIV;
        break;
    case CG_PRCKST_4:             /* T0 -> fc/4 */
        result /= CG_PRCK_4_DIV;
        break;
    case CG_PRCKST_8:             /* T0 -> fc/8 */
        result /= CG_PRCK_8_DIV;
        break;
    case CG_PRCKST_16:            /* T0 -> fc/16 */
        result /= CG_PRCK_16_DIV;
        break;
    case CG_PRCKST_32:            /* T0 -> fc/32 */
        result /= CG_PRCK_32_DIV;
        break;
    case CG_PRCKST_64:            /* T0 -> fc/64 */
        result /= CG_PRCK_64_DIV;
        break;
    case CG_PRCKST_128:           /* T0 -> fc/128 */
        result /= CG_PRCK_128_DIV;
        break;
    case CG_PRCKST_256:           /* T0 -> fc/256 */
        result /= CG_PRCK_256_DIV;
        break;
    case CG_PRCKST_512:           /* T0 -> fc/512 */
        result /= CG_PRCK_512_DIV;
        break;
    default:
        result = 0U;
        break;
    }

    return (result);
}

/*--------------------------------------------------*/
/**
  * @brief     Get PRCKST.
  * @param     p_obj       :CG object.
  * @retval    PRCKST.
  * @note      -
  * @attention This function is not available in interrupt.
  */
/*--------------------------------------------------*/
uint32_t cg_get_prckst(cg_t *p_obj)
{
    uint32_t result = 0U;

    /*------------------------------*/
    /*  Parameter Check             */
    /*------------------------------*/
#ifdef __DEBUG__
    /* Check the CG_NULL of address. */
    assert_param(IS_POINTER_NOT_NULL(p_obj));
    assert_param(IS_POINTER_NOT_NULL(p_obj->p_instance));
#endif /* #ifdef __DEBUG__ */

    /* Get PRCKST status. */
    result = (p_obj->p_instance->SYSCR & CG_PRCKST_MASK);
    return (result);
}

/*--------------------------------------------------*/
/**
  * @brief  Set the warm up time.
  * @param  p_obj : CG object.
  * @param  source: Set base clock for warm-up timer
  * @param  time  : Set count time for warm-up timer
  * @retval -
  * @attention This function is not available in interrupt.
  */
/*--------------------------------------------------*/
void cg_set_warm_up_time(cg_t *p_obj, uint32_t source, uint32_t time)
{
    uint32_t regval = 0;

    /*------------------------------*/
    /*  Parameter Check             */
    /*------------------------------*/
#ifdef __DEBUG__
    /* Check the CG_NULL of address. */
    assert_param(IS_POINTER_NOT_NULL(p_obj));
    assert_param(IS_POINTER_NOT_NULL(p_obj->p_instance));
#endif /* #ifdef __DEBUG__ */

    if (source == CG_WARM_UP_SRC_EHOSC) {
        regval = ((uint32_t)(((((uint64_t)time * EXTALH / HZ_1M) - 16UL) /16UL) << 20U));
        regval |= CG_WUPHCR_WUCLK_EHOSC;
    } else if (source == CG_WARM_UP_SRC_IHOSC1 ) {
        regval = ((uint32_t)(((((uint64_t)(time - 633UL) * (IXTALH / 10) / HZ_1M) - 41UL) /16UL) << 20U));
        if(regval < WU_TIME_INT_MIN)
        {
            regval = WU_TIME_INT_MIN;
        }
        else
        {
            regval = WUPHCR_WUPT_INT;
        }
        regval |= CG_WUPHCR_WUCLK_IHOSC1;
    }

    p_obj->p_instance->WUPHCR = regval;
}

/*--------------------------------------------------*/
/**
  * @brief Check whether warm up is completed or not.
  * @param  -
  * @retval    TXZ_DONE :Done.
  * @retval    TXZ_BUSY :Busy.
  * @attention This function is not available in interrupt.
  */
/*--------------------------------------------------*/
TXZ_WorkState cg_get_warm_up_state(void)
{
    TXZ_WorkState state = TXZ_BUSY;

    if (TSB_CG_WUPHCR_WUEF == 0U) {
        state = TXZ_DONE;
    } else {
        /* Do nothing */
    }

    return state;
}

/*--------------------------------------------------*/
/**
  * @brief  Start operation of warm up timer for oscillator.
  * @param  -
  * @retval -
  * @attention This function is not available in interrupt.
  */
/*--------------------------------------------------*/
void cg_start_warmup(void)
{
    TSB_CG_WUPHCR_WUON = 1U;
}

/*--------------------------------------------------*/
/**
  * @brief  Wait operation of warm up timer for oscillator.
  * @param  -
  * @retval -
  * @attention This function is not available in interrupt.
  */
/*--------------------------------------------------*/
void cg_wait_warmup(void)
{
    /* [CGWUPHCR]<WUEF>  : Read(wait for done) */
    while( TSB_CG_WUPHCR_WUEF != CGWUPHCR_WUEF_R_DONE )
    {
        /* no processing */
    }
}

/*--------------------------------------------------*/
/**
  * @brief     Set External high-speed oscillator select.
  * @param     p_obj       :CG object.
  * @param     eoscen      :Set External high-speed oscillator select.
  * @retval    -
  * @note      -
  * @attention This function is not available in interrupt.
  */
/*--------------------------------------------------*/
void cg_set_ehoscen(cg_t *p_obj, uint32_t eoscen)
{
    uint32_t regval = 0;

    /*------------------------------*/
    /*  Parameter Check             */
    /*------------------------------*/
#ifdef __DEBUG__
    /* Check the CG_NULL of address. */
    assert_param(IS_POINTER_NOT_NULL(p_obj));
    assert_param(IS_POINTER_NOT_NULL(p_obj->p_instance));
    assert_param(check_param_eoscen(eoscen));
#endif /* #ifdef __DEBUG__ */

    /* Set External high-speed oscillator select. */
    regval = p_obj->p_instance->OSCCR & CG_EOSCEN_MASK;
    regval |= eoscen;
    p_obj->p_instance->OSCCR = regval;
}

/*--------------------------------------------------*/
/**
  * @brief     Get External high-speed oscillator select.
  * @param     p_obj       :CG object.
  * @retval    CG_EHOSCEN_UNUSE, CG_EHOSCEN_EHOSC, CG_EHOSCEN_EHOCLKIN
  * @note      -
  * @attention This function is not available in interrupt.
  */
/*--------------------------------------------------*/
uint32_t cg_get_ehoscen(cg_t *p_obj)
{
    uint32_t retval = 0;

    /*------------------------------*/
    /*  Parameter Check             */
    /*------------------------------*/
#ifdef __DEBUG__
    /* Check the CG_NULL of address. */
    assert_param(IS_POINTER_NOT_NULL(p_obj));
    assert_param(IS_POINTER_NOT_NULL(p_obj->p_instance));
#endif /* #ifdef __DEBUG__ */

    /* Get External high-speed oscillator select. */
    retval = (p_obj->p_instance->OSCCR & CG_EOSCEN_GET_MASK);
    return (retval);
}

/*--------------------------------------------------*/
/**
  * @brief  Set the fosc base clock.
  * @param  source: Set base clock for fosc
  * @retval -
  * @attention This function is not available in interrupt.
  */
/*--------------------------------------------------*/
void cg_set_oscsel(uint32_t source)
{
    if (source == CG_OSCSEL_EHOSC) {
        TSB_CG_OSCCR_OSCSEL = 1U;
    } else {
        TSB_CG_OSCCR_OSCSEL = 0U;
    }
}

/*--------------------------------------------------*/
/**
  * @brief Check whether oscsel.
  * @param  -
  * @retval CG_OSCSEL_EHOSC  :EHOSC.
  * @retval CG_OSCSEL_IHOSC1 :IHOSC1.
  * @attention This function is not available in interrupt.
  */
/*--------------------------------------------------*/
uint32_t cg_get_oscsel(void)
{
    uint32_t retval = 0;

    if (TSB_CG_OSCCR_OSCF == 1U) {
        retval = CG_OSCSEL_EHOSC;
    } else {
        /* Do nothing */
    }

    return retval;
}

/*--------------------------------------------------*/
/**
  * @brief     Set Internal high-speed oscillator1 enable.
  * @param     -
  * @retval    TXZ_SUCCESS :Success.
  * @retval    TXZ_ERROR   :Failure.
  * @note      -
  * @attention This function is not available in interrupt.
  */
/*--------------------------------------------------*/
void cg_ihosc1_enable(void)
{
    /* Internal high-speed oscillator1 is enable. */
    TSB_CG_OSCCR_IHOSC1EN = 1U;
}

/*--------------------------------------------------*/
/**
  * @brief     Set Internal high-speed oscillator1 disable.
  * @param     -
  * @retval    TXZ_SUCCESS :Success.
  * @retval    TXZ_ERROR   :Failure.
  * @note      -
  * @attention This function is not available in interrupt.
  */
/*--------------------------------------------------*/
void cg_ihosc1_disable(void)
{
    /* Internal high-speed oscillator1 is disable. */
    TSB_CG_OSCCR_IHOSC1EN = 0U;
}

/*--------------------------------------------------*/
/**
  * @brief Check whether ihosc1 state.
  * @param  -
  * @retval    TXZ_DONE :Done.
  * @retval    TXZ_BUSY :Busy.
  * @attention This function is not available in interrupt.
  */
/*--------------------------------------------------*/
TXZ_WorkState cg_get_ihosc1_state(void)
{
    TXZ_WorkState state = TXZ_BUSY;

    if (TSB_CG_OSCCR_IHOSC1F == 1U) {
        state = TXZ_DONE;
    } else {
        /* Do nothing */
    }

    return state;
}

/*--------------------------------------------------*/
/**
  * @brief     Set PLL according register values.
  * @param     p_obj  : CG object.
  * @param     PLLset : PLLset.
  * @retval    -
  * @note      -
  * @attention This function is not available in interrupt.
  */
/*--------------------------------------------------*/
void cg_set_PLL(cg_t *p_obj, uint32_t PLLset)
{
    uint32_t regval = 0;

    /*------------------------------*/
    /*  Parameter Check             */
    /*------------------------------*/
#ifdef __DEBUG__
    /* Check the CG_NULL of address. */
    assert_param(IS_POINTER_NOT_NULL(p_obj));
    assert_param(IS_POINTER_NOT_NULL(p_obj->p_instance));
#endif /* #ifdef __DEBUG__ */

    /* Set PLLset value. */
    regval = p_obj->p_instance->PLL0SEL & CG_PLL0SET_MASK;
    regval |= PLLset;
    p_obj->p_instance->PLL0SEL = regval;
}

/*--------------------------------------------------*/
/**
  * @brief     Set PLL enable.
  * @param     -
  * @retval    -
  * @note      -
  * @attention This function is not available in interrupt.
  */
/*--------------------------------------------------*/
void cg_PLL_enable(void)
{
    /* PLL is enable. */
    TSB_CG_PLL0SEL_PLL0ON = 1U;
}

/*--------------------------------------------------*/
/**
  * @brief     Set PLL disable.
  * @param     -
  * @retval    -
  * @note      -
  * @attention This function is not available in interrupt.
  */
/*--------------------------------------------------*/
void cg_PLL_disable(void)
{
    /* PLL is disable. */
    TSB_CG_PLL0SEL_PLL0ON = 0U;
}

/*--------------------------------------------------*/
/**
  * @brief     Set PLL base clock.
  * @param     p_obj  : CG object.
  * @param     clock  : PLL base clock.
  * @retval    -
  * @note      -
  * @attention This function is not available in interrupt.
  */
/*--------------------------------------------------*/
void cg_set_PLL_clock(uint32_t clock)
{
    if (clock == CG_PLL0SEL_FPLL) {
        TSB_CG_PLL0SEL_PLL0SEL = 1U;
    } else {
        TSB_CG_PLL0SEL_PLL0SEL = 0U;
    }
}

/*--------------------------------------------------*/
/**
  * @brief Check whether PLL0SEL state.
  * @param  -
  * @retval    CG_FPLL :PLL use.
  * @retval    CG_FOSC :PLL unuse.
  * @attention This function is not available in interrupt.
  */
/*--------------------------------------------------*/
uint32_t cg_get_PLL_status(void)
{
    uint32_t retval = 0;

    if (TSB_CG_PLL0SEL_PLL0ST == 1U) {
        retval = CG_PLL0SEL_FPLL;
    } else {
        /* Do nothing */
    }

    return retval;
}

/*--------------------------------------------------*/
/**
  * @brief     Set STBYmode according register values.
  * @param     p_obj :CG object.
  * @param     mode  :STBYmode.
  * @retval    -
  * @note      -
  * @attention This function is not available in interrupt.
  */
/*--------------------------------------------------*/
void cg_set_stby_mode(cg_t *p_obj, uint32_t mode)
{
    /*------------------------------*/
    /*  Parameter Check             */
    /*------------------------------*/
#ifdef __DEBUG__
    /* Check the CG_NULL of address. */
    assert_param(IS_POINTER_NOT_NULL(p_obj));
    assert_param(IS_POINTER_NOT_NULL(p_obj->p_instance));
    assert_param(check_param_mode(mode));
#endif /* #ifdef __DEBUG__ */

    /* Set STBYmode status. */
    p_obj->p_instance->STBYCR = mode;
}

/*--------------------------------------------------*/
/**
  * @brief     Get STBYmode register values.
  * @param     p_obj       :CG object.
  * @retval    STBYmode.
  * @note      -
  * @attention This function is not available in interrupt.
  */
/*--------------------------------------------------*/
uint32_t cg_get_stby_mode(cg_t *p_obj)
{
    uint32_t result = 0U;

    /*------------------------------*/
    /*  Parameter Check             */
    /*------------------------------*/
#ifdef __DEBUG__
    /* Check the CG_NULL of address. */
    assert_param(IS_POINTER_NOT_NULL(p_obj));
    assert_param(IS_POINTER_NOT_NULL(p_obj->p_instance));
#endif /* #ifdef __DEBUG__ */

    /* Get STBYmode status. */
    result = p_obj->p_instance->STBYCR;

    return (result);
}

/*--------------------------------------------------*/
/**
  * @brief     Set CG protect enable.
  * @param     p_obj       :CG object.
  * @retval    -
  * @note      -
  * @attention This function is not available in interrupt.
  */
/*--------------------------------------------------*/
void cg_protect_enable(cg_t *p_obj)
{
    /*------------------------------*/
    /*  Parameter Check             */
    /*------------------------------*/
#ifdef __DEBUG__
    /* Check the CG_NULL of address. */
    assert_param(IS_POINTER_NOT_NULL(p_obj));
    assert_param(IS_POINTER_NOT_NULL(p_obj->p_instance));
#endif /* #ifdef __DEBUG__ */

    /* CG protect is enable. */
    p_obj->p_instance->PROTECT = CG_PROTECT_ENABLE;
}

/*--------------------------------------------------*/
/**
  * @brief     Set CG protect disable.
  * @param     p_obj       :CG object.
  * @retval    -
  * @note      -
  * @attention This function is not available in interrupt.
  */
/*--------------------------------------------------*/
void cg_protect_disable(cg_t *p_obj)
{
    /*------------------------------*/
    /*  Parameter Check             */
    /*------------------------------*/
#ifdef __DEBUG__
    /* Check the CG_NULL of address. */
    assert_param(IS_POINTER_NOT_NULL(p_obj));
    assert_param(IS_POINTER_NOT_NULL(p_obj->p_instance));
#endif /* #ifdef __DEBUG__ */

    /* CG protect is disable. */
    p_obj->p_instance->PROTECT = CG_PROTECT_DISABLE;
}

/*--------------------------------------------------*/
/**
  * @brief     Get CG system core clock.
  * @param     p_obj       :CG object.
  * @retval    -
  * @note      -
  * @attention This function is not available in interrupt.
  */
/*--------------------------------------------------*/
void cg_get_system_core_clock(cg_t *p_obj)
{
    uint32_t core_clock = 0U;
    uint32_t core_clock_input = 0U;
    uint32_t regval = 0U;
    uint32_t oscsel = 0U;
    uint32_t pll0sel = 0U;
    uint32_t pll0on = 0U;
    uint32_t pll0setval = 0U;
    uint32_t i = 0U;

    /*------------------------------*/
    /*  Parameter Check             */
    /*------------------------------*/
#ifdef __DEBUG__
    /* Check the CG_NULL of address. */
    assert_param(IS_POINTER_NOT_NULL(p_obj));
    assert_param(IS_POINTER_NOT_NULL(p_obj->p_instance));
#endif /* #ifdef __DEBUG__ */

    /* Determine clock frequency according to clock register values */
    /* System clock is high-speed clock */
    regval = p_obj->p_instance->OSCCR;
    oscsel = regval & CG_OSCSEL_EHOSC;
    if (oscsel) {  /* If system clock is External high-speed oscillator freq */
        core_clock = EXTALH;
    } else {                    /* If system clock is Internal high-speed oscillator freq */
        core_clock = IXTALH;
    }
    regval = p_obj->p_instance->PLL0SEL;
    pll0sel = regval & CG_PLL0SEL_FPLL;
    pll0on = regval & CG_PLL0SEL_PLL0ON_SET;
    if (pll0sel && pll0on) {                        /* If PLL enabled  */
        pll0setval = p_obj->p_instance->PLL0SEL & PLL0SEL_MASK;
        for (i = 0; i < SYSTEM_CORE_CLOCK_TABLE_MAX; i++) {
            if((core_clock == system_core_clock_table[i][0]) && (pll0setval == system_core_clock_table[i][1])){
                core_clock_input = system_core_clock_table[i][2];
            }
        }
    } else {                                       /* If PLL not used */
        core_clock_input = core_clock;
    }

    switch (p_obj->p_instance->SYSCR & 7U) {
    case 0U:                                 /* Gear -> fc */
        SystemCoreClock = core_clock_input;
        break;
    case 1U:                                 /* Gear -> fc/2 */
        SystemCoreClock = core_clock_input / 2U;
        break;
    case 2U:                                 /* Gear -> fc/4 */
        SystemCoreClock = core_clock_input / 4U;
        break;
    case 3U:                                 /* Gear -> fc/8 */
        if (core_clock_input >= EOSC_8M) {
            SystemCoreClock = core_clock_input / 8U;
        } else {
            SystemCoreClock = 0U;
        }
        break;
    case 4U:                                 /* Gear -> fc/16 */
        if (core_clock_input > EOSC_12M) {
            SystemCoreClock = core_clock_input / 16U;
        } else {
            SystemCoreClock = 0U;
        }
        break;
    case 5U:
    case 6U:
    case 7U:
        SystemCoreClock = 0U;
        break;
    default:
        SystemCoreClock = 0U;
        break;
    }
}

/*--------------------------------------------------*/
/**
  * @brief     Set CG FCEN.
  * @param     p_obj       :CG object.
  * @param     ipmena      :Set fcen register
  * @param     state       :TXZ_ENABLE or TXZ_DISABLE
  * @retval    -
  * @note      -
  * @attention This function is not available in interrupt.
  */
/*--------------------------------------------------*/
void cg_set_fcen(cg_t *p_obj, uint32_t fcipen, TXZ_FunctionalState state)
{
    /*------------------------------*/
    /*  Parameter Check             */
    /*------------------------------*/
#ifdef __DEBUG__
    /* Check the CG_NULL of address. */
    assert_param(IS_POINTER_NOT_NULL(p_obj));
    assert_param(IS_POINTER_NOT_NULL(p_obj->p_instance));
    assert_param(check_param_fcipen(fcipen));
#endif /* #ifdef __DEBUG__ */

    uint32_t regval = p_obj->p_instance->FCEN;

    if (state == TXZ_ENABLE) {
        regval |= (fcipen & CG_FCEN_ALL_ENABLE);
    } else {
        regval &= ~(fcipen & CG_FCEN_ALL_ENABLE);
    }
    p_obj->p_instance->FCEN = regval;
}

/*--------------------------------------------------*/
/**
  * @brief     Set CG ALL FCEN.
  * @param     p_obj       :CG object.
  * @param     state       :TXZ_ENABLE or TXZ_DISABLE
  * @retval    -
  * @note      -
  * @attention This function is not available in interrupt.
  */
/*--------------------------------------------------*/
void cg_set_all_fcen(cg_t *p_obj, TXZ_FunctionalState state)
{
    /*------------------------------*/
    /*  Parameter Check             */
    /*------------------------------*/
#ifdef __DEBUG__
    /* Check the CG_NULL of address. */
    assert_param(IS_POINTER_NOT_NULL(p_obj));
    assert_param(IS_POINTER_NOT_NULL(p_obj->p_instance));
#endif /* #ifdef __DEBUG__ */

    uint32_t regval = p_obj->p_instance->FCEN;

    if (state == TXZ_ENABLE) {
        regval = CG_FCEN_ALL_ENABLE;
    } else {
        regval = CG_FCEN_ALL_DISABLE;
    }
    p_obj->p_instance->FCEN = regval;
}

/*--------------------------------------------------*/
/**
  * @brief     Set CG SPCLKEN.
  * @param     p_obj       :CG object.
  * @param     ipmena      :Set spclken register
  * @param     state       :TXZ_ENABLE or TXZ_DISABLE
  * @retval    -
  * @note      -
  * @attention This function is not available in interrupt.
  */
/*--------------------------------------------------*/
void cg_set_spclken(cg_t *p_obj, uint32_t spclken, TXZ_FunctionalState state)
{
    /*------------------------------*/
    /*  Parameter Check             */
    /*------------------------------*/
#ifdef __DEBUG__
    /* Check the CG_NULL of address. */
    assert_param(IS_POINTER_NOT_NULL(p_obj));
    assert_param(IS_POINTER_NOT_NULL(p_obj->p_instance));
    assert_param(check_param_spclken(spclken));
#endif /* #ifdef __DEBUG__ */

    uint32_t regval = p_obj->p_instance->SPCLKEN;

    if (state == TXZ_ENABLE) {
        regval |= (spclken & CG_SPCLKEN_ALL_ENABLE);
    } else {
        regval &= ~(spclken & CG_SPCLKEN_ALL_ENABLE);
    }
    p_obj->p_instance->SPCLKEN = regval;
}

/*--------------------------------------------------*/
/**
  * @brief     Set CG ALL SPCLKEN.
  * @param     p_obj       :CG object.
  * @param     state       :TXZ_ENABLE or TXZ_DISABLE
  * @retval    -
  * @note      -
  * @attention This function is not available in interrupt.
  */
/*--------------------------------------------------*/
void cg_set_all_spclken(cg_t *p_obj, TXZ_FunctionalState state)
{
    /*------------------------------*/
    /*  Parameter Check             */
    /*------------------------------*/
#ifdef __DEBUG__
    /* Check the CG_NULL of address. */
    assert_param(IS_POINTER_NOT_NULL(p_obj));
    assert_param(IS_POINTER_NOT_NULL(p_obj->p_instance));
#endif /* #ifdef __DEBUG__ */

    uint32_t regval = p_obj->p_instance->SPCLKEN;

    if (state == TXZ_ENABLE) {
        regval = CG_SPCLKEN_ALL_ENABLE;
    } else {
        regval = CG_SPCLKEN_ALL_DISABLE;
    }
    p_obj->p_instance->SPCLKEN = regval;
}



/*------------------------------------------------------------------------------*/
/*  optional function                                                           */
/*------------------------------------------------------------------------------*/

/*------------------------------------------------------------------------------*/
/*  midium speed clock function                                                 */
/*------------------------------------------------------------------------------*/
#if (FSYSM_EQUIPPED == AVAILABLE)
/*--------------------------------------------------*/
/**
  * @brief     Set Middle PrescalerClock according register values.
  * @param     p_obj : CG object.
  * @param     fsysm : Middle PrescalerClock Frequency.
  * @retval    -
  * @note      -
  * @attention This function is not available in interrupt.
  */
/*--------------------------------------------------*/
void cg_set_fsysm(cg_t *p_obj, uint32_t fsysm)
{
    uint32_t regval = 0;

    /*------------------------------*/
    /*  Parameter Check             */
    /*------------------------------*/
#ifdef __DEBUG__
    /* Check the CG_NULL of address. */
    assert_param(IS_POINTER_NOT_NULL(p_obj));
    assert_param(IS_POINTER_NOT_NULL(p_obj->p_instance));
    assert_param(check_param_fsysm(fsysm));
#endif /* #ifdef __DEBUG__ */

    /* Set Gear status. */
    regval = p_obj->p_instance->SYSCR & CG_MCKSEL_SET_MASK;
    regval |= fsysm;
    p_obj->p_instance->SYSCR = regval;
}

/*--------------------------------------------------*/
/**
  * @brief     Update Middle PrescalerClock according register values.
  * @param     p_obj       :CG object.
  * @retval    Middle PrescalerClock Frequency.
  * @note      -
  * @attention This function is not available in interrupt.
  */
/*--------------------------------------------------*/
uint32_t cg_get_fsysm(cg_t *p_obj)
{
    uint32_t result = 0U;

    /*------------------------------*/
    /*  Parameter Check             */
    /*------------------------------*/
#ifdef __DEBUG__
    /* Check the CG_NULL of address. */
    assert_param(IS_POINTER_NOT_NULL(p_obj));
    assert_param(IS_POINTER_NOT_NULL(p_obj->p_instance));
#endif /* #ifdef __DEBUG__ */

    /* System core clock update */
    SystemCoreClockUpdate();

    switch (p_obj->p_instance->SYSCR & CG_MCKSELGST_MASK)
    {
    case CG_MCKSELGST_1:   /* fsysm -> fc/PRCK */
        result = SystemCoreClock / CG_MCKSEL_1_DIV;
        break;
    case CG_MCKSELGST_2:   /* fsysm -> fc/PRCK/2 */
        result = SystemCoreClock / CG_MCKSEL_2_DIV;
        break;
    case CG_MCKSELGST_4:   /* fsysm -> fc/PRCK/4 */
        result = SystemCoreClock / CG_MCKSEL_4_DIV;
        break;
    default:
        result = 0U;
        break;
    }   
    return (result);

}

/*--------------------------------------------------*/
/**
  * @brief     Update Middle PrescalerClock according register values.
  * @param     p_obj       :CG object.
  * @retval    Middle PrescalerClock Frequency.
  * @note      -
  * @attention This function is not available in interrupt.
  */
/*--------------------------------------------------*/
uint32_t cg_get_mphyt0(cg_t *p_obj)
{
    uint32_t result = 0U;

    /*------------------------------*/
    /*  Parameter Check             */
    /*------------------------------*/
#ifdef __DEBUG__
    /* Check the CG_NULL of address. */
    assert_param(IS_POINTER_NOT_NULL(p_obj));
    assert_param(IS_POINTER_NOT_NULL(p_obj->p_instance));
#endif /* #ifdef __DEBUG__ */

    /* System core clock update */
    SystemCoreClockUpdate();

    /* Get Gear status. */
    switch (p_obj->p_instance->SYSCR & CG_FSYS_MASK)
    {
    case CG_FSYS_1:             /* Gear -> fc */
        result = SystemCoreClock * CG_FSYS_1_MUL;
        break;
    case CG_FSYS_2:             /* Gear -> fc/2 */
        result = SystemCoreClock * CG_FSYS_2_MUL;
        break;
    case CG_FSYS_4:             /* Gear -> fc/4 */
        result = SystemCoreClock * CG_FSYS_4_MUL;
        break;
    case CG_FSYS_8:             /* Gear -> fc/8 */
        result = SystemCoreClock * CG_FSYS_8_MUL;
        break;
    case CG_FSYS_16:            /* Gear -> fc/16 */
        result = SystemCoreClock * CG_FSYS_16_MUL;
        break;
    default:
        result = 0U;
        break;
    }
    switch (p_obj->p_instance->SYSCR & CG_PRCKST_MASK)
    {
    case CG_PRCKST_1:             /* T0 -> fc */
        result /= CG_PRCK_1_DIV;
        break;
    case CG_PRCKST_2:             /* T0 -> fc/2 */
        result /= CG_PRCK_2_DIV;
        break;
    case CG_PRCKST_4:             /* T0 -> fc/4 */
        result /= CG_PRCK_4_DIV;
        break;
    case CG_PRCKST_8:             /* T0 -> fc/8 */
        result /= CG_PRCK_8_DIV;
        break;
    case CG_PRCKST_16:            /* T0 -> fc/16 */
        result /= CG_PRCK_16_DIV;
        break;
    case CG_PRCKST_32:            /* T0 -> fc/32 */
        result /= CG_PRCK_32_DIV;
        break;
    case CG_PRCKST_64:            /* T0 -> fc/64 */
        result /= CG_PRCK_64_DIV;
        break;
    case CG_PRCKST_128:           /* T0 -> fc/128 */
        result /= CG_PRCK_128_DIV;
        break;
    case CG_PRCKST_256:           /* T0 -> fc/256 */
        result /= CG_PRCK_256_DIV;
        break;
    case CG_PRCKST_512:           /* T0 -> fc/512 */
        result /= CG_PRCK_512_DIV;
        break;
    default:
        result = 0U;
        break;
    }
    
    switch (p_obj->p_instance->SYSCR & CG_MCKSELPST_MASK)
    {
    case CG_MCKSELPST_1:   /* T0 -> fc/PRCK */
        result /= CG_MCKSEL_1_DIV;
        break;
    case CG_MCKSELPST_2:   /* T0 -> fc/PRCK/2 */
        result /= CG_MCKSEL_2_DIV;
        break;
    case CG_MCKSELPST_4:   /* T0 -> fc/PRCK/4 */
        result /= CG_MCKSEL_4_DIV;
        break;
    default:
        result = 0U;
        break;
    }   
    return (result);
}

/*--------------------------------------------------*/
/**
  * @brief     Get MCKSELGST.
  * @param     p_obj       :CG object.
  * @retval    MCKSELGST.
  * @note      -
  * @attention This function is not available in interrupt.
  */
/*--------------------------------------------------*/
uint32_t cg_get_mckselgst(cg_t *p_obj)
{
    uint32_t result = 0U;

    /*------------------------------*/
    /*  Parameter Check             */
    /*------------------------------*/
#ifdef __DEBUG__
    /* Check the CG_NULL of address. */
    assert_param(IS_POINTER_NOT_NULL(p_obj));
    assert_param(IS_POINTER_NOT_NULL(p_obj->p_instance));
#endif /* #ifdef __DEBUG__ */

    /* Get MCKSELGST status. */
    result = (p_obj->p_instance->SYSCR & CG_MCKSELGST_MASK);
    return (result);
}

/*--------------------------------------------------*/
/**
  * @brief     Get MCKSELPST.
  * @param     p_obj       :CG object.
  * @retval    MCKSELPST.
  * @note      -
  * @attention This function is not available in interrupt.
  */
/*--------------------------------------------------*/
uint32_t cg_get_mckselpst(cg_t *p_obj)
{
    uint32_t result = 0U;

    /*------------------------------*/
    /*  Parameter Check             */
    /*------------------------------*/
#ifdef __DEBUG__
    /* Check the CG_NULL of address. */
    assert_param(IS_POINTER_NOT_NULL(p_obj));
    assert_param(IS_POINTER_NOT_NULL(p_obj->p_instance));
#endif /* #ifdef __DEBUG__ */

    /* Get MCKSELPST status. */
    result = (p_obj->p_instance->SYSCR & CG_MCKSELPST_MASK);
    return (result);
}

#else
void cg_set_fsysm(cg_t *p_obj, uint32_t fsysm){}
uint32_t cg_get_fsysm(cg_t *p_obj){return 0;}
uint32_t cg_get_mphyt0(cg_t *p_obj){return 0;}
uint32_t cg_get_mckselgst(cg_t *p_obj){return 0;}
uint32_t cg_get_mckselpst(cg_t *p_obj){return 0;}
#endif /* FSYSM_EQUIPPED */


/*------------------------------------------------------------------------------*/
/*  IHOSC2 function                                                             */
/*------------------------------------------------------------------------------*/
#if (IHOSC2_EQUIPPED == AVAILABLE)
/*--------------------------------------------------*/
/**
  * @brief Check whether ihosc2 state.
  * @param  -
  * @retval    TXZ_DONE :Done.
  * @retval    TXZ_BUSY :Busy.
  * @attention This function is not available in interrupt.
  */
/*--------------------------------------------------*/
TXZ_WorkState cg_get_ihosc2_state(void)
{
    TXZ_WorkState state = TXZ_BUSY;

    if (TSB_CG_OSCCR_IHOSC2F == 1U) {
        state = TXZ_DONE;
    } else {
        /* Do nothing */
    }

    return state;
}

/*--------------------------------------------------*/
/**
  * @brief     Set Internal high-speed oscillator2 enable.
  * @param     -
  * @retval    -
  * @note      -
  * @attention This function is not available in interrupt.
  */
/*--------------------------------------------------*/
void cg_ihosc2_enable(void)
{
    /* Internal high-speed oscillator2 is enable. */
    TSB_CG_OSCCR_IHOSC2EN = 1U;
}

/*--------------------------------------------------*/
/**
  * @brief     Set Internal high-speed oscillator2 disable.
  * @param     -
  * @retval    -
  * @note      -
  * @attention This function is not available in interrupt.
  */
/*--------------------------------------------------*/
void cg_ihosc2_disable(void)
{
    /* Internal high-speed oscillator2 is disable. */
    TSB_CG_OSCCR_IHOSC2EN = 0U;
}

#else
TXZ_WorkState cg_get_ihosc2_state(void){return TXZ_DONE;}
void cg_ihosc2_enable(void){}
void cg_ihosc2_disable(void){}
#endif /* IHOSC2_EQUIPPED */


/*------------------------------------------------------------------------------*/
/*  SCOUT function                                                              */
/*------------------------------------------------------------------------------*/
#if (SCOUT_EQUIPPED == AVAILABLE)
/*--------------------------------------------------*/
/**
  * @brief     Set SCOUT enable.
  * @param     -
  * @retval    -
  * @note      -
  * @attention This function is not available in interrupt.
  */
/*--------------------------------------------------*/
void cg_scout_enable(void)
{
    /* SCOUT is enable. */
    TSB_CG_SCOCR_SCOEN = CG_SCOCR_SCOEN_ENABLE;
}

/*--------------------------------------------------*/
/**
  * @brief     Set SCOUT disable.
  * @param     -
  * @retval    -
  * @note      -
  * @attention This function is not available in interrupt.
  */
/*--------------------------------------------------*/
void cg_scout_disable(void)
{
    /* SCOUT is disable. */
    TSB_CG_SCOCR_SCOEN = CG_SCOCR_SCOEN_DISABLE;
}

/*--------------------------------------------------*/
/**
  * @brief     Set SCOUT base clock.
  * @param     p_obj  : CG object.
  * @param     source : SCOUT base clock.
  * @retval    -
  * @note      -
  * @attention This function is not available in interrupt.
  */
/*--------------------------------------------------*/
void cg_set_scout_clock(cg_t *p_obj, uint32_t source)
{
    uint32_t regval = 0;

    /*------------------------------*/
    /*  Parameter Check             */
    /*------------------------------*/
#ifdef __DEBUG__
    /* Check the CG_NULL of address. */
    assert_param(IS_POINTER_NOT_NULL(p_obj));
    assert_param(IS_POINTER_NOT_NULL(p_obj->p_instance));
    assert_param(check_param_scosel(source));
#endif /* #ifdef __DEBUG__ */

    /* Set SCOUT base clock value. */
    regval = p_obj->p_instance->SCOCR & CG_SCOSEL_MASK;
    regval |= source;
    p_obj->p_instance->SCOCR = regval;
}

/*--------------------------------------------------*/
/**
  * @brief     Set SCOUT time.
  * @param     p_obj  : CG object.
  * @param     source  : SCOUT time.
  * @retval    -
  * @note      -
  * @attention This function is not available in interrupt.
  */
/*--------------------------------------------------*/
void cg_set_scout_time(cg_t *p_obj, uint32_t time)
{
    uint32_t regval = 0;

    /*------------------------------*/
    /*  Parameter Check             */
    /*------------------------------*/
#ifdef __DEBUG__
    /* Check the CG_NULL of address. */
    assert_param(IS_POINTER_NOT_NULL(p_obj));
    assert_param(IS_POINTER_NOT_NULL(p_obj->p_instance));
    assert_param(check_param_scodiv(time));
#endif /* #ifdef __DEBUG__ */

    /* Set SCOUT time value. */
    regval = p_obj->p_instance->SCOCR & CG_SCODIV_MASK;
    regval |= time;
    p_obj->p_instance->SCOCR = regval;
}

#else
void cg_scout_enable(void){}
void cg_scout_disable(void){}
void cg_set_scout_clock(cg_t *p_obj, uint32_t source){}
void cg_set_scout_time(cg_t *p_obj, uint32_t time){}
#endif /* SCOUT_EQUIPPED */


/*------------------------------------------------------------------------------*/
/*  low speed warm up function                                                  */
/*------------------------------------------------------------------------------*/
#if (LOW_WARMUP_EQUIPPED == AVAILABLE)
/*--------------------------------------------------*/
/**
  * @brief  Set the low warm up time.
  * @param  p_obj                  :CG object.
  * @param  time: Set count time for warm-up timer
  * @retval -
  * @attention This function is not available in interrupt.
  */
/*--------------------------------------------------*/
void cg_set_low_warm_up_time(cg_t *p_obj, uint32_t time)
{
    uint32_t work = 0;

    /*------------------------------*/
    /*  Parameter Check             */
    /*------------------------------*/
#ifdef __DEBUG__
    /* Check the CG_NULL of address. */
    assert_param(IS_POINTER_NOT_NULL(p_obj));
    assert_param(IS_POINTER_NOT_NULL(p_obj->p_instance));
#endif /* #ifdef __DEBUG__ */

    /* [CGWUPLCR]<WUPTL> :Warm up time        */
    /*-----------------------------------------------*/
    /* "1"counter  (s) = 1 / ELOSC                   */
    /* "1"counter (us) = (10^6) / ELOSC              */
    /* "x"counter (us) = time                        */
    /*-----------------------------------------------*/
    /* x : time = 1 : (10^6) / ELOSC                 */
    /*-----------------------------------------------*/
    {
        uint64_t x = (uint64_t)((uint64_t)(time) * (uint64_t)(ELOSC_CFG_CLOCK));
        x = (uint64_t)((uint64_t)(x / (uint64_t)(1000000)) - 16);
        if (x > (uint64_t)(0x7FFFF))
        {
            /* invalid value */
        }
        work = (uint32_t)x;
    }
    work  &= (uint32_t)(0xFFFFFFF0);
    work <<= 8;
    p_obj->p_instance->WUPLCR = work;
}

/*--------------------------------------------------*/
/**
  * @brief  Start operation of warm up timer for low oscillator.
  * @param  -
  * @retval -
  * @attention This function is not available in interrupt.
  */
/*--------------------------------------------------*/
void cg_start_low_warmup(void)
{
    TSB_CG_WUPLCR_WULON = 1U;
}

/*--------------------------------------------------*/
/**
  * @brief Check whether low warm up is completed or not.
  * @param  -
  * @retval    TXZ_DONE :Done.
  * @retval    TXZ_BUSY :Busy.
  * @attention This function is not available in interrupt.
  */
/*--------------------------------------------------*/
TXZ_WorkState cg_get_low_warm_up_state(void)
{
    TXZ_WorkState state = TXZ_BUSY;

    if ( TSB_CG_WUPLCR_WULEF == 0U) {
        state = TXZ_DONE;
    } else {
        /* Do nothing */
    }

    return state;
}

#else
void cg_set_low_warm_up_time(cg_t *p_obj, uint32_t time){}
void cg_start_low_warmup(void){}
TXZ_WorkState cg_get_low_warm_up_state(void){return TXZ_DONE;}
#endif /* SCOUT_EQUIPPED */


/*------------------------------------------------------------------------------*/
/*  FSYSMENA function                                                           */
/*------------------------------------------------------------------------------*/
#if (FSYSMENA_EQUIPPED == AVAILABLE)
/*--------------------------------------------------*/
/**
  * @brief     Set CG FSYSMENA.
  * @param     p_obj       :CG object.
  * @param     ipmena      :Set fsysmena register
  * @param     state       :TXZ_ENABLE or TXZ_DISABLE
  * @retval    -
  * @note      -
  * @attention This function is not available in interrupt.
  */
/*--------------------------------------------------*/
void cg_set_fsysmena(cg_t *p_obj, uint32_t ipmena, TXZ_FunctionalState state)
{
    /*------------------------------*/
    /*  Parameter Check             */
    /*------------------------------*/
#ifdef __DEBUG__
    /* Check the CG_NULL of address. */
    assert_param(IS_POINTER_NOT_NULL(p_obj));
    assert_param(IS_POINTER_NOT_NULL(p_obj->p_instance));
    assert_param(check_param_ipmena(ipmena));
#endif /* #ifdef __DEBUG__ */

    uint32_t regval = p_obj->p_instance->FSYSMENA;

    if (state == TXZ_ENABLE) {
        regval |= (ipmena & CG_FSYSMENA_ALL_ENABLE);
    } else {
        regval &= ~(ipmena & CG_FSYSMENA_ALL_ENABLE);
    }
    p_obj->p_instance->FSYSMENA = regval;
}

/*--------------------------------------------------*/
/**
  * @brief     Set CG ALL FSYSMENA.
  * @param     p_obj       :CG object.
  * @param     state       :TXZ_ENABLE or TXZ_DISABLE
  * @retval    -
  * @note      -
  * @attention This function is not available in interrupt.
  */
/*--------------------------------------------------*/
void cg_set_all_fsysmena(cg_t *p_obj, TXZ_FunctionalState state)
{
    /*------------------------------*/
    /*  Parameter Check             */
    /*------------------------------*/
#ifdef __DEBUG__
    /* Check the CG_NULL of address. */
    assert_param(IS_POINTER_NOT_NULL(p_obj));
    assert_param(IS_POINTER_NOT_NULL(p_obj->p_instance));
#endif /* #ifdef __DEBUG__ */

    uint32_t regval = p_obj->p_instance->FSYSMENA;

    if (state == TXZ_ENABLE) {
        regval = CG_FSYSMENA_ALL_ENABLE;
    } else {
        regval = CG_FSYSMENA_ALL_DISABLE;
    }
    p_obj->p_instance->FSYSMENA = regval;
}

#else
void cg_set_fsysmena(cg_t *p_obj, uint32_t ipmena, TXZ_FunctionalState state){}
void cg_set_all_fsysmena(cg_t *p_obj, TXZ_FunctionalState state){}
#endif /* FSYSMENA_EQUIPPED */


/*------------------------------------------------------------------------------*/
/*  FSYSMENB function                                                           */
/*------------------------------------------------------------------------------*/
#if (FSYSMENB_EQUIPPED == AVAILABLE)
/*--------------------------------------------------*/
/**
  * @brief     Set CG FSYSMENB.
  * @param     p_obj       :CG object.
  * @param     ipmenb      :Set fsysmenb register
  * @param     state       :TXZ_ENABLE or TXZ_DISABLE
  * @retval    -
  * @note      -
  * @attention This function is not available in interrupt.
  */
/*--------------------------------------------------*/
void cg_set_fsysmenb(cg_t *p_obj, uint32_t ipmenb, TXZ_FunctionalState state)
{
    /*------------------------------*/
    /*  Parameter Check             */
    /*------------------------------*/
#ifdef __DEBUG__
    /* Check the CG_NULL of address. */
    assert_param(IS_POINTER_NOT_NULL(p_obj));
    assert_param(IS_POINTER_NOT_NULL(p_obj->p_instance));
    assert_param(check_param_ipmenb(ipmenb));
#endif /* #ifdef __DEBUG__ */

    uint32_t regval = p_obj->p_instance->FSYSMENB;

    if (state == TXZ_ENABLE) {
        regval |= (ipmenb & CG_FSYSMENB_ALL_ENABLE);
    } else {
        regval &= ~(ipmenb & CG_FSYSMENB_ALL_ENABLE);
    }
    p_obj->p_instance->FSYSMENB = regval;
}

/*--------------------------------------------------*/
/**
  * @brief     Set CG ALL FSYSMENB.
  * @param     p_obj       :CG object.
  * @param     state       :TXZ_ENABLE or TXZ_DISABLE
  * @retval    -
  * @note      -
  * @attention This function is not available in interrupt.
  */
/*--------------------------------------------------*/
void cg_set_all_fsysmenb(cg_t *p_obj, TXZ_FunctionalState state)
{
    /*------------------------------*/
    /*  Parameter Check             */
    /*------------------------------*/
#ifdef __DEBUG__
    /* Check the CG_NULL of address. */
    assert_param(IS_POINTER_NOT_NULL(p_obj));
    assert_param(IS_POINTER_NOT_NULL(p_obj->p_instance));
#endif /* #ifdef __DEBUG__ */

    uint32_t regval = p_obj->p_instance->FSYSMENB;

    if (state == TXZ_ENABLE) {
        regval = CG_FSYSMENB_ALL_ENABLE;
    } else {
        regval = CG_FSYSMENB_ALL_DISABLE;
    }
    p_obj->p_instance->FSYSMENB = regval;
}

#else
void cg_set_fsysmenb(cg_t *p_obj, uint32_t ipmenb, TXZ_FunctionalState state){}
void cg_set_all_fsysmenb(cg_t *p_obj, TXZ_FunctionalState state){}
#endif /* FSYSMENB_EQUIPPED */


/*------------------------------------------------------------------------------*/
/*  FSYSMENC function                                                           */
/*------------------------------------------------------------------------------*/
#if (FSYSMENC_EQUIPPED == AVAILABLE)
/*--------------------------------------------------*/
/**
  * @brief     Set CG FSYSMENC.
  * @param     p_obj       :CG object.
  * @param     ipmenc      :Set fsysmenc register
  * @param     state       :TXZ_ENABLE or TXZ_DISABLE
  * @retval    -
  * @note      -
  * @attention This function is not available in interrupt.
  */
/*--------------------------------------------------*/
void cg_set_fsysmenc(cg_t *p_obj, uint32_t ipmenc, TXZ_FunctionalState state)
{
    /*------------------------------*/
    /*  Parameter Check             */
    /*------------------------------*/
#ifdef __DEBUG__
    /* Check the CG_NULL of address. */
    assert_param(IS_POINTER_NOT_NULL(p_obj));
    assert_param(IS_POINTER_NOT_NULL(p_obj->p_instance));
    assert_param(check_param_ipmenc(ipmenc));
#endif /* #ifdef __DEBUG__ */

    uint32_t regval = p_obj->p_instance->FSYSMENC;

    if (state == TXZ_ENABLE) {
        regval |= (ipmenc & CG_FSYSMENC_ALL_ENABLE);
    } else {
        regval &= ~(ipmenc & CG_FSYSMENC_ALL_ENABLE);
    }
    p_obj->p_instance->FSYSMENC = regval;
}

/*--------------------------------------------------*/
/**
  * @brief     Set CG ALL FSYSMENC.
  * @param     p_obj       :CG object.
  * @param     state       :TXZ_ENABLE or TXZ_DISABLE
  * @retval    -
  * @note      -
  * @attention This function is not available in interrupt.
  */
/*--------------------------------------------------*/
void cg_set_all_fsysmenc(cg_t *p_obj, TXZ_FunctionalState state)
{
    /*------------------------------*/
    /*  Parameter Check             */
    /*------------------------------*/
#ifdef __DEBUG__
    /* Check the CG_NULL of address. */
    assert_param(IS_POINTER_NOT_NULL(p_obj));
    assert_param(IS_POINTER_NOT_NULL(p_obj->p_instance));
#endif /* #ifdef __DEBUG__ */

    uint32_t regval = p_obj->p_instance->FSYSMENC;

    if (state == TXZ_ENABLE) {
        regval = CG_FSYSMENC_ALL_ENABLE;
    } else {
        regval = CG_FSYSMENC_ALL_DISABLE;
    }
    p_obj->p_instance->FSYSMENC = regval;
}

#else
void cg_set_fsysmenc(cg_t *p_obj, uint32_t ipmenc, TXZ_FunctionalState state){}
void cg_set_all_fsysmenc(cg_t *p_obj, TXZ_FunctionalState state){}
#endif /* FSYSMENC_EQUIPPED */


/*------------------------------------------------------------------------------*/
/*  FSYSENA function                                                            */
/*------------------------------------------------------------------------------*/
#if (FSYSENA_EQUIPPED == AVAILABLE)
/*--------------------------------------------------*/
/**
  * @brief     Set CG FSYSENA.
  * @param     p_obj       :CG object.
  * @param     ipmena      :Set fsysena register
  * @param     state       :TXZ_ENABLE or TXZ_DISABLE
  * @retval    -
  * @note      -
  * @attention This function is not available in interrupt.
  */
/*--------------------------------------------------*/
void cg_set_fsysena(cg_t *p_obj, uint32_t ipena, TXZ_FunctionalState state)
{
    /*------------------------------*/
    /*  Parameter Check             */
    /*------------------------------*/
#ifdef __DEBUG__
    /* Check the CG_NULL of address. */
    assert_param(IS_POINTER_NOT_NULL(p_obj));
    assert_param(IS_POINTER_NOT_NULL(p_obj->p_instance));
    assert_param(check_param_ipena(ipena));
#endif /* #ifdef __DEBUG__ */

    uint32_t regval = p_obj->p_instance->FSYSENA;

    if (state == TXZ_ENABLE) {
        regval |= (ipena & CG_FSYSENA_ALL_ENABLE);
    } else {
        regval &= ~(ipena & CG_FSYSENA_ALL_ENABLE);
    }
    p_obj->p_instance->FSYSENA = regval;
}

/*--------------------------------------------------*/
/**
  * @brief     Set CG ALL FSYSENA.
  * @param     p_obj       :CG object.
  * @param     state       :TXZ_ENABLE or TXZ_DISABLE
  * @retval    -
  * @note      -
  * @attention This function is not available in interrupt.
  */
/*--------------------------------------------------*/
void cg_set_all_fsysena(cg_t *p_obj, TXZ_FunctionalState state)
{
    /*------------------------------*/
    /*  Parameter Check             */
    /*------------------------------*/
#ifdef __DEBUG__
    /* Check the CG_NULL of address. */
    assert_param(IS_POINTER_NOT_NULL(p_obj));
    assert_param(IS_POINTER_NOT_NULL(p_obj->p_instance));
#endif /* #ifdef __DEBUG__ */

    uint32_t regval = p_obj->p_instance->FSYSENA;

    if (state == TXZ_ENABLE) {
        regval = CG_FSYSENA_ALL_ENABLE;
    } else {
        regval = CG_FSYSENA_ALL_DISABLE;
    }
    p_obj->p_instance->FSYSENA = regval;
}

#else
void cg_set_fsysena(cg_t *p_obj, uint32_t ipena, TXZ_FunctionalState state){}
void cg_set_all_fsysena(cg_t *p_obj, TXZ_FunctionalState state){}
#endif /* FSYSENA_EQUIPPED */

/*------------------------------------------------------------------------------*/
/*  FSYSENB function                                                            */
/*------------------------------------------------------------------------------*/
#if (FSYSENB_EQUIPPED == AVAILABLE)
/*--------------------------------------------------*/
/**
  * @brief     Set CG FSYSENB.
  * @param     p_obj       :CG object.
  * @param     ipenb       :Set fsysenb register
  * @param     state       :TXZ_ENABLE or TXZ_DISABLE
  * @retval    -
  * @note      -
  * @attention This function is not available in interrupt.
  */
/*--------------------------------------------------*/
void cg_set_fsysenb(cg_t *p_obj, uint32_t ipenb, TXZ_FunctionalState state)
{
    /*------------------------------*/
    /*  Parameter Check             */
    /*------------------------------*/
#ifdef __DEBUG__
    /* Check the CG_NULL of address. */
    assert_param(IS_POINTER_NOT_NULL(p_obj));
    assert_param(IS_POINTER_NOT_NULL(p_obj->p_instance));
    assert_param(check_param_ipenb(ipenb));
#endif /* #ifdef __DEBUG__ */

    uint32_t regval = p_obj->p_instance->FSYSENB;

    if (state == TXZ_ENABLE) {
        regval |= (ipenb & CG_FSYSENB_ALL_ENABLE);
    } else {
        regval &= ~(ipenb & CG_FSYSENB_ALL_ENABLE);
    }
    p_obj->p_instance->FSYSENB = regval;
}

/*--------------------------------------------------*/
/**
  * @brief     Set CG ALL FSYSENB.
  * @param     p_obj       :CG object.
  * @param     state       :TXZ_ENABLE or TXZ_DISABLE
  * @retval    -
  * @note      -
  * @attention This function is not available in interrupt.
  */
/*--------------------------------------------------*/
void cg_set_all_fsysenb(cg_t *p_obj, TXZ_FunctionalState state)
{
    /*------------------------------*/
    /*  Parameter Check             */
    /*------------------------------*/
#ifdef __DEBUG__
    /* Check the CG_NULL of address. */
    assert_param(IS_POINTER_NOT_NULL(p_obj));
    assert_param(IS_POINTER_NOT_NULL(p_obj->p_instance));
#endif /* #ifdef __DEBUG__ */

    uint32_t regval = p_obj->p_instance->FSYSENB;

    if (state == TXZ_ENABLE) {
        regval = CG_FSYSENB_ALL_ENABLE;
    } else {
        regval = CG_FSYSENB_ALL_DISABLE;
    }
    p_obj->p_instance->FSYSENB = regval;
}
#else
void cg_set_fsysenb(cg_t *p_obj, uint32_t ipenb, TXZ_FunctionalState state){}
void cg_set_all_fsysenb(cg_t *p_obj, TXZ_FunctionalState state){}
#endif /* FSYSENB_EQUIPPED */


/*------------------------------------------------------------------------------*/
/*  EXTEND2 function                                                            */
/*------------------------------------------------------------------------------*/
#if (EXTEND2_EQUIPPED == AVAILABLE)

/*--------------------------------------------------*/
/**
  * @brief     CG EXTEND2 function.
  * @param     p_obj       :CG object.
  * @param     bit         :CG EXTEND2 register
  * @retval    -
  * @note      -
  * @attention This function is not available in interrupt.
  */
/*--------------------------------------------------*/
void cg_extend2_func(cg_t *p_obj, uint32_t bit)
{
    /*------------------------------*/
    /*  Parameter Check             */
    /*------------------------------*/
#ifdef __DEBUG__
    /* Check the CG_NULL of address. */
    assert_param(IS_POINTER_NOT_NULL(p_obj));
    assert_param(IS_POINTER_NOT_NULL(p_obj->p_instance));
    assert_param(check_param_extend2(bit));
#endif /* #ifdef __DEBUG__ */

    /* fs is enable. */
    p_obj->p_instance->EXTEND2 = CG_EXTEND2_0;
    p_obj->p_instance->EXTEND2 = bit;
    __NOP();
    __NOP();
    __NOP();
    __NOP();
    p_obj->p_instance->EXTEND2 = CG_EXTEND2_0;
}

#else
void cg_extend2_func(cg_t *p_obj, uint32_t bit){}
#endif /* EXTEND2_EQUIPPED */


/*------------------------------------------------------------------------------*/
/*  RLM LOSCCR function                                                         */
/*------------------------------------------------------------------------------*/
#if (RLM_LOSCCR_EQUIPPED == AVAILABLE)

/*--------------------------------------------------*/
/**
  * @brief     Set fs enable.
  * @param     p_obj       :RLM object.
  * @retval    -
  * @note      -
  * @attention This function is not available in interrupt.
  */
/*--------------------------------------------------*/
void rlm_fs_enable(rlm_t *p_obj)
{
    uint8_t regval = 0;

    /*------------------------------*/
    /*  Parameter Check             */
    /*------------------------------*/
#ifdef __DEBUG__
    /* Check the CG_NULL of address. */
    assert_param(IS_POINTER_NOT_NULL(p_obj));
    assert_param(IS_POINTER_NOT_NULL(p_obj->p_instance));
#endif /* #ifdef __DEBUG__ */

    /* fs is enable. */
    regval = p_obj->p_instance->LOSCCR & RLM_XTEN_MASK;
    regval |= RLM_XTEN_ENABLE;
    p_obj->p_instance->LOSCCR = regval;
}

/*--------------------------------------------------*/
/**
  * @brief     Set fs disable.
  * @param     p_obj       :RLM object.
  * @retval    -
  * @note      -
  * @attention This function is not available in interrupt.
  */
/*--------------------------------------------------*/
void rlm_fs_disable(rlm_t *p_obj)
{
    uint8_t regval = 0;

    /*------------------------------*/
    /*  Parameter Check             */
    /*------------------------------*/
#ifdef __DEBUG__
    /* Check the CG_NULL of address. */
    assert_param(IS_POINTER_NOT_NULL(p_obj));
    assert_param(IS_POINTER_NOT_NULL(p_obj->p_instance));
#endif /* #ifdef __DEBUG__ */

    /* fs is enable. */
    regval = p_obj->p_instance->LOSCCR & RLM_XTEN_MASK;
    regval |= RLM_XTEN_DISABLE;
    p_obj->p_instance->LOSCCR = regval;
}

#else
void rlm_fs_enable(rlm_t *p_obj){}
void rlm_fs_disable(rlm_t *p_obj){}
#endif /* RLM_LOSCCR_EQUIPPED */


/*------------------------------------------------------------------------------*/
/*  RLM LOSCCR IHOSC2 function                                                            */
/*------------------------------------------------------------------------------*/
#if (RLM_LOSCCR_IHOSC2_EQUIPPED == AVAILABLE)

/*--------------------------------------------------*/
/**
  * @brief     Set RLM IHOSC2 enable.
  * @param     p_obj       :RLM object.
  * @retval    -
  * @note      -
  * @attention This function is not available in interrupt.
  */
/*--------------------------------------------------*/
void rlm_ihosc2_enable(rlm_t *p_obj)
{
    uint8_t regval = 0;

    /*------------------------------*/
    /*  Parameter Check             */
    /*------------------------------*/
#ifdef __DEBUG__
    /* Check the CG_NULL of address. */
    assert_param(IS_POINTER_NOT_NULL(p_obj));
    assert_param(IS_POINTER_NOT_NULL(p_obj->p_instance));
#endif /* #ifdef __DEBUG__ */

    /* RLM IHOSC2 is enable. */
    regval = p_obj->p_instance->LOSCCR & RLM_IHOSC2_MASK;
    regval |= RLM_IHOSC2_ENABLE;
    p_obj->p_instance->LOSCCR = regval;
}

/*--------------------------------------------------*/
/**
  * @brief     Set RLM IHOSC2 disable.
  * @param     p_obj       :RLM object.
  * @retval    -
  * @note      -
  * @attention This function is not available in interrupt.
  */
/*--------------------------------------------------*/
void rlm_ihosc2_disable(rlm_t *p_obj)
{
    uint8_t regval = 0;

    /*------------------------------*/
    /*  Parameter Check             */
    /*------------------------------*/
#ifdef __DEBUG__
    /* Check the CG_NULL of address. */
    assert_param(IS_POINTER_NOT_NULL(p_obj));
    assert_param(IS_POINTER_NOT_NULL(p_obj->p_instance));
#endif /* #ifdef __DEBUG__ */

    /* RLM IHOSC2 is enable. */
    regval = p_obj->p_instance->LOSCCR & RLM_IHOSC2_MASK;
    regval |= RLM_IHOSC2_DISABLE;
    p_obj->p_instance->LOSCCR = regval;
}

/*--------------------------------------------------*/
/**
  * @brief     Get RLM IHOSC2 state.
  * @param     p_obj       :RLM object.
  * @retval    TXZ_DONE :Done.
  * @retval    TXZ_BUSY :Busy.
  * @note      -
  * @attention This function is not available in interrupt.
  */
/*--------------------------------------------------*/
TXZ_WorkState rlm_get_ihosc2_state(rlm_t *p_obj)
{
    TXZ_WorkState state = TXZ_BUSY;

    if (p_obj->p_instance->LOSCCR & RLM_IHOSC2_FLAG) {
        state = TXZ_DONE;
    } else {
        /* Do nothing */
    }

    return state;
}

/*--------------------------------------------------*/
/**
  * @brief     Set RLM ELCLKIN.
  * @param     p_obj       :RLM object.
  * @retval    -
  * @note      -
  * @attention This function is not available in interrupt.
  */
/*--------------------------------------------------*/
void rlm_set_elclkin(rlm_t *p_obj)
{
    uint8_t regval = 0;

    /*------------------------------*/
    /*  Parameter Check             */
    /*------------------------------*/
#ifdef __DEBUG__
    /* Check the CG_NULL of address. */
    assert_param(IS_POINTER_NOT_NULL(p_obj));
    assert_param(IS_POINTER_NOT_NULL(p_obj->p_instance));
#endif /* #ifdef __DEBUG__ */

    /* Set RLM ELCLKIN. */
    regval = p_obj->p_instance->LOSCCR & RLM_DRCOSCL_MASK;
    regval |= RLM_DRCOSCL_ELCLKIN;
    p_obj->p_instance->LOSCCR = regval;
}

/*--------------------------------------------------*/
/**
  * @brief     Set RLM ELOSC.
  * @param     p_obj       :RLM object.
  * @retval    -
  * @note      -
  * @attention This function is not available in interrupt.
  */
/*--------------------------------------------------*/
void rlm_set_elosc(rlm_t *p_obj)
{
    uint8_t regval = 0;

    /*------------------------------*/
    /*  Parameter Check             */
    /*------------------------------*/
#ifdef __DEBUG__
    /* Check the CG_NULL of address. */
    assert_param(IS_POINTER_NOT_NULL(p_obj));
    assert_param(IS_POINTER_NOT_NULL(p_obj->p_instance));
#endif /* #ifdef __DEBUG__ */

    /* Set RLM ELOSC. */
    regval = p_obj->p_instance->LOSCCR & RLM_DRCOSCL_MASK;
    regval |= RLM_DRCOSCL_ELOSC;
    p_obj->p_instance->LOSCCR = regval;
}

#else
void rlm_ihosc2_enable(rlm_t *p_obj){}
void rlm_ihosc2_disable(rlm_t *p_obj){}
TXZ_WorkState rlm_get_ihosc2_state(rlm_t *p_obj){return TXZ_DONE;}
void rlm_set_elclkin(rlm_t *p_obj){}
void rlm_set_elosc(rlm_t *p_obj){}
#endif /* RLM_LOSCCR_IHOSC2_EQUIPPED */


/*------------------------------------------------------------------------------*/
/*  RLM SHTDNOP PROTECT function                                                            */
/*------------------------------------------------------------------------------*/
#if (RLM_SHTDNOP_PROTECT_EQUIPPED == AVAILABLE)

/*--------------------------------------------------*/
/**
  * @brief     Set RLM port keep enable.
  * @param     p_obj       :RLM object.
  * @retval    -
  * @note      -
  * @attention This function is not available in interrupt.
  */
/*--------------------------------------------------*/
void rlm_port_keep_enable(rlm_t *p_obj)
{
    uint8_t regval = 0;

    /*------------------------------*/
    /*  Parameter Check             */
    /*------------------------------*/
#ifdef __DEBUG__
    /* Check the CG_NULL of address. */
    assert_param(IS_POINTER_NOT_NULL(p_obj));
    assert_param(IS_POINTER_NOT_NULL(p_obj->p_instance));
#endif /* #ifdef __DEBUG__ */

    /* RLM ptkeep is enable. */
    regval = p_obj->p_instance->SHTDNOP & RLM_PTKEEP_MASK;
    regval |= RLM_PTKEEP_ENABLE;
    p_obj->p_instance->SHTDNOP = regval;
}

/*--------------------------------------------------*/
/**
  * @brief     Set RLM port keep disable.
  * @param     p_obj       :RLM object.
  * @retval    -
  * @note      -
  * @attention This function is not available in interrupt.
  */
/*--------------------------------------------------*/
void rlm_port_keep_disable(rlm_t *p_obj)
{
    uint8_t regval = 0;

    /*------------------------------*/
    /*  Parameter Check             */
    /*------------------------------*/
#ifdef __DEBUG__
    /* Check the CG_NULL of address. */
    assert_param(IS_POINTER_NOT_NULL(p_obj));
    assert_param(IS_POINTER_NOT_NULL(p_obj->p_instance));
#endif /* #ifdef __DEBUG__ */

    /* RLM ptkeep is disable. */
    regval = p_obj->p_instance->SHTDNOP & RLM_PTKEEP_MASK;
    regval |= RLM_PTKEEP_DISABLE;
    p_obj->p_instance->SHTDNOP = regval;
}

/*--------------------------------------------------*/
/**
  * @brief     Set RLM protect enable.
  * @param     p_obj       :RLM object.
  * @retval    -
  * @note      -
  * @attention This function is not available in interrupt.
  */
/*--------------------------------------------------*/
void rlm_protect_enable(rlm_t *p_obj)
{
    /*------------------------------*/
    /*  Parameter Check             */
    /*------------------------------*/
#ifdef __DEBUG__
    /* Check the CG_NULL of address. */
    assert_param(IS_POINTER_NOT_NULL(p_obj));
    assert_param(IS_POINTER_NOT_NULL(p_obj->p_instance));
#endif /* #ifdef __DEBUG__ */

    /* RLM protect is enable. */
    p_obj->p_instance->PROTECT = RLM_PROTECT_ENABLE;
}

/*--------------------------------------------------*/
/**
  * @brief     Set RLM protect disable.
  * @param     p_obj       :RLM object.
  * @retval    -
  * @note      -
  * @attention This function is not available in interrupt.
  */
/*--------------------------------------------------*/
void rlm_protect_disable(rlm_t *p_obj)
{
    /*------------------------------*/
    /*  Parameter Check             */
    /*------------------------------*/
#ifdef __DEBUG__
    /* Check the CG_NULL of address. */
    assert_param(IS_POINTER_NOT_NULL(p_obj));
    assert_param(IS_POINTER_NOT_NULL(p_obj->p_instance));
#endif /* #ifdef __DEBUG__ */

    /* RLM protect is disable. */
    p_obj->p_instance->PROTECT = RLM_PROTECT_DISABLE;
}

#else
void rlm_port_keep_enable(rlm_t *p_obj){}
void rlm_port_keep_disable(rlm_t *p_obj){}
void rlm_protect_enable(rlm_t *p_obj){}
void rlm_protect_disable(rlm_t *p_obj){}
#endif /* RLM_SHTDNOP_PROTECT_EQUIPPED */

/*--------------------------------------------------*/
/** 
  * @brief  RAM register set.
  * @param  none.
  * @retval none.
  * @note   -
  */
/*--------------------------------------------------*/
#if (FC_FCRACCR_EQUIPPED == AVAILABLE)
void fc_ram_con_reg_set(uint32_t sysclock)
{
    uint32_t regval = 0;

    if(sysclock <= FC_RACCR_THRESHOLD) /* less than Threshold   */
    {
       regval = (uint32_t)FC_RACCR_RAMLC_1CLK;
       TSB_FC->KCR = FC_KCR_KEYCODE;
       TSB_FC->RACCR = regval;
       while(TSB_FC->RACCR != (uint32_t)FC_RACCR_RAMLC_1CLK)
       {
           /* no processing */
       }
    }
    else                           /* more than Threshold       */
    {
       regval = (uint32_t)FC_RACCR_RAMLC_2CLK;
       TSB_FC->KCR = FC_KCR_KEYCODE;
       TSB_FC->RACCR = regval;
       while(TSB_FC->RACCR != (uint32_t)FC_RACCR_RAMLC_2CLK)
       {
           /* no processing */
       }
    }
}
#else
void fc_ram_con_reg_set(uint32_t sysclock) {}
#endif /* FC_FCRACCR_EQUIPPED */

/**
 *  @}
 */ /* End of group CG_Exported_functions */

/**
 *  @} 
 */ /* End of group CG */

/**
 *  @} 
 */ /* End of group Periph_Driver */

#ifdef __cplusplus
}
#endif /* __cplusplus */

