/** 
 *******************************************************************************
 * @file    flash.c
 * @brief   This file provides API functions for FLASH.
 * @version V1.0.0
 * 
 * DO NOT USE THIS SOFTWARE WITHOUT THE SOFTWARE LICENSE AGREEMENT.
 * 
 * Copyright(C) Toshiba Electronic Device Solutions Corporation 2022
 *******************************************************************************
 */
#ifdef __cplusplus
 extern "C" {
#endif

/*------------------------------------------------------------------------------*/
/*  Includes                                                                    */
/*------------------------------------------------------------------------------*/
#include <string.h>
#include "flash.h"

/**
 *  @addtogroup Periph_Driver Peripheral Driver
 *  @{
 */

/** 
 *  @addtogroup FLASH 
 *  @brief FLASH Driver
 *  @{
 */


/*------------------------------------------------------------------------------*/
/*  Macro Function                                                              */
/*------------------------------------------------------------------------------*/
/** 
 *  @defgroup FLASH_Private_macro FLASH Private Macro
 *  @{
 */

/** 
 *  @defgroup Parameter_Result Parameter Result
 *  @brief    Whether the parameter is specified or not.
 *  @{
 */
#define FLASH_PARAM_OK             ((int32_t)1)              /*!< Parameter is valid(specified).         */
#define FLASH_PARAM_NG             ((int32_t)0)              /*!< Parameter is invalid(not specified).   */
/**
 *  @}
 */ /* End of group Parameter_Result */

/** 
 *  @}
 */ /* End of group FLASH_Private_macro */


/*------------------------------------------------------------------------------*/
/*  Configuration                                                               */
/*------------------------------------------------------------------------------*/
/** 
 *  @defgroup FLASH_Private_define FLASH Private Define
 *  @{
 */

/** 
 *  @}
 */ /* End of group FLASH_Private_define */


/*------------------------------------------------------------------------------*/
/*  Macro Definition                                                            */
/*------------------------------------------------------------------------------*/
/** 
 *  @defgroup FLASH_Private_define FLASH Private Define
 *  @{
 */
#define FC_KCR_KEYCODE      (0xA74A9D23UL)              /*!< The specific code that writes the FCKCR register. */
#define FC_BRANK_VALUE      (uint32_t)(0xFFFFFFFFUL)    /*!< Brank value */
#define FC_CMD_BC1_ADDR     (0x00000550UL)              /*!< The lower address of the first bus cycle when uses commans */
#define FC_CMD_BC2_ADDR     (0x00000AA0UL)              /*!< The lower address of the second bus cycle when uses commans */
#define FC_BANK_USER_INFO   (0x00000007UL)              /*!< Bank Change User Information Area */
#define FC_BANK_CODE_FLASH  (0x00000000UL)              /*!< Bank Change Code Flash */
#define FC_BUFFER_DISABLE   (0x00000007UL)              /*!< Flash Buffer Disable nad Clear */
#define FC_BUFFER_ENABLE    (0x00000000UL)              /*!< Flash Buffer Enable */

#define FC_PROTECT_CODE_ADDR (0x00002000UL)             /*!< The lower address of protect command */
#define FC_PROTECT_DATA_ADDR (0x00000100UL)             /*!< The lower address of protect command */
#define FC_SECURITY_ADDR    (0x5E001000UL)              /*!< Flash Security address */

#define FC_ACCR_FDLC_4      (0x00000300UL)              /*!< Data Flash read clock 4clock  */
#define FC_ACCR_FDLC_5      (0x00000400UL)              /*!< Data Flash read clock 5clock  */
#define FC_ACCR_FCLC_4      (0x00000003UL)              /*!< Code Flash read clock 4clock  */
#define FC_ACCR_FCLC_5      (0x00000004UL)              /*!< Code Flash read clock 5clock  */

/* FCSR0 register */
#define FC_AREASEL_EXPECT_AREA0 (uint32_t)(0x00000000UL)    /*!< RW, Selects expect area0       */
#define FC_AREASEL_EXPECT_AREA1 (uint32_t)(0x00000000UL)    /*!< RW, Selects expect area1       */
#define FC_AREASEL_EXPECT_AREA4 (uint32_t)(0x00000000UL)    /*!< RW, Selects expect area4       */
#define FC_AREASEL_AREA0        (uint32_t)(0x00000007UL)    /*!< RW, Selects area0              */
#define FC_AREASEL_AREA1        (uint32_t)(0x00000070UL)    /*!< RW, Selects area1              */
#define FC_AREASEL_AREA4        (uint32_t)(0x00070000UL)    /*!< RW, Selects area4              */
#define FC_AREASEL_MASK_AREA0   (uint32_t)(0xFF8F0FF8UL)    /*!< RW, Selects area0              */
#define FC_AREASEL_MASK_AREA1   (uint32_t)(0xFF8F0F8FUL)    /*!< RW, Selects area1              */
#define FC_AREASEL_MASK_AREA4   (uint32_t)(0xFF880FFFUL)    /*!< RW, Selects area4              */
#define FC_AREASEL0_WRITE_MODE  (uint32_t)(0x04000000UL)    /*!< R,  Write Mode                 */
#define FC_AREASEL1_WRITE_MODE  (uint32_t)(0x08000000UL)    /*!< R,  Write Mode                 */
#define FC_AREASEL4_WRITE_MODE  (uint32_t)(0x40000000UL)    /*!< R,  Write Mode                 */

#define FC_SEC_MASK             (uint32_t)(0x00000000UL)    /*!< Flash security bit mask  */
#define FC_SEC_ENABLE           (uint32_t)(0x00000001UL)    /*!< Flash security enable  */

/** 
 *  @}
 */ /* End of group FLASH_Private_define */


/*------------------------------------------------------------------------------*/
/*  Enumerated Type Definition                                                  */
/*------------------------------------------------------------------------------*/
/** 
 *  @defgroup FLASH_Private_typedef FLASH Private Typedef
 *  @{
 */

/**
 *  @}
 */ /* End of group FLASH_Private_typedef */


/*------------------------------------------------------------------------------*/
/*  Private Member                                                              */
/*------------------------------------------------------------------------------*/
/** 
 *  @defgroup FLASH_Private_variables FLASH Private Variables
 *  @{
 */
/**
 *  @}
 */ /* End of group FLASH_Private_variables */


/*------------------------------------------------------------------------------*/
/*  Private Function                                                            */
/*------------------------------------------------------------------------------*/
/** 
 *  @defgroup FLASH_Private_fuctions FLASH Private Fuctions
 *  @{
 */
#ifdef __DEBUG__
    __STATIC_INLINE int32_t check_param_flash_page_code(fc_code_flash_page_number_t param);
    __STATIC_INLINE int32_t check_param_flash_page_data(fc_data_flash_page_number_t param);
    __STATIC_INLINE int32_t check_param_flash_status(fc_sr0_t param);
    __STATIC_INLINE int32_t check_param_code_flash_area(fc_code_flash_area_number_t param);
    __STATIC_INLINE int32_t check_param_data_flash_area(fc_data_flash_area_number_t param);
    __STATIC_INLINE int32_t check_param_flash_block_data(fc_data_flash_block_number_t param);
    __STATIC_INLINE int32_t check_param_flash_block_code(fc_code_flash_block_number_t param);
    __STATIC_INLINE int32_t check_param_flash_kind(fc_erase_kind_t param);
#endif /* #ifdef __DEBUG__ */
__STATIC_INLINE TXZ_Result fc_set_code_areasel(flash_t *p_obj, fc_code_flash_area_number_t area, TXZ_FunctionalState state);
__STATIC_INLINE TXZ_Result fc_set_data_areasel(flash_t *p_obj, fc_data_flash_area_number_t area, TXZ_FunctionalState state);
static uint32_t get_data_flash_page_address(fc_data_flash_page_number_t page);
static uint32_t get_data_flash_block_address(fc_data_flash_block_number_t block);
static uint32_t get_data_flash_area_address(fc_data_flash_area_number_t area);
static uint32_t get_code_flash_page_address(fc_code_flash_page_number_t page);
static void fc_write_command(flash_t *p_obj, uint32_t* src_address, uint32_t* dst_address, uint32_t size);
static void fc_write_command4(flash_t *p_obj, uint32_t* src_address, uint32_t* dst_address, uint32_t size);
static void fc_erase_command(flash_t *p_obj, uint32_t* flash_top_address, uint32_t* erase_top_address, fc_erase_kind_t kind);
static void fc_erase_command4(flash_t *p_obj, uint32_t* flash_top_address, uint32_t* erase_top_address, fc_erase_kind_t kind);
static TXZ_Result fc_blank_check(uint32_t* address, uint32_t size);
static void fc_bank_change(flash_t *p_obj, uint32_t bank);

#ifdef __DEBUG__
__STATIC_INLINE int32_t check_param_flash_page_code(fc_code_flash_page_number_t param)
{
    int32_t result = FLASH_PARAM_NG;

    uint32_t max = FC_CODE_FLASH_PAGE_MAX;
    if (param <= max)
    {
        result = FLASH_PARAM_OK;
    }
    return (result);
}
__STATIC_INLINE int32_t check_param_flash_page_data(fc_data_flash_page_number_t param)
{
    int32_t result = FLASH_PARAM_NG;

    if (param <= FC_DATA_FLASH_PAGE_MAX)
    {
        result = FLASH_PARAM_OK;
    }
    return (result);
}
__STATIC_INLINE int32_t check_param_flash_status(fc_sr0_t param)
{
    int32_t result = FLASH_PARAM_NG;

    switch (param) 
    {
    case FC_SR0_RDYBSY:
        result = FLASH_PARAM_OK;
        break;
    default:
        /* no process */
        break;
    }
    return (result);
}
__STATIC_INLINE int32_t check_param_data_flash_area(fc_data_flash_area_number_t param)
{
    int32_t result = FLASH_PARAM_NG;

    switch (param) 
    {
    case FC_DATA_FLASH_AREA4:
        result = FLASH_PARAM_OK;
        break;
    default:
        /* no process */
        break;
    }
    return (result);
}
__STATIC_INLINE int32_t check_param_code_flash_area(fc_code_flash_area_number_t param)
{
    int32_t result = FLASH_PARAM_NG;

    switch (param) 
    {
    case FC_CODE_FLASH_AREA0:
#if (CODE_FLASH_AREA1_EQUIPPED == AVAILABLE)
    case FC_CODE_FLASH_AREA1:
#endif
        result = FLASH_PARAM_OK;
        break;
    default:
        /* no process */
        break;
    }
    return (result);
}
__STATIC_INLINE int32_t check_param_flash_block_data(fc_data_flash_block_number_t param)
{
    int32_t result = FLASH_PARAM_NG;

    if (param <= FC_DATA_FLASH_BLOCK_MAX) {
        result = FLASH_PARAM_OK;
    }
    return (result);
}
__STATIC_INLINE int32_t check_param_flash_block_code(fc_code_flash_block_number_t param)
{
    int32_t result = FLASH_PARAM_NG;

    if (param <= FC_CODE_FLASH_BLOCK_MAX) {
        result = FLASH_PARAM_OK;
    }
    return (result);
}
__STATIC_INLINE int32_t check_param_flash_kind(fc_erase_kind_t param)
{
     int32_t result = FLASH_PARAM_NG;

    switch (param) 
    {
    case FC_ERASE_KIND_PAGE:
    case FC_ERASE_KIND_BLOCK:
    case FC_ERASE_KIND_AREA:
        result = FLASH_PARAM_OK;
        break;
    default:
        /* no process */
        break;
    }
    return (result);
}
#endif /* #ifdef __DEBUG__ */

/*--------------------------------------------------*/
/** 
  * @brief  FLASH Register set.
  * @param  r0       : Input data.
  * @param  r1       : FCKCR data.
  * @param  r2       : FCKCR Register address.
  * @param  r3       : Input Register address.
  * @return none.
  * @note   It works in the inner RAM.
  */
/*--------------------------------------------------*/
#if defined ( __SES_ARM )
__attribute__((noinline))
#endif
static void fc_reg_set(volatile uint32_t r0, volatile uint32_t r1, volatile uint32_t r2, volatile uint32_t r3)
{
    __asm("str r1, [r2]");
    __asm("str r0, [r3]");
    return;
}

/*--------------------------------------------------*/
/** 
  * @brief  Enable or Disable the Code AREA
  * @param  p_obj       : FLASH object.
  * @param  area        : code flash area number.
  * @param  state       : Enable or Disable.
  * @return Result.
  * @retval TXZ_SUCCESS :Success.
  * @retval TXZ_ERROR   :Failure.
  * @note   It works in the inner RAM.
  *         This function write the FCAREASEL regiset.
  */
/*--------------------------------------------------*/
__STATIC_INLINE TXZ_Result fc_set_code_areasel(flash_t *p_obj, fc_code_flash_area_number_t area, TXZ_FunctionalState state)
{
    /*------------------------------*/
    /*  Parameter Check             */
    /*------------------------------*/
#ifdef __DEBUG__
    /* Check the FLASH_NULL of address. */
    assert_param(IS_POINTER_NOT_NULL(p_obj));
    assert_param(IS_POINTER_NOT_NULL(p_obj->p_instance));
    assert_param(check_param_code_flash_area(area));
#endif /* #ifdef __DEBUG__ */

    TXZ_Result retval = TXZ_ERROR;
    uint32_t mask = 0;
    volatile uint32_t regval       = 0;
    volatile uint32_t kcr          = FC_KCR_KEYCODE;
    volatile uint32_t kcr_address  = (uint32_t)&p_obj->p_instance->KCR;
    volatile uint32_t areasel_address = (uint32_t)&p_obj->p_instance->AREASEL;
    uint32_t kcr_tmp               = kcr;
    uint32_t kcr_addr_tmp          = kcr_address;
    uint32_t areasel_addr_tmp      = areasel_address;

#if (CODE_FLASH_AREA1_EQUIPPED == UNAVAILABLE)
    regval  = p_obj->p_instance->AREASEL & FC_AREASEL_MASK_AREA0;
    regval |= (state == TXZ_ENABLE) ? FC_AREASEL_AREA0: FC_AREASEL_EXPECT_AREA0;
#else
    if (area == FC_CODE_FLASH_AREA0) {
        regval  = p_obj->p_instance->AREASEL & FC_AREASEL_MASK_AREA0;
        regval |= (state == TXZ_ENABLE) ? FC_AREASEL_AREA0: FC_AREASEL_EXPECT_AREA0;
    } else {
        regval = p_obj->p_instance->AREASEL & FC_AREASEL_MASK_AREA1;
        regval |= (state == TXZ_ENABLE) ? FC_AREASEL_AREA1: FC_AREASEL_EXPECT_AREA1;
    }
#endif

    /* Writes the FCKCR register the KEYCODE. */
    /* Selects the area0 */
    fc_reg_set(regval, kcr_tmp, kcr_addr_tmp, areasel_addr_tmp);

#if (CODE_FLASH_AREA1_EQUIPPED == UNAVAILABLE)
    mask = FC_AREASEL0_WRITE_MODE;
#else
    if (area == FC_CODE_FLASH_AREA0) {
        mask = FC_AREASEL0_WRITE_MODE;
    } else {
        mask = FC_AREASEL1_WRITE_MODE;
    }
#endif
    while(1){
        uint32_t i = p_obj->p_instance->AREASEL;
        if (state == TXZ_ENABLE) {
            /* Confirms the SSF0 of the FCAREASEL register is set. */
            if((i & mask) == mask){
                retval = TXZ_SUCCESS;
                break;
            }
        } else {
            /* Confirms the SSF0 of the FCAREASEL register is not set. */
            if((i & mask) != mask){
                retval = TXZ_SUCCESS;
                break;
            }
        }
    }

    return(retval);
}

/*--------------------------------------------------*/
/** 
  * @brief  Enable or Disable the Data AREA
  * @param  p_obj       : FLASH object.
  * @param  area        : data flash area number.
  * @param  state       : Enable or Disable.
  * @return Result.
  * @retval TXZ_SUCCESS :Success.
  * @retval TXZ_ERROR   :Failure.
  * @note   It works in the inner RAM.
  *         This function write the FCAREASEL regiset.
  */
/*--------------------------------------------------*/
__STATIC_INLINE TXZ_Result fc_set_data_areasel(flash_t *p_obj, fc_data_flash_area_number_t area, TXZ_FunctionalState state)
{
    /*------------------------------*/
    /*  Parameter Check             */
    /*------------------------------*/
#ifdef __DEBUG__
    /* Check the FLASH_NULL of address. */
    assert_param(IS_POINTER_NOT_NULL(p_obj));
    assert_param(IS_POINTER_NOT_NULL(p_obj->p_instance));
    assert_param(check_param_data_flash_area(area));
#endif /* #ifdef __DEBUG__ */

    TXZ_Result retval = TXZ_ERROR;
    uint32_t mask = 0;
    volatile uint32_t regval       = 0;
    volatile uint32_t kcr          = FC_KCR_KEYCODE;
    volatile uint32_t kcr_address  = (uint32_t)&p_obj->p_instance->KCR;
    volatile uint32_t areasel_address = (uint32_t)&p_obj->p_instance->AREASEL;
    uint32_t kcr_tmp               = kcr;
    uint32_t kcr_addr_tmp          = kcr_address;
    uint32_t areasel_addr_tmp      = areasel_address;

    if (area == FC_DATA_FLASH_AREA4) {
        regval = p_obj->p_instance->AREASEL & FC_AREASEL_MASK_AREA4;
        regval |= (state == TXZ_ENABLE) ? FC_AREASEL_AREA4: FC_AREASEL_EXPECT_AREA4;
    }

    /* Writes the FCKCR register the KEYCODE. */
    /* Selects the area4 */
    fc_reg_set(regval, kcr_tmp, kcr_addr_tmp, areasel_addr_tmp);

    /* Confirms the FCAREASEL register the SSF4 was set. */
    if (area == FC_DATA_FLASH_AREA4) {
        mask = FC_AREASEL4_WRITE_MODE;
    }
    while(1){
        uint32_t i = p_obj->p_instance->AREASEL;
        if (state == TXZ_ENABLE) {
            /* Confirms the SSF0 of the FCAREASEL register is set. */
            if((i & mask) == mask){
                retval = TXZ_SUCCESS;
                break;
            }
        } else {
            /* Confirms the SSF0 of the FCAREASEL register is not set. */
            if((i & mask) != mask){
                retval = TXZ_SUCCESS;
                break;
            }
        }
    }

    return(retval);
}

/*--------------------------------------------------*/
/** 
  * @brief  Get data flash address by page number.
  * @param  page : page number
  * @return address
  */
/*--------------------------------------------------*/
static uint32_t get_data_flash_page_address(fc_data_flash_page_number_t page)
{
    uint32_t addr = 0x00000000;

#ifdef __DEBUG__
    /* Check the FLASH_NULL of address. */
    assert_param(check_param_flash_page_data(page));
#endif /* #ifdef __DEBUG__ */

    addr = FC_DATA_FLASH_ADDRESS_TOP + (FC_DATA_PAGE_SIZE * (uint32_t)page);

    return addr;
}

/*--------------------------------------------------*/
/** 
  * @brief  Get data flash address by block number.
  * @param  block : block number
  * @return address
  */
/*--------------------------------------------------*/
static uint32_t get_data_flash_block_address(fc_data_flash_block_number_t block)
{
    uint32_t addr = 0x00000000;

#ifdef __DEBUG__
    /* Check the FLASH_NULL of address. */
    assert_param(check_param_flash_block_data(block));
#endif /* #ifdef __DEBUG__ */

    addr = FC_DATA_FLASH_ADDRESS_TOP + (FC_DATA_BLOCK_SIZE * (uint32_t)block);

    return addr;
}

/*--------------------------------------------------*/
/** 
  * @brief  Get data flash address by area number.
  * @param  area : area number
  * @return address
  */
/*--------------------------------------------------*/
static uint32_t get_data_flash_area_address(fc_data_flash_area_number_t area)
{
    uint32_t addr = 0x00000000;

#ifdef __DEBUG__
    /* Check the FLASH_NULL of address. */
    assert_param(check_param_data_flash_area(area));
#endif /* #ifdef __DEBUG__ */

    addr = FC_DATA_FLASH_ADDRESS_TOP + (FC_DATA_AREA_SIZE * (uint32_t)area);

    return addr;
}

/*--------------------------------------------------*/
/** 
  * @brief  Get code flash address by page number.
  * @param  page : page number
  * @return address
  */
/*--------------------------------------------------*/
static uint32_t get_code_flash_page_address(fc_code_flash_page_number_t page)
{
    uint32_t addr = 0x00000000;

#ifdef __DEBUG__
    /* Check the FLASH_NULL of address. */
    assert_param(check_param_flash_page_code(page));
#endif /* #ifdef __DEBUG__ */

    addr = FC_CODE_FLASH_ADDRESS_TOP + (FC_CODE_PAGE_SIZE * (uint32_t)page);

    return addr;
}

/*--------------------------------------------------*/
/** 
  * @brief  Writes data of the FLASH ROM.
  * @param  p_obj       : FLASH object.
  * @param  src_address : write src address.
  * @param  dst_address : write dst address.
  * @param  size        : write size.
  * @return -
  * @note   It works in the inner RAM.
  */
/*--------------------------------------------------*/
static void fc_write_command(flash_t *p_obj, uint32_t* src_address, uint32_t* dst_address, uint32_t size)
{
    /*------------------------------*/
    /*  Parameter Check             */
    /*------------------------------*/
#ifdef __DEBUG__
    /* Check the FLASH_NULL of address. */
    assert_param(IS_POINTER_NOT_NULL(p_obj));
    assert_param(IS_POINTER_NOT_NULL(size));
#endif /* #ifdef __DEBUG__ */

    TXZ_Result retval;
    volatile uint32_t* addr1 = (uint32_t *) (FC_CODE_FLASH_ADDRESS_TOP + FC_CMD_BC1_ADDR);
    volatile uint32_t* addr2 = (uint32_t *) (FC_CODE_FLASH_ADDRESS_TOP + FC_CMD_BC2_ADDR);
    volatile uint32_t* addr3 = (uint32_t *) ((uint32_t)dst_address + FC_CODE_FLASH_ADDRESS_TOP);
    uint32_t* source = (uint32_t *) src_address;

#if (CODE_FLASH_AREA1_EQUIPPED == UNAVAILABLE)
    retval = fc_set_code_areasel(p_obj, FC_CODE_FLASH_AREA0, TXZ_ENABLE);
#else
    if((uint32_t)addr3 < FC_CODE_FLASH_AREA1_ADDRESS){
        /* Enables the AREA0. Write Mode. */
        retval = fc_set_code_areasel(p_obj, FC_CODE_FLASH_AREA0, TXZ_ENABLE);
    }else{
        /* Enables the AREA1. Write Mode. */
        retval = fc_set_code_areasel(p_obj, FC_CODE_FLASH_AREA1, TXZ_ENABLE);
        addr1 = (uint32_t *)((uint32_t)addr1 | FC_CODE_FLASH_AREA1_ADDRESS);
        addr2 = (uint32_t *)((uint32_t)addr2 | FC_CODE_FLASH_AREA1_ADDRESS);
    }
#endif /* #ifdef (CODE_FLASH_AREA1_EQUIPPED == UNAVAILABLE) */

    if(retval == TXZ_SUCCESS){
        uint32_t i;

        *addr1 = (0x000000AAUL); /* bus cycle 1 */
        *addr2 = (0x00000055UL); /* bus cycle 2 */
        *addr1 = (0x000000A0UL); /* bus cycle 3 */
        for(i=(0UL); i<size; i+=(0x4UL)){
            *addr3 = *source;
            source++;
        }

        /* Confirmation of the works start of ROM. */
        while(fc_get_status(p_obj) == TXZ_DONE){
        };

        /* Waits for a finish of the works in the code Flash ROM. */
        while(fc_get_status(p_obj) == TXZ_BUSY){
        };
    }

#if (CODE_FLASH_AREA1_EQUIPPED == UNAVAILABLE)
    retval = fc_set_code_areasel(p_obj, FC_CODE_FLASH_AREA0, TXZ_DISABLE);
#else
    if((uint32_t)addr3 < FC_CODE_FLASH_AREA1_ADDRESS){
        /* Disables the AREA0. Read Mode. */
        retval = fc_set_code_areasel(p_obj, FC_CODE_FLASH_AREA0, TXZ_DISABLE);
    }else{
        /* Disables the AREA1. Read Mode. */
        retval = fc_set_code_areasel(p_obj, FC_CODE_FLASH_AREA1, TXZ_DISABLE);
    }
#endif /* #ifdef (CODE_FLASH_AREA1_EQUIPPED == UNAVAILABLE) */
}

/*--------------------------------------------------*/
/** 
  * @brief  Writes data of the DATA FLASH.
  * @param  p_obj       : FLASH object.
  * @param  src_address : write src address.
  * @param  dst_address : write dst address.
  * @param  size        : write size.
  * @return -
  * @note   It works in the inner RAM.
  */
/*--------------------------------------------------*/
static void fc_write_command4(flash_t *p_obj, uint32_t* src_address, uint32_t* dst_address, uint32_t size)
{
    /*------------------------------*/
    /*  Parameter Check             */
    /*------------------------------*/
#ifdef __DEBUG__
    /* Check the FLASH_NULL of address. */
    assert_param(IS_POINTER_NOT_NULL(p_obj));
    assert_param(IS_POINTER_NOT_NULL(size));
#endif /* #ifdef __DEBUG__ */

    uint32_t retval;
    volatile uint32_t* addr1;
    volatile uint32_t* addr2;
    volatile uint32_t* addr3;
    uint32_t* source = (uint32_t *) src_address;

    addr1 = (uint32_t *) ((uint32_t)FC_DATA_FLASH_ADDRESS_TOP + FC_CMD_BC1_ADDR);
    addr2 = (uint32_t *) ((uint32_t)FC_DATA_FLASH_ADDRESS_TOP + FC_CMD_BC2_ADDR);
    addr3 = (uint32_t *) ((uint32_t)dst_address);
    /* Enables the AREA4. Write Mode. */
    retval = fc_set_data_areasel(p_obj, FC_DATA_FLASH_AREA4, TXZ_ENABLE);

    if(retval == TXZ_SUCCESS){
        uint32_t i;

        *addr1 = (0x000000AAUL); /* bus cycle 1 */
        *addr2 = (0x00000055UL); /* bus cycle 2 */
        if((uint32_t)dst_address >= 0x30000000)
        {
          *addr1 = (0x000000C0UL); /* bus cycle 3 */
        }else{
          *addr1 = (0x000000A0UL); /* bus cycle 3 */
        }
        for(i=(0UL); i<size; i+=(0x4UL)){
            *addr3 = *source;
            source++;
        }

        /* Confirmation of the works start of ROM. */
        while(fc_get_status(p_obj) == TXZ_DONE){
        };

        /* Waits for a finish of the works in the code DATA Flash. */
        while(fc_get_status(p_obj) == TXZ_BUSY){
        };
    }

    /* Disables the AREA4. Read Mode. */
    retval = fc_set_data_areasel(p_obj, FC_DATA_FLASH_AREA4, TXZ_DISABLE);
}
/*--------------------------------------------------*/
/** 
  * @brief  Verifies data of the FLASH ROM.
  * @param  src_address : verify src address.
  * @param  dst_address : verify dst address.
  * @param  size        : verify size.
  * @return Result.
  * @retval TXZ_SUCCESS :Success.
  * @retval TXZ_ERROR   :Failure.
  * @note   It works in the inner RAM.
  */
/*--------------------------------------------------*/
TXZ_Result fc_verify_check(uint32_t* src_address, uint32_t* dst_address, uint32_t size)
{
    /*------------------------------*/
    /*  Parameter Check             */
    /*------------------------------*/
#ifdef __DEBUG__
    assert_param(IS_POINTER_NOT_NULL(size));
#endif /* #ifdef __DEBUG__ */

    TXZ_Result retval = TXZ_ERROR;
    int res = memcmp(src_address, dst_address, size);

    if(res == 0){
        retval = TXZ_SUCCESS;
    }

    return(retval);
}


/*--------------------------------------------------*/
/** 
  * @brief  Auro page erase command of the flash ROM.
  * @param  p_obj             : FLASH object.
  * @param  flash_top_address : flash top address
  * @param  erase_top_address : erase top address
  * @param  kind              : Chip, Area, Block, Page, etc.
  * @return -
  * @note   This function erases specified place of the flash ROM.
  */
/*--------------------------------------------------*/
static void fc_erase_command(flash_t *p_obj, uint32_t* flash_top_address, uint32_t* erase_top_address, fc_erase_kind_t kind)
{
    /*------------------------------*/
    /*  Parameter Check             */
    /*------------------------------*/
#ifdef __DEBUG__
    /* Check the FLASH_NULL of address. */
    assert_param(IS_POINTER_NOT_NULL(p_obj));
    assert_param(check_param_flash_kind(kind));
#endif /* #ifdef __DEBUG__ */

    TXZ_Result retval;
    volatile uint32_t *addr1 = (uint32_t *) ((uint32_t)flash_top_address + FC_CMD_BC1_ADDR);
    volatile uint32_t *addr2 = (uint32_t *) ((uint32_t)flash_top_address + FC_CMD_BC2_ADDR);
    volatile uint32_t *addr3 = (uint32_t *) erase_top_address;

#if (CODE_FLASH_AREA1_EQUIPPED == UNAVAILABLE)
    /* Enables the AREA0. Write Mode. */
    retval = fc_set_code_areasel(p_obj, FC_CODE_FLASH_AREA0, TXZ_ENABLE);
#else
    if((uint32_t)addr3 < FC_CODE_FLASH_AREA1_ADDRESS){
        /* Enables the AREA0. Write Mode. */
        retval = fc_set_code_areasel(p_obj, FC_CODE_FLASH_AREA0, TXZ_ENABLE);
    }else{
        /* Enables the AREA1. Write Mode. */
        retval = fc_set_code_areasel(p_obj, FC_CODE_FLASH_AREA1, TXZ_ENABLE);
        addr1 = (uint32_t *)((uint32_t)addr1 | FC_CODE_FLASH_AREA1_ADDRESS);
        addr2 = (uint32_t *)((uint32_t)addr2 | FC_CODE_FLASH_AREA1_ADDRESS);
    }
#endif /* #ifdef (CODE_FLASH_AREA1_EQUIPPED == UNAVAILABLE) */

    if(retval == TXZ_SUCCESS){
        *addr1 = (0x000000AAUL);
        *addr2 = (0x00000055UL);
        *addr1 = (0x00000080UL);
        *addr1 = (0x000000AAUL);
        *addr2 = (0x00000055UL);
        *addr3 = kind;

        /* Confirmation of the works start of ROM. */
        while(fc_get_status(p_obj) == TXZ_DONE){
        };

        /* Waits for a finish of the works in the code Flash ROM. */
        while(fc_get_status(p_obj) == TXZ_BUSY){
        };
    }

#if (CODE_FLASH_AREA1_EQUIPPED == UNAVAILABLE)
    /* Disables the AREA0. Read Mode. */
    retval = fc_set_code_areasel(p_obj, FC_CODE_FLASH_AREA0, TXZ_DISABLE);
#else
    if((uint32_t)addr3 < FC_CODE_FLASH_AREA1_ADDRESS){
        /* Disables the AREA0. Read Mode. */
        retval = fc_set_code_areasel(p_obj, FC_CODE_FLASH_AREA0, TXZ_DISABLE);
    }else{
        /* Disables the AREA1. Read Mode. */
        retval = fc_set_code_areasel(p_obj, FC_CODE_FLASH_AREA1, TXZ_DISABLE);
    }
#endif /* #ifdef (CODE_FLASH_AREA1_EQUIPPED == UNAVAILABLE) */
}


/*--------------------------------------------------*/
/** 
  * @brief  Auto page erase command of the flash ROM.
  * @param  p_obj             : FLASH object.
  * @param  flash_top_address : flash top address
  * @param  erase_top_address : erase top address
  * @param  kind              : Chip, Area, Block, Page, etc.
  * @return -
  * @note   This function erases specified place of the flash ROM.
  */
/*--------------------------------------------------*/
static void fc_erase_command4(flash_t *p_obj, uint32_t* flash_top_address, uint32_t* erase_top_address, fc_erase_kind_t kind)
{
    /*------------------------------*/
    /*  Parameter Check             */
    /*------------------------------*/
#ifdef __DEBUG__
    /* Check the FLASH_NULL of address. */
    assert_param(IS_POINTER_NOT_NULL(p_obj));
    assert_param(check_param_flash_kind(kind));
#endif /* #ifdef __DEBUG__ */

    TXZ_Result retval;
    volatile uint32_t *addr1 = (uint32_t *) ((uint32_t)flash_top_address + FC_CMD_BC1_ADDR);
    volatile uint32_t *addr2 = (uint32_t *) ((uint32_t)flash_top_address + FC_CMD_BC2_ADDR);
    volatile uint32_t *addr3 = (uint32_t *) erase_top_address;

    /* Enables the AREA4. Write Mode. */
    retval = fc_set_data_areasel(p_obj, FC_DATA_FLASH_AREA4, TXZ_ENABLE);

    if(retval == TXZ_SUCCESS){
        *addr1 = (0x000000AAUL);
        *addr2 = (0x00000055UL);
        *addr1 = (0x00000080UL);
        *addr1 = (0x000000AAUL);
        *addr2 = (0x00000055UL);
        *addr3 = kind;

        /* Confirmation of the works start of ROM. */
        while(fc_get_status(p_obj) == TXZ_DONE){
        };

        /* Waits for a finish of the works in the code DATA Flash. */
        while(fc_get_status(p_obj) == TXZ_BUSY){
        };
    }

    /* Disables the AREA4. Read Mode. */
    retval = fc_set_data_areasel(p_obj, FC_DATA_FLASH_AREA4, TXZ_DISABLE);
}

/*--------------------------------------------------*/
/** 
  * @brief  Checks a blank of the DATA FLASH every 4bytes.
  * @param  address   : blank check address
  * @param  size      : blank check size
  * @return Result.
  * @retval TXZ_SUCCESS :Success.
  * @retval TXZ_ERROR   :Failure.
  * @note   It works in the inner RAM.
  */
/*--------------------------------------------------*/
static TXZ_Result fc_blank_check( uint32_t* address, uint32_t size)
{
    TXZ_Result retval = TXZ_SUCCESS;

    uint32_t i;

    /*------------------------------*/
    /*  Parameter Check             */
    /*------------------------------*/
#ifdef __DEBUG__
    assert_param(IS_POINTER_NOT_NULL(size));
#endif /* #ifdef __DEBUG__ */

    for(i=0; i<(size/sizeof(uint32_t)); i++){
        uint32_t* addr = &address[i];
        if(*addr != FC_BRANK_VALUE){
            retval = TXZ_ERROR;
            break;
        }
    }
    return (retval);
}


/*--------------------------------------------------*/
/** 
  * @brief  bank change.
  * @param  p_obj : FLASH object.
  * @param  bank  : FC_BANK_USER_INFO, FC_BANK_CODE_FLASH
  * @return none.
  * @note   It works in the inner RAM.
  */
/*--------------------------------------------------*/
static void fc_bank_change(flash_t *p_obj, uint32_t bank)
{
    /*------------------------------*/
    /*  Parameter Check             */
    /*------------------------------*/
#ifdef __DEBUG__
    /* Check the FLASH_NULL of address. */
    assert_param(IS_POINTER_NOT_NULL(p_obj));
    assert_param(IS_POINTER_NOT_NULL(p_obj->p_instance));
#endif /* #ifdef __DEBUG__ */

    if(bank == FC_BANK_USER_INFO){
        /* Flash Buffer disable */
        fc_read_buf_set(p_obj, TXZ_DISABLE);
        /* change user information area */
        p_obj->p_instance->BNKCR = FC_BANK_USER_INFO;
        while(p_obj->p_instance->BNKCR != FC_BANK_USER_INFO);
    }
    else if(bank == FC_BANK_CODE_FLASH){
        /* change code flash */
        p_obj->p_instance->BNKCR = FC_BANK_CODE_FLASH;
        while(p_obj->p_instance->BNKCR != FC_BANK_CODE_FLASH);
        /* Flash Buffer enable */
        fc_read_buf_set(p_obj, TXZ_ENABLE);
    }
    else {
        /* do nothing */
    }
}



/**
 *  @}
 */ /* End of group FLASH_Private_fuctions */


/*------------------------------------------------------------------------------*/
/*  Public Function                                                             */
/*------------------------------------------------------------------------------*/
/** 
 *  @defgroup FLASH_Exported_functions FLASH Exported Functions
 *  @{
 */

/*--------------------------------------------------*/
/** 
  * @brief  Get the status of the flash auto operation.
  * @param  p_obj    : FLASH object.
  * @return Result.
  * @retval TXZ_BUSY : Busy.
  * @retval TXZ_DONE : Done.
  * @note   It works in the inner RAM.
  */
/*--------------------------------------------------*/
TXZ_WorkState fc_get_status(flash_t *p_obj)
{
    TXZ_WorkState retval = TXZ_BUSY;
    uint32_t work32;

    /*------------------------------*/
    /*  Parameter Check             */
    /*------------------------------*/
#ifdef __DEBUG__
    /* Check the FLASH_NULL of address. */
    assert_param(IS_POINTER_NOT_NULL(p_obj));
    assert_param(check_param_flash_status(p_obj->init.status));
#endif /* #ifdef __DEBUG__ */

    /* Reads the FCSR0. Masks the other specfic status */
    work32 = p_obj->p_instance->SR0 & (uint32_t)p_obj->init.status;

    /* Confirms the specific status of the flash ROM */
    if(work32 == (uint32_t)p_obj->init.status){
        retval = TXZ_DONE;
    }
    return (retval);
}


/*--------------------------------------------------*/
/** 
  * @brief  Auto write command of the code flash ROM.
  * @param  p_obj       : FLASH object.
  * @return Result.
  * @retval TXZ_SUCCESS :Success.
  * @retval TXZ_ERROR   :Failure.
  * @note   It works in the inner RAM.
  *         This function writes 16bytes data to the code Flash ROM.
  */
/*--------------------------------------------------*/
TXZ_Result fc_write_code_flash(flash_t *p_obj)
{
    TXZ_Result retval = TXZ_SUCCESS;

    /*------------------------------*/
    /*  Parameter Check             */
    /*------------------------------*/
#ifdef __DEBUG__
    /* Check the FLASH_NULL of address. */
    assert_param(IS_POINTER_NOT_NULL(p_obj));
    assert_param(IS_POINTER_NOT_NULL(p_obj->init.size));
#endif /* #ifdef __DEBUG__ */

    /* Checks the code Flash ROM status */
    if(fc_get_status(p_obj) == TXZ_DONE){

        uint32_t i;
        /* Checks the code Flash ROM status */
        for(i=0;i<p_obj->init.size;i+=(uint32_t)(0x10UL)){
            /* Writes 16bytes data. */
            fc_write_command(p_obj, (uint32_t*)((uint32_t)p_obj->init.src_address+i), (uint32_t*)((uint32_t)p_obj->init.dst_address+i), (uint32_t)(0x10UL));
        }

        /* Verifies user data and data of the Flash ROM. */
        retval = fc_verify_check(p_obj->init.src_address, p_obj->init.dst_address, p_obj->init.size);
    }

    return (retval);
}


/*--------------------------------------------------*/
/** 
  * @brief  Auto write command of the data flash.
  * @param  p_obj       : FLASH object.
  * @return Result.
  * @retval TXZ_SUCCESS :Success.
  * @retval TXZ_ERROR   :Failure.
  * @note   It works in the inner RAM.
  *         This function writes 16bytes data to the DATA Flash.
  */
/*--------------------------------------------------*/
TXZ_Result fc_write_data_flash(flash_t *p_obj)
{
    TXZ_Result retval = TXZ_SUCCESS;

    /*------------------------------*/
    /*  Parameter Check             */
    /*------------------------------*/
#ifdef __DEBUG__
    /* Check the FLASH_NULL of address. */
    assert_param(IS_POINTER_NOT_NULL(p_obj));
    assert_param(IS_POINTER_NOT_NULL(p_obj->init.size));
#endif /* #ifdef __DEBUG__ */

    /* Checks the code DATA Flash status */
    if(fc_get_status(p_obj) == TXZ_DONE){

        uint32_t i;

       for(i=0;i<p_obj->init.size;i+=(uint32_t)(0x4UL)){
            /* Writes 4bytes data. */
            fc_write_command4(p_obj, (uint32_t*)((uint32_t)p_obj->init.src_address+i), (uint32_t*)((uint32_t)p_obj->init.dst_address+i), (uint32_t)(0x4UL));
       }
        /* Verifies user data and data of the Flash ROM. */
        retval = fc_verify_check(p_obj->init.src_address, p_obj->init.dst_address, p_obj->init.size);
    }
    return (retval);
}


/*--------------------------------------------------*/
/** 
  * @brief  Auro page erase command of the code flash ROM.
  * @param  p_obj       : FLASH object.
  * @return Result.
  * @retval TXZ_SUCCESS :Success.
  * @retval TXZ_ERROR   :Failure.
  * @note   It works in the inner RAM.
  *         This function erases specified page of the code Flash ROM and checks a blank.
  */
/*--------------------------------------------------*/
TXZ_Result fc_erase_page_code_flash(flash_t *p_obj)
{
    TXZ_Result retval = TXZ_SUCCESS;
    uint32_t addr;

    /*------------------------------*/
    /*  Parameter Check             */
    /*------------------------------*/
#ifdef __DEBUG__
    /* Check the FLASH_NULL of address. */
    assert_param(IS_POINTER_NOT_NULL(p_obj));
    assert_param(check_param_flash_page_code(p_obj->init.first_page_code));
#endif /* #ifdef __DEBUG__ */

    /* Checks the code Flash ROM status */
    if(fc_get_status(p_obj) == TXZ_DONE){
        /* Checks the number of maximum pages. */
        if((p_obj->init.first_page_code + p_obj->init.num_of_pages_code) <= FC_NUMBER_OF_CODE_PAGES){
            uint8_t i;
            for(i=0; i<p_obj->init.num_of_pages_code ; i++){
                /* Erases the specific page. */
                addr = get_code_flash_page_address((fc_code_flash_page_number_t)(p_obj->init.first_page_code + i));
                fc_erase_command(p_obj, 
                                (uint32_t*)FC_CODE_FLASH_ADDRESS_TOP,
                                (uint32_t*)addr,
                                FC_ERASE_KIND_PAGE);
            }
            /* Checks a blank of the specific page. */
            addr = get_code_flash_page_address(p_obj->init.first_page_code);
            if(fc_blank_check((uint32_t*)addr, FC_CODE_PAGE_SIZE*(uint32_t)p_obj->init.num_of_pages_code) == TXZ_ERROR){
                retval = TXZ_ERROR;
            }
        }
        else{
            retval = TXZ_ERROR;
        }
    }
    else {
        retval = TXZ_ERROR;
    }

    return (retval);
}


/*--------------------------------------------------*/
/** 
  * @brief  Auro page erase command of the data flash.
  * @param  p_obj       : FLASH object.
  * @return Result.
  * @retval TXZ_SUCCESS :Success.
  * @retval TXZ_ERROR   :Failure.
  * @note   It works in the inner RAM.
  *         This function erases specified page of the data Flash and checks a blank.
  */
/*--------------------------------------------------*/
TXZ_Result fc_erase_page_data_flash(flash_t *p_obj)
{
    TXZ_Result retval = TXZ_SUCCESS;
    uint32_t addr;

    /*------------------------------*/
    /*  Parameter Check             */
    /*------------------------------*/
#ifdef __DEBUG__
    /* Check the FLASH_NULL of address. */
    assert_param(IS_POINTER_NOT_NULL(p_obj));
    assert_param(check_param_flash_page_data(p_obj->init.first_page_data));
#endif /* #ifdef __DEBUG__ */

   /* Checks the data Flash status */
    if(fc_get_status(p_obj) == TXZ_DONE){
        /* Checks the number of maximum pages. */
        if((p_obj->init.first_page_data + p_obj->init.num_of_pages_data) <= FC_NUMBER_OF_DATA_PAGES){
            uint8_t i;
            for(i=0; i<p_obj->init.num_of_pages_data ; i++){
                /* Erases the specific page. */
                addr = get_data_flash_page_address((fc_data_flash_page_number_t)(p_obj->init.first_page_data + i));
                fc_erase_command4(p_obj,
                                (uint32_t*)FC_DATA_FLASH_ADDRESS_TOP,
                                (uint32_t*)addr,
                                FC_ERASE_KIND_PAGE);
            }
            /* Checks a blank of the specific page. */
            addr = get_data_flash_page_address(p_obj->init.first_page_data);
            if(fc_blank_check((uint32_t*)addr, FC_DATA_PAGE_SIZE*(uint32_t)p_obj->init.num_of_pages_data) == TXZ_ERROR){
                retval = TXZ_ERROR;
            }
        }
        else{
            retval = TXZ_ERROR;
        }
    }
    else {
        retval = TXZ_ERROR;
    }

    return (retval);
}


/*--------------------------------------------------*/
/** 
  * @brief  Checks a blank of the code FLASH ROM of specified pages.
  * @param  p_obj       : FLASH object.
  * @return Result.
  * @retval TXZ_SUCCESS :Success.
  * @retval TXZ_ERROR   :Failure.
  * @note   It works in the inner RAM.
  */
/*--------------------------------------------------*/
TXZ_Result fc_blank_check_page_code_flash(flash_t *p_obj)
{
    /*------------------------------*/
    /*  Parameter Check             */
    /*------------------------------*/
#ifdef __DEBUG__
    /* Check the FLASH_NULL of address. */
    assert_param(IS_POINTER_NOT_NULL(p_obj));
    assert_param(check_param_flash_page_code(p_obj->init.first_page_code));
    assert_param(check_param_flash_page_code(p_obj->init.last_page_code));
#endif /* #ifdef __DEBUG__ */

    TXZ_Result retval;

    uint32_t* address = (uint32_t*)get_code_flash_page_address(p_obj->init.first_page_code);
    uint32_t size = ((uint32_t)(p_obj->init.last_page_code - p_obj->init.first_page_code + 1) * (uint32_t)FC_CODE_PAGE_SIZE);

    retval = fc_blank_check(address, size);

    return (retval);
}

/*--------------------------------------------------*/
/** 
  * @brief  Checks a blank of the data FLASH of specified pages.
  * @param  p_obj       : FLASH object.
  * @return Result.
  * @retval TXZ_SUCCESS :Success.
  * @retval TXZ_ERROR   :Failure.
  * @note   It works in the inner RAM.
  */
/*--------------------------------------------------*/
TXZ_Result fc_blank_check_page_data_flash(flash_t *p_obj)
{
    /*------------------------------*/
    /*  Parameter Check             */
    /*------------------------------*/
#ifdef __DEBUG__
    /* Check the FLASH_NULL of address. */
    assert_param(IS_POINTER_NOT_NULL(p_obj));
    assert_param(check_param_flash_page_data(p_obj->init.first_page_data));
    assert_param(check_param_flash_page_data(p_obj->init.last_page_data));
#endif /* #ifdef __DEBUG__ */

    TXZ_Result retval;

    uint32_t* address = (uint32_t*)get_data_flash_page_address(p_obj->init.first_page_data);
    uint32_t size = ((uint32_t)(p_obj->init.last_page_data - p_obj->init.first_page_data + 1) * (uint32_t)FC_DATA_PAGE_SIZE);

    retval = fc_blank_check(address, size);

    return (retval);
}

/*--------------------------------------------------*/
/** 
  * @brief  Auto block erase command of the data flash.
  * @param  p_obj       : FLASH object.
  * @return Result.
  * @retval TXZ_SUCCESS :Success.
  * @retval TXZ_ERROR   :Failure.
  * @note   It works in the inner RAM.
  *         This function erases specified block of the data Flash and checks a blank.
  */
/*--------------------------------------------------*/
TXZ_Result fc_erase_block_data_flash(flash_t *p_obj)
{
    TXZ_Result retval = TXZ_SUCCESS;
    uint32_t addr;

    /*------------------------------*/
    /*  Parameter Check             */
    /*------------------------------*/
#ifdef __DEBUG__
    /* Check the FLASH_NULL of address. */
    assert_param(IS_POINTER_NOT_NULL(p_obj));
    assert_param(check_param_flash_block_data(p_obj->init.first_block));
#endif /* #ifdef __DEBUG__ */

    /* Checks the data Flash status */
    if(fc_get_status(p_obj) == TXZ_DONE){
        /* Checks the number of maximum blocks. */
        if((p_obj->init.first_block + p_obj->init.num_of_block) <= FC_NUMBER_OF_DATA_BLOCKS){
            uint8_t i;
            for(i=0; i<p_obj->init.num_of_block ; i++){
                /* Erases the specific block. */
                addr = get_data_flash_block_address((fc_data_flash_block_number_t)(p_obj->init.first_block + i));
                fc_erase_command4(p_obj,
                                (uint32_t*)FC_DATA_FLASH_ADDRESS_TOP,
                                (uint32_t*)addr,
                                FC_ERASE_KIND_BLOCK);
            }
            /* Checks a blank of the specific block. */
            addr = get_data_flash_block_address(p_obj->init.first_block);
            if(fc_blank_check((uint32_t*)addr, FC_DATA_BLOCK_SIZE*(uint32_t)p_obj->init.num_of_block) == TXZ_ERROR){
                retval = TXZ_ERROR;
            }
        }
        else{
            retval = TXZ_ERROR;
        }
    }
    else {
        retval = TXZ_ERROR;
    }

    return (retval);
}

/*--------------------------------------------------*/
/** 
  * @brief  Checks a blank of the data FLASH of specified blocks.
  * @param  p_obj       : FLASH object.
  * @return Result.
  * @retval TXZ_SUCCESS :Success.
  * @retval TXZ_ERROR   :Failure.
  * @note   It works in the inner RAM.
  */
/*--------------------------------------------------*/
TXZ_Result fc_blank_check_block_data_flash(flash_t *p_obj)
{
    /*------------------------------*/
    /*  Parameter Check             */
    /*------------------------------*/
#ifdef __DEBUG__
    /* Check the FLASH_NULL of address. */
    assert_param(IS_POINTER_NOT_NULL(p_obj));
    assert_param(check_param_flash_block_data(p_obj->init.first_block));
    assert_param(check_param_flash_block_data(p_obj->init.last_block));
#endif /* #ifdef __DEBUG__ */

    TXZ_Result retval;

    uint32_t* address = (uint32_t*)get_data_flash_block_address(p_obj->init.first_block);
    uint32_t size = ((uint32_t)(p_obj->init.last_block - p_obj->init.first_block + 1) * (uint32_t)FC_DATA_BLOCK_SIZE);

    retval = fc_blank_check(address, size);

    return (retval);
}
/*--------------------------------------------------*/
/** 
  * @brief  Auto area erase command of the data flash.
  * @param  p_obj       : FLASH object.
  * @return Result.
  * @retval TXZ_SUCCESS :Success.
  * @retval TXZ_ERROR   :Failure.
  * @note   It works in the inner RAM.
  *         This function erases specified block of the data Flash and checks a blank.
  */
/*--------------------------------------------------*/
TXZ_Result fc_erase_area_data_flash(flash_t *p_obj)
{
    TXZ_Result retval = TXZ_SUCCESS;
    uint32_t addr;

    /*------------------------------*/
    /*  Parameter Check             */
    /*------------------------------*/
#ifdef __DEBUG__
    /* Check the FLASH_NULL of address. */
    assert_param(IS_POINTER_NOT_NULL(p_obj));
    assert_param(check_param_data_flash_area(p_obj->init.area));
#endif /* #ifdef __DEBUG__ */

    /* Checks the data Flash status */
    if(fc_get_status(p_obj) == TXZ_DONE){
        /* Checks the number of maximum blocks. */
        if(p_obj->init.area == 0){
            /* Erases the specific block. */
            addr = get_data_flash_area_address(p_obj->init.area);
            fc_erase_command4(p_obj,
                            (uint32_t*)FC_DATA_FLASH_ADDRESS_TOP,
                            (uint32_t*)addr,
                            FC_ERASE_KIND_AREA);
            /* Checks a blank of the specific block. */
            if(fc_blank_check((uint32_t*)addr, FC_DATA_AREA_SIZE) == TXZ_ERROR){
                retval = TXZ_ERROR;
            }
        }
        else{
            retval = TXZ_ERROR;
        }
    }
    else {
        retval = TXZ_ERROR;
    }

    return (retval);
}
/*--------------------------------------------------*/
/** 
  * @brief  Checks a blank of the data FLASH of specified area.
  * @param  p_obj       : FLASH object.
  * @return Result.
  * @retval TXZ_SUCCESS :Success.
  * @retval TXZ_ERROR   :Failure.
  * @note   It works in the inner RAM.
  */
/*--------------------------------------------------*/
TXZ_Result fc_blank_check_area_data_flash(flash_t *p_obj)
{
    /*------------------------------*/
    /*  Parameter Check             */
    /*------------------------------*/
#ifdef __DEBUG__
    /* Check the FLASH_NULL of address. */
    assert_param(IS_POINTER_NOT_NULL(p_obj));
    assert_param(check_param_data_flash_area(p_obj->init.area));
#endif /* #ifdef __DEBUG__ */

    TXZ_Result retval;

    uint32_t* address = (uint32_t*)get_data_flash_area_address(p_obj->init.area);
    uint32_t size = ((uint32_t)(1) * (uint32_t)FC_DATA_AREA_SIZE);

    retval = fc_blank_check(address, size);

    return (retval);
}


/*--------------------------------------------------*/
/** 
  * @brief  Auto write command of the user information area.
  * @param  p_obj       : FLASH object.
  * @return Result.
  * @retval TXZ_SUCCESS :Success.
  * @retval TXZ_ERROR   :Failure.
  * @note   It works in the inner RAM.
  *         This function writes 16bytes data to the code Flash ROM.
  */
/*--------------------------------------------------*/
TXZ_Result fc_write_user_information_area(flash_t *p_obj)
{
    TXZ_Result retval = TXZ_SUCCESS;
    uint32_t user_info_addr;
    uint32_t dst_addr;
    uint32_t i;

    /*------------------------------*/
    /*  Parameter Check             */
    /*------------------------------*/
#ifdef __DEBUG__
    /* Check the FLASH_NULL of address. */
    assert_param(IS_POINTER_NOT_NULL(p_obj));
    assert_param(IS_POINTER_NOT_NULL(p_obj->init.size));
#endif /* #ifdef __DEBUG__ */

    /* Checks the code Flash ROM status */
    if(fc_get_status(p_obj) == TXZ_DONE){
        fc_bank_change(p_obj, FC_BANK_USER_INFO);

        user_info_addr = get_code_flash_page_address(FC_CODE_FLASH_PAGE5);
        /* Checks the code Flash ROM status */
        for(i=0;i<p_obj->init.size;i+=(uint32_t)(0x10UL)){
            /* Writes 16bytes data. */
            dst_addr = user_info_addr - FC_CODE_FLASH_ADDRESS_TOP + i;
            fc_write_command(p_obj, (uint32_t *)((uint32_t)p_obj->init.src_address+i), (uint32_t *)dst_addr, (uint32_t)(0x10UL));
        }

        /* Verifies user data and data of the Flash ROM. */
        retval = fc_verify_check(p_obj->init.src_address, (uint32_t *)user_info_addr, p_obj->init.size);

        fc_bank_change(p_obj, FC_BANK_CODE_FLASH);
    }
    else {
        retval = TXZ_ERROR;
    }

    return (retval);
}

/*--------------------------------------------------*/
/** 
  * @brief  Auro page erase command of the user information area.
  * @param  p_obj       : FLASH object.
  * @return Result.
  * @retval TXZ_SUCCESS :Success.
  * @retval TXZ_ERROR   :Failure.
  * @note   It works in the inner RAM.
  *         This function erases specified page of the code Flash ROM and checks a blank.
  */
/*--------------------------------------------------*/
TXZ_Result fc_erase_user_information_area(flash_t *p_obj)
{
    TXZ_Result retval = TXZ_SUCCESS;
    uint32_t user_info_addr;

    /*------------------------------*/
    /*  Parameter Check             */
    /*------------------------------*/
#ifdef __DEBUG__
    /* Check the FLASH_NULL of address. */
    assert_param(IS_POINTER_NOT_NULL(p_obj));
#endif /* #ifdef __DEBUG__ */

    /* Checks the code Flash ROM status */
    if(fc_get_status(p_obj) == TXZ_DONE){
        fc_bank_change(p_obj, FC_BANK_USER_INFO);

        /* Erases the specific page. */
        user_info_addr = get_code_flash_page_address(FC_CODE_FLASH_PAGE5);
        fc_erase_command(p_obj,
                        (uint32_t*)FC_CODE_FLASH_ADDRESS_TOP,
                        (uint32_t*)user_info_addr,
                        FC_ERASE_KIND_PAGE);

        /* Checks a blank of the specific page. */
        if(fc_blank_check((uint32_t*)user_info_addr, FC_CODE_PAGE_SIZE) == TXZ_ERROR){
            retval = TXZ_ERROR;
        }

        fc_bank_change(p_obj, FC_BANK_CODE_FLASH);
    }
    else {
        retval = TXZ_ERROR;
    }

    return (retval);
}

/*--------------------------------------------------*/
/** 
  * @brief  Raed of the user information area.
  * @param  p_obj       : FLASH object.
  * @retval none.
  * @note   It works in the inner RAM.
  */
/*--------------------------------------------------*/
void fc_read_user_information_area(flash_t *p_obj)
{
    /*------------------------------*/
    /*  Parameter Check             */
    /*------------------------------*/
#ifdef __DEBUG__
    /* Check the FLASH_NULL of address. */
    assert_param(IS_POINTER_NOT_NULL(p_obj));
#endif /* #ifdef __DEBUG__ */

    uint32_t i;
    uint32_t *dst = p_obj->init.dst_address;
    uint32_t *src = (uint32_t *)get_code_flash_page_address(FC_CODE_FLASH_PAGE5);

    fc_read_clock_set(p_obj);
    fc_bank_change(p_obj, FC_BANK_USER_INFO);

    for(i=(0UL); i<p_obj->init.size; i+=(0x4UL)){
        *dst = *src;
        dst++;
        src++;
    }

    fc_bank_change(p_obj, FC_BANK_CODE_FLASH);
}

/*--------------------------------------------------*/
/** 
  * @brief  Read clock set.
  * @param  p_obj       : FLASH object.
  * @retval none.
  * @note   It works in the inner RAM.
  */
/*--------------------------------------------------*/
void fc_read_clock_set(flash_t *p_obj)
{
    /*------------------------------*/
    /*  Parameter Check             */
    /*------------------------------*/
#ifdef __DEBUG__
    /* Check the FLASH_NULL of address. */
    assert_param(IS_POINTER_NOT_NULL(p_obj));
    assert_param(IS_POINTER_NOT_NULL(p_obj->p_instance));
#endif /* #ifdef __DEBUG__ */

    volatile uint32_t regval       = 0;
    volatile uint32_t kcr          = FC_KCR_KEYCODE;
    volatile uint32_t kcr_address  = (uint32_t)&p_obj->p_instance->KCR;
    volatile uint32_t accr_address = (uint32_t)&p_obj->p_instance->REGISTER_NAME_OF_FCACCR;
    uint32_t kcr_tmp               = kcr;
    uint32_t kcr_addr_tmp          = kcr_address;
    uint32_t accr_addr_tmp         = accr_address;

    SystemCoreClockUpdate();
    if(SystemCoreClock > SYSCORECLOCK_ACCR)
    {
        regval = (uint32_t)(FC_ACCR_FDLC_5 | FC_ACCR_FCLC_5);
        fc_reg_set(regval, kcr_tmp, kcr_addr_tmp, accr_addr_tmp);
        while(p_obj->p_instance->REGISTER_NAME_OF_FCACCR != (uint32_t)(FC_ACCR_FDLC_5 | FC_ACCR_FCLC_5))
        {
            /* no processing */
        }
    } else {
        regval = (uint32_t)(FC_ACCR_FDLC_4 | FC_ACCR_FCLC_4);
        fc_reg_set(regval, kcr_tmp, kcr_addr_tmp, accr_addr_tmp);
        while(p_obj->p_instance->REGISTER_NAME_OF_FCACCR != (uint32_t)(FC_ACCR_FDLC_4 | FC_ACCR_FCLC_4))
        {
            /* no processing */
        }
    }
}

/*--------------------------------------------------*/
/** 
  * @brief  fc fixed clock set.
  * @param  p_obj       : FLASH object.
  * @param  sysclock    : system clock (Hz)
  * @retval none.
  * @note   It works in the inner RAM.
  */
/*--------------------------------------------------*/
void fc_fixed_clock_set(flash_t *p_obj, uint32_t sysclock )
{
    /*------------------------------*/
    /*  Parameter Check             */
    /*------------------------------*/
#ifdef __DEBUG__
    /* Check the FLASH_NULL of address. */
    assert_param(IS_POINTER_NOT_NULL(p_obj));
    assert_param(IS_POINTER_NOT_NULL(p_obj->p_instance));
    assert_param(IS_POINTER_NOT_NULL(sysclock));
#endif /* #ifdef __DEBUG__ */

    volatile uint32_t regval       = 0;
    volatile uint32_t kcr          = FC_KCR_KEYCODE;
    volatile uint32_t kcr_address  = (uint32_t)&p_obj->p_instance->KCR;
    volatile uint32_t accr_address = (uint32_t)&p_obj->p_instance->REGISTER_NAME_OF_FCACCR;
    uint32_t kcr_tmp               = kcr;
    uint32_t kcr_addr_tmp          = kcr_address;
    uint32_t accr_addr_tmp         = accr_address;

    if(sysclock > SYSCORECLOCK_ACCR)
    {
        regval = (uint32_t)(FC_ACCR_FDLC_5 | FC_ACCR_FCLC_5);
        fc_reg_set(regval, kcr_tmp, kcr_addr_tmp, accr_addr_tmp);
        while(p_obj->p_instance->REGISTER_NAME_OF_FCACCR != (uint32_t)(FC_ACCR_FDLC_5 | FC_ACCR_FCLC_5))
        {
            /* no processing */
        }
    } else {
        regval = (uint32_t)(FC_ACCR_FDLC_4 | FC_ACCR_FCLC_4);
        fc_reg_set(regval, kcr_tmp, kcr_addr_tmp, accr_addr_tmp);
        while(p_obj->p_instance->REGISTER_NAME_OF_FCACCR != (uint32_t)(FC_ACCR_FDLC_4 | FC_ACCR_FCLC_4))
        {
            /* no processing */
        }
    }
}

/*--------------------------------------------------*/
/** 
  * @brief  Protect code flash page set.
  * @param  p_obj       : FLASH object.
  * @param  page        : page to protect.
  * @retval TXZ_SUCCESS :Success.
  * @retval TXZ_ERROR   :Failure.
  * @note   It works in the inner RAM.
  */
/*--------------------------------------------------*/
TXZ_Result fc_protect_page_code_flash_set(flash_t *p_obj, fc_code_flash_page_number_t page)
{
    /*------------------------------*/
    /*  Parameter Check             */
    /*------------------------------*/
#ifdef __DEBUG__
    /* Check the FLASH_NULL of address. */
    assert_param(IS_POINTER_NOT_NULL(p_obj));
    assert_param(IS_POINTER_NOT_NULL(p_obj->p_instance));
    assert_param(check_param_flash_page_code(page));
#endif /* #ifdef __DEBUG__ */

    TXZ_Result retval = TXZ_SUCCESS;
    volatile uint32_t *addr1;
    volatile uint32_t *addr2;
    volatile uint32_t *addr3;
    volatile uint32_t regval       = 0;
    volatile uint32_t kcr          = FC_KCR_KEYCODE;
    volatile uint32_t kcr_address  = (uint32_t)&p_obj->p_instance->KCR;
    volatile uint32_t pmr0_address = (uint32_t)&p_obj->p_instance->PMR0;
    uint32_t kcr_tmp               = kcr;
    uint32_t kcr_addr_tmp          = kcr_address;
    uint32_t pmr0_addr_tmp         = pmr0_address;

    /* Checks the code Flash ROM status */
    if(fc_get_status(p_obj) == TXZ_DONE){

        /* protect bit set */
        addr1 = (uint32_t *) ((uint32_t)FC_CODE_FLASH_ADDRESS_TOP + FC_CMD_BC1_ADDR);
        addr2 = (uint32_t *) ((uint32_t)FC_CODE_FLASH_ADDRESS_TOP + FC_CMD_BC2_ADDR);
        addr3 = (uint32_t *) ((uint32_t)FC_CODE_FLASH_ADDRESS_TOP + FC_PROTECT_CODE_ADDR + ((uint32_t)page << 4));
        regval = p_obj->p_instance->PMR0 | (uint32_t)(1 << (uint32_t)page);
        fc_reg_set(regval, kcr_tmp, kcr_addr_tmp,  pmr0_addr_tmp);

        fc_set_code_areasel(p_obj, FC_CODE_FLASH_AREA0, TXZ_ENABLE);
        {
            *addr1 = (0x000000AAUL);
            *addr2 = (0x00000055UL);
            *addr1 = (0x0000009AUL);
            *addr3 = (0x0000009AUL);

            /* Confirmation of the works start of ROM. */
            while(fc_get_status(p_obj) == TXZ_DONE){
            };

            /* Waits for a finish of the works in the code DATA Flash. */
            while(fc_get_status(p_obj) == TXZ_BUSY){
            };
        }
        fc_set_code_areasel(p_obj, FC_CODE_FLASH_AREA0, TXZ_DISABLE);
    }
    else {
        retval = TXZ_ERROR;
    }

    return (retval);
}

/*--------------------------------------------------*/
/** 
  * @brief  Protect code flash block set.
  * @param  p_obj       : FLASH object.
  * @param  block       : block to protect.
  * @retval TXZ_SUCCESS :Success.
  * @retval TXZ_ERROR   :Failure.
  * @note   It works in the inner RAM.
  */
/*--------------------------------------------------*/
TXZ_Result fc_protect_block_code_flash_set(flash_t *p_obj, fc_code_flash_block_number_t block)
{
    /*------------------------------*/
    /*  Parameter Check             */
    /*------------------------------*/
#ifdef __DEBUG__
    /* Check the FLASH_NULL of address. */
    assert_param(IS_POINTER_NOT_NULL(p_obj));
    assert_param(IS_POINTER_NOT_NULL(p_obj->p_instance));
    assert_param(check_param_flash_block_code(block));
#endif /* #ifdef __DEBUG__ */

    TXZ_Result retval = TXZ_SUCCESS;
    volatile uint32_t *addr1;
    volatile uint32_t *addr2;
    volatile uint32_t *addr3;
    volatile uint32_t regval       = 0;
    volatile uint32_t kcr          = FC_KCR_KEYCODE;
    volatile uint32_t kcr_address  = (uint32_t)&p_obj->p_instance->KCR;
    volatile uint32_t pmr1_address = (uint32_t)&p_obj->p_instance->PMR1;
#if (FCPMR2_REGISTER_EQUIPPED == AVAILABLE)
    volatile uint32_t pmr2_address = (uint32_t)&p_obj->p_instance->PMR2;
#endif
    uint32_t kcr_tmp               = kcr;
    uint32_t kcr_addr_tmp          = kcr_address;
    uint32_t pmr1_addr_tmp         = pmr1_address;
#if (FCPMR2_REGISTER_EQUIPPED == AVAILABLE)
    uint32_t pmr2_addr_tmp         = pmr2_address;
#endif
    uint32_t shift_count = 0;

    /* Checks the code Flash ROM status */
    if(fc_get_status(p_obj) == TXZ_DONE){

        /* protect bit set */
        addr1 = (uint32_t *) ((uint32_t)FC_CODE_FLASH_ADDRESS_TOP + FC_CMD_BC1_ADDR);
        addr2 = (uint32_t *) ((uint32_t)FC_CODE_FLASH_ADDRESS_TOP + FC_CMD_BC2_ADDR);
        addr3 = (uint32_t *) ((uint32_t)FC_CODE_FLASH_ADDRESS_TOP + FC_PROTECT_CODE_ADDR + (((uint32_t)block + 7) << 4));
        
        shift_count = (uint32_t)block % 32; /* handle 32 block/register */
#if (FCPMR2_REGISTER_EQUIPPED == UNAVAILABLE)
        regval = p_obj->p_instance->PMR1 | (uint32_t)(1 << shift_count);
        fc_reg_set(regval, kcr_tmp, kcr_addr_tmp,  pmr1_addr_tmp);

        fc_set_code_areasel(p_obj, FC_CODE_FLASH_AREA0, TXZ_ENABLE);
        {
            *addr1 = (0x000000AAUL);
            *addr2 = (0x00000055UL);
            *addr1 = (0x0000009AUL);
            *addr3 = (0x0000009AUL);

            /* Confirmation of the works start of ROM. */
            while(fc_get_status(p_obj) == TXZ_DONE){
            };

            /* Waits for a finish of the works in the code DATA Flash. */
            while(fc_get_status(p_obj) == TXZ_BUSY){
            };
        }
        fc_set_code_areasel(p_obj, FC_CODE_FLASH_AREA0, TXZ_DISABLE);
#else
        /* code flash has over 32 blocks */
        if (block <= FC_CODE_FLASH_BLOCK31)
        {
            /* block 1~31 (block0 is accessed by page number 0~7) */
            regval = p_obj->p_instance->PMR1 | (uint32_t)(1 << shift_count);
            fc_reg_set(regval, kcr_tmp, kcr_addr_tmp,  pmr1_addr_tmp);
        }
        else
        {
            /* block 32~63 */
            regval = p_obj->p_instance->PMR2 | (uint32_t)(1 << shift_count);
            fc_reg_set(regval, kcr_tmp, kcr_addr_tmp,  pmr2_addr_tmp);
        }
        fc_set_code_areasel(p_obj, FC_CODE_FLASH_AREA0, TXZ_ENABLE);

        {
            *addr1 = (0x000000AAUL);
            *addr2 = (0x00000055UL);
            *addr1 = (0x0000009AUL);
            *addr3 = (0x0000009AUL);

            /* Confirmation of the works start of ROM. */
            while(fc_get_status(p_obj) == TXZ_DONE){
            };

            /* Waits for a finish of the works in the code DATA Flash. */
            while(fc_get_status(p_obj) == TXZ_BUSY){
            };
        }

        fc_set_code_areasel(p_obj, FC_CODE_FLASH_AREA0, TXZ_DISABLE);
#endif
    }
    else {
        retval = TXZ_ERROR;
    }

    return (retval);
}

/*--------------------------------------------------*/
/** 
  * @brief  Protect data flash block set.
  * @param  p_obj       : FLASH object.
  * @param  block       : block to protect.
  * @retval TXZ_SUCCESS :Success.
  * @retval TXZ_ERROR   :Failure.
  * @note   It works in the inner RAM.
  */
/*--------------------------------------------------*/
TXZ_Result fc_protect_block_data_flash_set(flash_t *p_obj, fc_data_flash_block_number_t block)
{
    /*------------------------------*/
    /*  Parameter Check             */
    /*------------------------------*/
#ifdef __DEBUG__
    /* Check the FLASH_NULL of address. */
    assert_param(IS_POINTER_NOT_NULL(p_obj));
    assert_param(IS_POINTER_NOT_NULL(p_obj->p_instance));
    assert_param(check_param_flash_block_data(block));
#endif /* #ifdef __DEBUG__ */

    TXZ_Result retval = TXZ_SUCCESS;
    volatile uint32_t *addr1;
    volatile uint32_t *addr2;
    volatile uint32_t *addr3;
    volatile uint32_t regval       = 0;
    volatile uint32_t kcr          = FC_KCR_KEYCODE;
    volatile uint32_t kcr_address  = (uint32_t)&p_obj->p_instance->KCR;
    volatile uint32_t pmr6_address = (uint32_t)&p_obj->p_instance->PMR6;
    uint32_t kcr_tmp               = kcr;
    uint32_t kcr_addr_tmp          = kcr_address;
    uint32_t pmr6_addr_tmp         = pmr6_address;

    /* Checks the code Flash ROM status */
    if(fc_get_status(p_obj) == TXZ_DONE){
        /* protect bit set */
        addr1 = (uint32_t *) ((uint32_t)FC_DATA_FLASH_ADDRESS_TOP + FC_CMD_BC1_ADDR);
        addr2 = (uint32_t *) ((uint32_t)FC_DATA_FLASH_ADDRESS_TOP + FC_CMD_BC2_ADDR);
        addr3  = (uint32_t *) ((uint32_t)FC_DATA_FLASH_ADDRESS_TOP + FC_PROTECT_DATA_ADDR + ((uint32_t)block << 2));
        regval = p_obj->p_instance->PMR6 | (uint32_t)(1 << (uint32_t)block);
        fc_reg_set(regval, kcr_tmp, kcr_addr_tmp,  pmr6_addr_tmp);

        fc_set_data_areasel(p_obj, FC_DATA_FLASH_AREA4, TXZ_ENABLE);
        {
            *addr1 = (0x000000AAUL);
            *addr2 = (0x00000055UL);
            *addr1 = (0x0000009AUL);
            *addr3 = (0x0000009AUL);

            /* Confirmation of the works start of ROM. */
            while(fc_get_status(p_obj) == TXZ_DONE){
            };

            /* Waits for a finish of the works in the code DATA Flash. */
            while(fc_get_status(p_obj) == TXZ_BUSY){
            };
        }
        fc_set_data_areasel(p_obj, FC_DATA_FLASH_AREA4, TXZ_DISABLE);
    }
    else {
        retval = TXZ_ERROR;
    }

    return (retval);
}

/*--------------------------------------------------*/
/** 
  * @brief  Protect code flash page clear.
  * @param  p_obj       : FLASH object.
  * @param  page        : page to clear protect.
  * @retval TXZ_SUCCESS :Success.
  * @retval TXZ_ERROR   :Failure.
  * @note   It works in the inner RAM.
  */
/*--------------------------------------------------*/
TXZ_Result fc_protect_page_code_flash_clear(flash_t *p_obj, fc_code_flash_page_number_t page)
{
    /*------------------------------*/
    /*  Parameter Check             */
    /*------------------------------*/
#ifdef __DEBUG__
    /* Check the FLASH_NULL of address. */
    assert_param(IS_POINTER_NOT_NULL(p_obj));
    assert_param(IS_POINTER_NOT_NULL(p_obj->p_instance));
    assert_param(check_param_flash_page_code(page));
#endif /* #ifdef __DEBUG__ */

    TXZ_Result retval = TXZ_SUCCESS;
    volatile uint32_t *addr1;
    volatile uint32_t *addr2;
    volatile uint32_t *addr3;
    volatile uint32_t regval       = 0;
    volatile uint32_t kcr          = FC_KCR_KEYCODE;
    volatile uint32_t kcr_address  = (uint32_t)&p_obj->p_instance->KCR;
    volatile uint32_t pmr0_address = (uint32_t)&p_obj->p_instance->PMR0;
    uint32_t kcr_tmp               = kcr;
    uint32_t kcr_addr_tmp          = kcr_address;
    uint32_t pmr0_addr_tmp         = pmr0_address;
    uint32_t mask = 0;

    /* Checks the code Flash ROM status */
    if(fc_get_status(p_obj) == TXZ_DONE){

        /* protect bit set */
        addr1 = (uint32_t *) ((uint32_t)FC_CODE_FLASH_ADDRESS_TOP + FC_CMD_BC1_ADDR);
        addr2 = (uint32_t *) ((uint32_t)FC_CODE_FLASH_ADDRESS_TOP + FC_CMD_BC2_ADDR);
        addr3 = (uint32_t *) ((uint32_t)FC_CODE_FLASH_ADDRESS_TOP + FC_PROTECT_CODE_ADDR);

        mask = (uint32_t)(1 << (uint32_t)page);
        regval = p_obj->p_instance->PMR0;
        regval &= ~mask;
        fc_reg_set(regval, kcr_tmp, kcr_addr_tmp,  pmr0_addr_tmp);

        fc_set_code_areasel(p_obj, FC_CODE_FLASH_AREA0, TXZ_ENABLE);
        {
            *addr1 = (0x000000AAUL);
            *addr2 = (0x00000055UL);
            *addr1 = (0x00000080UL);
            *addr1 = (0x000000AAUL);
            *addr2 = (0x00000055UL);
            *addr3 = (0x00000060UL);

            /* Confirmation of the works start of ROM. */
            while(fc_get_status(p_obj) == TXZ_DONE){
            };

            /* Waits for a finish of the works in the code DATA Flash. */
            while(fc_get_status(p_obj) == TXZ_BUSY){
            };
        }
        fc_set_code_areasel(p_obj, FC_CODE_FLASH_AREA0, TXZ_DISABLE);
    }
    else {
        retval = TXZ_ERROR;
    }

    return (retval);
}

/*--------------------------------------------------*/
/** 
  * @brief  Protect code flash block clear.
  * @param  p_obj       : FLASH object.
  * @param  block       : block to clear protect.
  * @retval TXZ_SUCCESS :Success.
  * @retval TXZ_ERROR   :Failure.
  * @note   It works in the inner RAM.
  */
/*--------------------------------------------------*/
TXZ_Result fc_protect_block_code_flash_clear(flash_t *p_obj, fc_code_flash_block_number_t block)
{
    /*------------------------------*/
    /*  Parameter Check             */
    /*------------------------------*/
#ifdef __DEBUG__
    /* Check the FLASH_NULL of address. */
    assert_param(IS_POINTER_NOT_NULL(p_obj));
    assert_param(IS_POINTER_NOT_NULL(p_obj->p_instance));
    assert_param(check_param_flash_block_code(block));
#endif /* #ifdef __DEBUG__ */

    TXZ_Result retval = TXZ_SUCCESS;
    volatile uint32_t *addr1;
    volatile uint32_t *addr2;
    volatile uint32_t *addr3;
    volatile uint32_t regval       = 0;
    volatile uint32_t kcr          = FC_KCR_KEYCODE;
    volatile uint32_t kcr_address  = (uint32_t)&p_obj->p_instance->KCR;
    volatile uint32_t pmr1_address = (uint32_t)&p_obj->p_instance->PMR1;
#if (FCPMR2_REGISTER_EQUIPPED == AVAILABLE)
    volatile uint32_t pmr2_address = (uint32_t)&p_obj->p_instance->PMR2;
#endif
    uint32_t kcr_tmp               = kcr;
    uint32_t kcr_addr_tmp          = kcr_address;
    uint32_t pmr1_addr_tmp         = pmr1_address;
#if (FCPMR2_REGISTER_EQUIPPED == AVAILABLE)
    uint32_t pmr2_addr_tmp         = pmr2_address;
#endif
    uint32_t shift_count = 0;
    uint32_t mask = 0;

    /* Checks the code Flash ROM status */
    if(fc_get_status(p_obj) == TXZ_DONE){

        /* protect bit set */
        addr1 = (uint32_t *) ((uint32_t)FC_CODE_FLASH_ADDRESS_TOP + FC_CMD_BC1_ADDR);
        addr2 = (uint32_t *) ((uint32_t)FC_CODE_FLASH_ADDRESS_TOP + FC_CMD_BC2_ADDR);
        addr3 = (uint32_t *) ((uint32_t)FC_CODE_FLASH_ADDRESS_TOP + FC_PROTECT_CODE_ADDR);

        shift_count = (uint32_t)block % 32; /* handle 32 block/register */
#if (FCPMR2_REGISTER_EQUIPPED == UNAVAILABLE)
        mask = (uint32_t)(1 << shift_count);
        regval = p_obj->p_instance->PMR1;
        regval &= ~mask;
        fc_reg_set(regval, kcr_tmp, kcr_addr_tmp,  pmr1_addr_tmp);

        fc_set_code_areasel(p_obj, FC_CODE_FLASH_AREA0, TXZ_ENABLE);
        {
            *addr1 = (0x000000AAUL);
            *addr2 = (0x00000055UL);
            *addr1 = (0x00000080UL);
            *addr1 = (0x000000AAUL);
            *addr2 = (0x00000055UL);
            *addr3 = (0x00000060UL);

            /* Confirmation of the works start of ROM. */
            while(fc_get_status(p_obj) == TXZ_DONE){
            };

            /* Waits for a finish of the works in the code DATA Flash. */
            while(fc_get_status(p_obj) == TXZ_BUSY){
            };
        }
        fc_set_code_areasel(p_obj, FC_CODE_FLASH_AREA0, TXZ_DISABLE);
#else
        /* code flash has over 32 blocks */
        if (block <= FC_CODE_FLASH_BLOCK31)
        {
            /* block 1~31 (block0 is accessed by page number 0~7) */
            mask = (uint32_t)(1 << shift_count);
            regval = p_obj->p_instance->PMR1;
            regval &= ~mask;
            fc_reg_set(regval, kcr_tmp, kcr_addr_tmp,  pmr1_addr_tmp);
        }
        else
        {
            /* block 32~63 */
            mask = (uint32_t)(1 << shift_count);
            regval = p_obj->p_instance->PMR2;
            regval &= ~mask;
            fc_reg_set(regval, kcr_tmp, kcr_addr_tmp,  pmr2_addr_tmp);
        }
        fc_set_code_areasel(p_obj, FC_CODE_FLASH_AREA0, TXZ_ENABLE);

        {
            *addr1 = (0x000000AAUL);
            *addr2 = (0x00000055UL);
            *addr1 = (0x00000080UL);
            *addr1 = (0x000000AAUL);
            *addr2 = (0x00000055UL);
            *addr3 = (0x00000060UL);

            /* Confirmation of the works start of ROM. */
            while(fc_get_status(p_obj) == TXZ_DONE){
            };

            /* Waits for a finish of the works in the code DATA Flash. */
            while(fc_get_status(p_obj) == TXZ_BUSY){
            };
        }

        fc_set_code_areasel(p_obj, FC_CODE_FLASH_AREA0, TXZ_DISABLE);
#endif
    }
    else {
        retval = TXZ_ERROR;
    }

    return (retval);
}

/*--------------------------------------------------*/
/** 
  * @brief  Protect data flash block clear.
  * @param  p_obj       : FLASH object.
  * @param  block       : block to clear protect.
  * @retval TXZ_SUCCESS :Success.
  * @retval TXZ_ERROR   :Failure.
  * @note   It works in the inner RAM.
  */
/*--------------------------------------------------*/
TXZ_Result fc_protect_block_data_flash_clear(flash_t *p_obj, fc_data_flash_block_number_t block)
{
    /*------------------------------*/
    /*  Parameter Check             */
    /*------------------------------*/
#ifdef __DEBUG__
    /* Check the FLASH_NULL of address. */
    assert_param(IS_POINTER_NOT_NULL(p_obj));
    assert_param(IS_POINTER_NOT_NULL(p_obj->p_instance));
    assert_param(check_param_flash_block_data(block));
#endif /* #ifdef __DEBUG__ */

    TXZ_Result retval = TXZ_SUCCESS;
    volatile uint32_t *addr1;
    volatile uint32_t *addr2;
    volatile uint32_t *addr3;
    volatile uint32_t regval       = 0;
    volatile uint32_t kcr          = FC_KCR_KEYCODE;
    volatile uint32_t kcr_address  = (uint32_t)&p_obj->p_instance->KCR;
    volatile uint32_t pmr6_address = (uint32_t)&p_obj->p_instance->PMR6;
    uint32_t kcr_tmp               = kcr;
    uint32_t kcr_addr_tmp          = kcr_address;
    uint32_t pmr6_addr_tmp         = pmr6_address;
    uint32_t mask = 0;

    /* Checks the code Flash ROM status */
    if(fc_get_status(p_obj) == TXZ_DONE){

        /* protect bit set */
        addr1 = (uint32_t *) ((uint32_t)FC_DATA_FLASH_ADDRESS_TOP + FC_CMD_BC1_ADDR);
        addr2 = (uint32_t *) ((uint32_t)FC_DATA_FLASH_ADDRESS_TOP + FC_CMD_BC2_ADDR);
        addr3 = (uint32_t *) ((uint32_t)FC_DATA_FLASH_ADDRESS_TOP + FC_PROTECT_DATA_ADDR);
        mask = (uint32_t)(1 << (uint32_t)block);
        regval = p_obj->p_instance->PMR6;
        regval &= ~mask;
        fc_reg_set(regval, kcr_tmp, kcr_addr_tmp,  pmr6_addr_tmp);

        /* currently area4 only */
        fc_set_data_areasel(p_obj, FC_DATA_FLASH_AREA4, TXZ_ENABLE);
        {
            *addr1 = (0x000000AAUL);
            *addr2 = (0x00000055UL);
            *addr1 = (0x00000080UL);
            *addr1 = (0x000000AAUL);
            *addr2 = (0x00000055UL);
            *addr3 = (0x00000060UL);

            /* Confirmation of the works start of ROM. */
            while(fc_get_status(p_obj) == TXZ_DONE){
            };

            /* Waits for a finish of the works in the code DATA Flash. */
            while(fc_get_status(p_obj) == TXZ_BUSY){
            };
        }
        fc_set_data_areasel(p_obj, FC_DATA_FLASH_AREA4, TXZ_DISABLE);
    }
    else {
        retval = TXZ_ERROR;
    }

    return (retval);
}

/*--------------------------------------------------*/
/** 
  * @brief  Check code flash page protect status.
  * @param  p_obj    : FLASH object.
  * @param  page     : page to protect.
  * @retval TXZ_BUSY : Busy.
  * @retval TXZ_DONE : Done.
  * @note   It works in the inner RAM.
  */
/*--------------------------------------------------*/
TXZ_WorkState fc_protect_page_code_flash_status(flash_t *p_obj, fc_code_flash_page_number_t page)
{
    TXZ_WorkState retval = TXZ_DONE;
    uint32_t reg = 0;
    uint32_t mask = 0;

    /*------------------------------*/
    /*  Parameter Check             */
    /*------------------------------*/
#ifdef __DEBUG__
    /* Check the FLASH_NULL of address. */
    assert_param(IS_POINTER_NOT_NULL(p_obj));
    assert_param(IS_POINTER_NOT_NULL(p_obj->p_instance));
    assert_param(check_param_flash_page_code(page));
#endif /* #ifdef __DEBUG__ */

    /* protect bit check */
    reg = p_obj->p_instance->PSR0;
    mask = (uint32_t)(1 << (uint32_t)page);
    if ((reg & mask) == mask) {
        retval = TXZ_BUSY;
    }

    return (retval);
}


/*--------------------------------------------------*/
/** 
  * @brief  Check code flash block protect status.
  * @param  p_obj    : FLASH object.
  * @param  block    : block to protect.
  * @retval TXZ_BUSY : Busy.
  * @retval TXZ_DONE : Done.
  * @note   It works in the inner RAM.
  */
/*--------------------------------------------------*/
TXZ_WorkState fc_protect_block_code_flash_status(flash_t *p_obj, fc_code_flash_block_number_t block)
{
    TXZ_WorkState retval = TXZ_DONE;
    uint32_t reg = 0;
    uint32_t mask = 0;
    uint32_t shift_count = 0;

    /*------------------------------*/
    /*  Parameter Check             */
    /*------------------------------*/
#ifdef __DEBUG__
    /* Check the FLASH_NULL of address. */
    assert_param(IS_POINTER_NOT_NULL(p_obj));
    assert_param(IS_POINTER_NOT_NULL(p_obj->p_instance));
    assert_param(check_param_flash_block_code(block));
#endif /* #ifdef __DEBUG__ */

    shift_count = (uint32_t)block % 32; /* handle 32 block/register */
#if (FCPSR2_REGISTER_EQUIPPED == UNAVAILABLE)
    reg = p_obj->p_instance->PSR1;
    mask = (uint32_t)(1 << shift_count);
#else
    /* code flash has over 32 blocks */
    if (block <= FC_CODE_FLASH_BLOCK31)
    {
        /* block 1~31 (block0 is accessed by page number 0~7) */
        reg = p_obj->p_instance->PSR1;
        mask = (uint32_t)(1 << shift_count);
    }
    else
    {
        /* block 32~63 */
        reg = p_obj->p_instance->PSR2;
        mask = (uint32_t)(1 << shift_count);
    }
#endif

    /* protect bit check */
    if ((reg & mask) == mask){
        retval = TXZ_BUSY;
    }

    return (retval);
}

/*--------------------------------------------------*/
/** 
  * @brief  Check data flash block protect status.
  * @param  p_obj    : FLASH object.
  * @param  block    : block to protect.
  * @retval TXZ_BUSY : Busy.
  * @retval TXZ_DONE : Done.
  * @note   It works in the inner RAM.
  */
/*--------------------------------------------------*/
TXZ_WorkState fc_protect_block_data_flash_status(flash_t *p_obj, fc_data_flash_block_number_t block)
{
    TXZ_WorkState retval = TXZ_DONE;
    uint32_t reg = 0;
    uint32_t mask = 0;

    /*------------------------------*/
    /*  Parameter Check             */
    /*------------------------------*/
#ifdef __DEBUG__
    /* Check the FLASH_NULL of address. */
    assert_param(IS_POINTER_NOT_NULL(p_obj));
    assert_param(IS_POINTER_NOT_NULL(p_obj->p_instance));
    assert_param(check_param_flash_block_data(block));
#endif /* #ifdef __DEBUG__ */

    /* protect bit check */
    reg = p_obj->p_instance->PSR6;
    mask = (uint32_t)(1 << (uint32_t)block);
    if ((reg & mask) == mask) {
        retval = TXZ_BUSY;
    }

    return (retval);
}

/*--------------------------------------------------*/
/** 
  * @brief  Security set.
  * @param  p_obj              : FLASH object.
  * @retval TXZ_SUCCESS :Success.
  * @retval TXZ_ERROR   :Failure.
  * @note   It works in the inner RAM.
  */
/*--------------------------------------------------*/
TXZ_Result fc_security_set(flash_t *p_obj)
{
    /*------------------------------*/
    /*  Parameter Check             */
    /*------------------------------*/
#ifdef __DEBUG__
    /* Check the FLASH_NULL of address. */
    assert_param(IS_POINTER_NOT_NULL(p_obj));
    assert_param(IS_POINTER_NOT_NULL(p_obj->p_instance));
#endif /* #ifdef __DEBUG__ */

    TXZ_Result retval = TXZ_SUCCESS;
    volatile uint32_t *addr1 = (uint32_t *) ((uint32_t)FC_CODE_FLASH_ADDRESS_TOP + FC_CMD_BC1_ADDR);
    volatile uint32_t *addr2 = (uint32_t *) ((uint32_t)FC_CODE_FLASH_ADDRESS_TOP + FC_CMD_BC2_ADDR);
    volatile uint32_t *addr3 = (uint32_t *) FC_SECURITY_ADDR;
    volatile uint32_t regval       = 0;
    volatile uint32_t kcr          = FC_KCR_KEYCODE;
    volatile uint32_t kcr_address  = (uint32_t)&p_obj->p_instance->KCR;
    volatile uint32_t sbmr_address = (uint32_t)&p_obj->p_instance->SBMR;
    uint32_t kcr_tmp               = kcr;
    uint32_t kcr_addr_tmp          = kcr_address;
    uint32_t sbmr_addr_tmp         = sbmr_address;

    /* Checks the code Flash ROM status */
    if(fc_get_status(p_obj) == TXZ_DONE){

        /* security bit set */
        regval = FC_SEC_ENABLE;
        fc_reg_set(regval, kcr_tmp, kcr_addr_tmp,  sbmr_addr_tmp);
        fc_set_code_areasel(p_obj, FC_CODE_FLASH_AREA0, TXZ_ENABLE);
#if (CODE_FLASH_AREA1_EQUIPPED == AVAILABLE)
        fc_set_code_areasel(p_obj, FC_CODE_FLASH_AREA1, TXZ_ENABLE);
#endif
        fc_set_data_areasel(p_obj, FC_DATA_FLASH_AREA4, TXZ_ENABLE);

        *addr1 = (0x000000AAUL);
        *addr2 = (0x00000055UL);
        *addr1 = (0x0000009AUL);
        *addr3 = (0x0000009AUL);

        /* Confirmation of the works start of ROM. */
        while(fc_get_status(p_obj) == TXZ_DONE){
        };

        /* Waits for a finish of the works in the code DATA Flash. */
        while(fc_get_status(p_obj) == TXZ_BUSY){
        };
        fc_set_code_areasel(p_obj, FC_CODE_FLASH_AREA0, TXZ_DISABLE);
#if (CODE_FLASH_AREA1_EQUIPPED == AVAILABLE)
        fc_set_code_areasel(p_obj, FC_CODE_FLASH_AREA1, TXZ_DISABLE);
#endif
        fc_set_data_areasel(p_obj, FC_DATA_FLASH_AREA4, TXZ_DISABLE);
    }
    else {
        retval = TXZ_ERROR;
    }

    return (retval);
}

/*--------------------------------------------------*/
/** 
  * @brief  Security clear.
  * @param  p_obj              : FLASH object.
  * @retval TXZ_SUCCESS :Success.
  * @retval TXZ_ERROR   :Failure.
  * @note   It works in the inner RAM.
  */
/*--------------------------------------------------*/
TXZ_Result fc_security_clear(flash_t *p_obj)
{
    /*------------------------------*/
    /*  Parameter Check             */
    /*------------------------------*/
#ifdef __DEBUG__
    /* Check the FLASH_NULL of address. */
    assert_param(IS_POINTER_NOT_NULL(p_obj));
    assert_param(IS_POINTER_NOT_NULL(p_obj->p_instance));
#endif /* #ifdef __DEBUG__ */

    TXZ_Result retval = TXZ_SUCCESS;
    volatile uint32_t *addr1 = (uint32_t *) ((uint32_t)FC_CODE_FLASH_ADDRESS_TOP + FC_CMD_BC1_ADDR);
    volatile uint32_t *addr2 = (uint32_t *) ((uint32_t)FC_CODE_FLASH_ADDRESS_TOP + FC_CMD_BC2_ADDR);
    volatile uint32_t *addr3 = (uint32_t *) FC_SECURITY_ADDR;
    volatile uint32_t regval       = 0;
    volatile uint32_t kcr          = FC_KCR_KEYCODE;
    volatile uint32_t kcr_address  = (uint32_t)&p_obj->p_instance->KCR;
    volatile uint32_t sbmr_address = (uint32_t)&p_obj->p_instance->SBMR;
    uint32_t kcr_tmp               = kcr;
    uint32_t kcr_addr_tmp          = kcr_address;
    uint32_t sbmr_addr_tmp         = sbmr_address;

    /* Checks the code Flash ROM status */
    if(fc_get_status(p_obj) == TXZ_DONE){

        /* security bit set */
        regval = FC_SEC_MASK;
        fc_reg_set(regval, kcr_tmp, kcr_addr_tmp,  sbmr_addr_tmp);
        fc_set_code_areasel(p_obj, FC_CODE_FLASH_AREA0, TXZ_ENABLE);
#if (CODE_FLASH_AREA1_EQUIPPED == AVAILABLE)
        fc_set_code_areasel(p_obj, FC_CODE_FLASH_AREA1, TXZ_ENABLE);
#endif
        fc_set_data_areasel(p_obj, FC_DATA_FLASH_AREA4, TXZ_ENABLE);

        *addr1 = (0x000000AAUL);
        *addr2 = (0x00000055UL);
        *addr1 = (0x00000080UL);
        *addr1 = (0x000000AAUL);
        *addr2 = (0x00000055UL);
        *addr3 = (0x00000060UL);

        /* Confirmation of the works start of ROM. */
        while(fc_get_status(p_obj) == TXZ_DONE){
        };

        /* Waits for a finish of the works in the code DATA Flash. */
        while(fc_get_status(p_obj) == TXZ_BUSY){
        };
        fc_set_code_areasel(p_obj, FC_CODE_FLASH_AREA0, TXZ_DISABLE);
#if (CODE_FLASH_AREA1_EQUIPPED == AVAILABLE)
        fc_set_code_areasel(p_obj, FC_CODE_FLASH_AREA1, TXZ_DISABLE);
#endif
        fc_set_data_areasel(p_obj, FC_DATA_FLASH_AREA4, TXZ_DISABLE);
    }
    else {
        retval = TXZ_ERROR;
    }

    return (retval);
}


/*--------------------------------------------------*/
/** 
  * @brief  Check security status.
  * @param  p_obj              : FLASH object.
  * @retval TXZ_BUSY : Busy.
  * @retval TXZ_DONE : Done.
  * @note   It works in the inner RAM.
  */
/*--------------------------------------------------*/
TXZ_WorkState fc_security_status(flash_t *p_obj)
{
    TXZ_WorkState retval = TXZ_DONE;

    /*------------------------------*/
    /*  Parameter Check             */
    /*------------------------------*/
#ifdef __DEBUG__
    /* Check the FLASH_NULL of address. */
    assert_param(IS_POINTER_NOT_NULL(p_obj));
    assert_param(IS_POINTER_NOT_NULL(p_obj->p_instance));
#endif /* #ifdef __DEBUG__ */

    /* security bit check */
    if((p_obj->p_instance->SSR & FC_SEC_ENABLE) == FC_SEC_ENABLE){
        retval = TXZ_BUSY;
    }
    return (retval);
}

/*--------------------------------------------------*/
/** 
  * @brief  read buffer enable/disable.
  * @param  p_obj : FLASH object.
  * @param  state : read buffer enable/disable.
  * @return none.
  * @note   It works in the inner RAM.
  */
/*--------------------------------------------------*/
void fc_read_buf_set(flash_t *p_obj, TXZ_FunctionalState state)
{
    /*------------------------------*/
    /*  Parameter Check             */
    /*------------------------------*/
#ifdef __DEBUG__
    /* Check the FLASH_NULL of address. */
    assert_param(IS_POINTER_NOT_NULL(p_obj));
    assert_param(IS_POINTER_NOT_NULL(p_obj->p_instance));
#endif /* #ifdef __DEBUG__ */

    if(state == TXZ_ENABLE){
        /* Flash Buffer enable */
        p_obj->p_instance->BUFDISCLR = FC_BUFFER_ENABLE;
        while(p_obj->p_instance->BUFDISCLR != FC_BUFFER_ENABLE);
    }
    else {
        /* Flash Buffer disable */
        p_obj->p_instance->BUFDISCLR = FC_BUFFER_DISABLE;
        while(p_obj->p_instance->BUFDISCLR != FC_BUFFER_DISABLE);
    }
}
/**
 *  @}
 */ /* End of group FLASH_Exported_functions */

/**
 *  @}
 */ /* End of group FLASH */

/**
 *  @} 
 */ /* End of group Periph_Driver */

#ifdef __cplusplus
}
#endif /* __cplusplus */
